﻿

'
#Region "Informacion sobre la estructura"

'/**
'--------------------------------------------------
' Version .....: 15/08/2008 
'               [dd/mm/aaaa] 
' Original en .: Patron_Moneda
'--------------------------------------------------
'
'
' -------------------------------------------------
' Estructura.: Dinero
' Lenguaje ..: (.NET Framework 3.5) - Visual Basic .NET (Version 9.0) - 2008
' Autor .....: Datos y Programas - AGUILA - Joaquin
' Emilio ....: mailto:joaquin@medina.name
'
' DESCRIPCION.:
'    Representa una cantidad de dinero (por ejemplo 100,25 euros, 50.23 dolares)
'
'    Campos de la estructura:
'         Parte entera
'         Parte decimal
'         Cultura
'
' OBSERVACIONES.:
'     La cantidad de dinero se almacena de la siguiente manera: por un lado la  
'     parte entera en una variable de tipo Long, y por otro la parte decimal
'      en una variable de tipo Integer. 
'     En lugar de almacenar el nombre de la moneda, se almacena un valor de cultura, 
'     de esta forma, con un único valor almacenado, tenemos acceso a todos los 
'     datos que necesitamos para manejar una moneda, como  pueden ser, el nombre, 
'     el símbolo, el numero de decimales que maneja, el formato numérico,
'     el separador decimal, etc.
'     Las posiciones decimales que se conservan del numero son nueve(9), de esta  
'     forma me aseguro que en los cálculos con dinero no tenga problemas de  
'     redondeo por falta de decimales  
'     La propiedad Valor, devuelve un número decimal con los decimales que usa 
'     la moneda, en el caso del euro 2 decimales)
'     Las operaciones internas se realizan con todos los decimales
'     La función ToString, devuelve el valor almacenado con el formato local del  
'     dinero, es decir, si tengo almacenados euros, mostrara la cantidad con solo  
'     dos(2) decimales y con el símbolo monetario (€)
'
'     Sin la referecia [Microsoft.VisualBasic] Sin Usar ControlChars
'
'  LICENCIA DE USO.:
'    Copyright (C) 2008 Joaquín Medina Serrano. 
'    Este código es de mi propiedad sin que exista, ni ahora, ni en el futuro,  
'    ningún género de dudas. No es un código libre, esta sujeto a las  
'    siguientes limitaciones: 
'    1)	A pesar de las ¿Exhaustivas? , ¿Intensas?, y ¿cuidadosas? (ha, ha, ha...)  
'       pruebas a que he sometido este código, el principio de Murphy siempre  
'       está trabajando y si algo puede ir mal seguro que va mal. Por esa razón,  
'       Este código se proporciona *Como esta* sin ninguna garantía de ninguna clase 
'    2) La idea de escribirlo es la de ganar dinero con él.  
'       Si usando este código, ganas dinero, yo quiero mi parte.  
'       En caso contrario, puedes usarlo como creas conveniente, pero debes cumplir  
'       las siguientes condiciones:
'       a) Debes indicar claramente el autor (o sea mi nombre) 
'          y el sitio Web de donde se lo has copiado (o sea mi web) 
'       b) Debes invitar a una cerveza a su creador, (o sea a mi), 
'          si no me localizas, me conformo con que me envíes  un *Emilio*  
'          contándome para que lo has empleado.
'       c) Si haces algo interesante con el, me gustaría verlo 
' ---------------------------------------------- 
'
' BIBLIOGRAFIA.:
' Referencias msdn 
'  http://msdn.microsoft.com/es-es/library/system.globalization.numberformatinfo.aspx
'  http://msdn.microsoft.com/es-es/library/system.globalization.cultureinfo.aspx
'  http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo(en-us).aspx
'
'Patron Money - Paginas web consultadas
' http://martinfowler.com/eaaCatalog/index.html - 
'    A short summary of the patterns in 
'    Patterns of Enterprise Application Architecture (P of EAA). 
' http://martinfowler.com/eaaCatalog/money.html - Money
' https://secure.codeproject.com/KB/dotnet/ExtremeFloatingPoint1.aspx - 
'     Floating Point in .NET part 1: Concepts and Formats
' http://www.codeproject.com/KB/vb/moneyDatatype.aspx - Money DataType
' http://www.noticeablydifferent.com/CodeSamples/Money.aspx - 
'  C# 2.0 Money/Currency Class 
' http://www.codeproject.com/KB/recipes/MoneyTypeForCLR.aspx - 
'A Money type for the CLR
' http://www.xe.com/iso4217.php - ISO 4217 Currency Code List
' http://es.wikipedia.org/wiki/ISO_4217 - ISO 4217 Wikipedia
' http://research.corsaire.com/whitepapers/080715%20-breaking-the-bank-numeric-processing.pdf
'     Breaking the Bank (Vulnerabilities in Numeric Processing 
'     within Financial Applications) 
'
' MODIFICACIONES [dd/mm/aaaa] 
' - 15/08/2008  - Creación 
' -------------------------------------------------
'
'*/
'
#End Region


' -------------------------------------------------
Option Explicit On
Option Strict On
Option Infer Off
Option Compare Binary

Imports System.IO

#Region "documentacion de la clase"
''' -------------------------------------------------
'''  <summary>
'''    El tipo de valor Dinero es una estructura, y representa una 
'''    cantidad de dinero
'''  </summary>
'''  <example>
'''    <code>
'''       Dim d1 As Dinero 
'''       D1 = 25.04D
'''       Dim d2 As Dinero = 35.25D
''' 
'''       Dim sumas As Dinero = d1 + d2
'''       Dim diferencia  As Dinero = d1 - d2
''' 
'''       Si no se hace nada más se crea el dinero en la moneda de la cultura donde 
'''       se está trabajando, es decir, si estamos en España la moneda serán euros, 
'''       y si estamos en EEUU la moneda serán dólares.
'''       Pero existen constructores que permiten crear una moneda determinada, 
'''       por ejemplo podemos crear una moneda en dólares
''' 
'''       Dim d3 = New Dinero(129.56D, "en-US")
'''       O bien
'''       Dim d3 = New Dinero(129.56D,New System.Globalization.CultureInfo("en-US"))
''' 
'''       O podemos crear la moneda que usan los italianos (euros, evidentemente)
'''       Dim d3 = New Dinero(129.56D, "it-IT")
'''       O bien
'''       Dim d3 = New Dinero(129.56D, New System.Globalization.CultureInfo"it-IT"))
'''    </code>
'''  </example>
'''
'''  <remarks>
'''    <para><b>Sobre el patrón de diseño moneda</b></para>
'''    <para>Martin Fowler, la persona describio los primeros 
'''    patrones de diseño, describe en Patterns of Enterprise Application 
'''    Architecture [PEAA] 
'''    <a href="http://www.martinfowler.com/eaaCatalog/index.html" target="_top">
'''             http://www.martinfowler.com/eaaCatalog/index.html</a> 
'''    [documento que contiene una relacion de patrones empresariales],    
'''    <a href="http://martinfowler.com/eaaCatalog/money.html" target="_top">
'''    el patron Money</a>, 
'''    entre otros muchos patrones empresariales y dice (mas o menos) 
'''    lo siguiente:</para>
'''    <para>Una gran cantidad de ordenadores trabajan manipulando 
'''    dinero, y ningun ¿lenguaje? / ¿Sistema? de la actualidad contiene 
'''    una clase o un tipo que defina el dinero. Esta falta causa problemas 
'''    ente los que se encuentran los problemas de redondeo de cifras 
'''    en las operaciones de cambio de divisas. Lo bueno que tiene 
'''    la programacion orientada a objetos es que se pueden solventar 
'''    esos problemas creando una clase dinero [Money, en el original] 
'''    que resuelva esos problemas p.488. (Traduccion bastante libre)</para>
'''    <para></para>
'''    <para><b>La estructura Dinero</b></para>
'''    <para>El tipo de valor Dinero es una Estructura y representa 
'''    una cantidad de dinero comprendida entre mas 79,228,162,514,264,337,593,543,950,335 
'''    y menos 79,228,162,514,264,337,593,543,950,335. El tipo de valor 
'''    Dinero es adecuado para realizar calculos financieros que requieren 
'''    un gran numero de digitos integrales y fraccionarios significativos 
'''    sin errores de redondeo.</para>
'''    <para>La estructura dinero mantiene la informacion del dinero 
'''    en tres campos diferentes:</para>
'''    <para>La parte entera del dinero se guarda en una variable de 
'''    tipo <see cref="System.Long">Long</see> (lo que supone 19 posiciones numericas)</para>
'''    <para>La parte decimal del dinero se guarda en una variable 
'''    del tipo <see cref="System.Integer">Integer</see> con nueve (9) posiciones decimales. Si hay 
'''    mas posiciones se truncan y redondean a nueve cifras decimales</para>
'''    <para>La informacion referente al tipo de moneda: como puede 
'''    ser, el nombre (euros), el simbolo (€) el numero de decimales 
'''    con los que trabaja la moneda, el simbolo que se usa como separador 
'''    decimal (,) el separador de los millares (.) el formato de los 
'''    numeros negativos, etc. etc.  se guarda como un valor de 
'''    <see cref="System.Globalization.CultureInfo">[CultureInfo]</see>
'''    asociado a cada cantidad de dinero.</para>
'''    <para>A traves de la 
'''    <see cref="System.Globalization.CultureInfo">[CultureInfo]</see> 
'''    puedo acceder a la clase
'''    <see cref="System.Globalization.RegionInfo">[RegionInfo]</see> 
'''    y al al clase 
'''    <see cref="System.Globalization.NumberFormatInfo">[NumberFormatInfo]</see>. 
'''    Con lo que tengo acceso a toda 
'''    la informacion que ncesito sobre la moneda.</para>
'''    <para>Puede existir algun problema porque la correspondencia 
'''    entre monedas y paises no es del tipo 1 a 1, sino que hay casos 
'''    en los que varios paises tienen la misma moneda (el ejemplo 
'''    mas claro es el Euro) y al contrario, hay paises que tienen 
'''    mas de una moneda (aunque yo no lo tengo tan claro, no parece 
'''    logico, aunque manifiesto mi ignorancia sobre este punto).</para>
'''    <para></para>
'''    <para></para>
'''    <para><b>Consideraciones sobre conversiones</b></para>
'''    <para>Este tipo proporciona metodos que convierten valores Dinero 
'''    en valores de tipo 
'''    <see cref="System.Char">Char</see>,
'''    <see cref="System.SByte">SByte</see>,
'''    <see cref="System.Int16">Int16</see>, 
'''    <see cref="System.Int32">Int32</see>, 
'''    <see cref="System.Int64">Int64</see>,
'''    <see cref="System.Byte">Byte</see>,
'''    <see cref="System.UInt16">UInt16</see>, 
'''    <see cref="System.UInt32">UInt32</see>, y 
'''    <see cref="System.UInt64">UInt64</see>, 
'''    y a la inversa. Las conversiones de otros tipos 
'''    de valor en valores de tipo Dinero son conversiones de ampliacion 
'''    en las que nunca se pierde informacion ni se producen excepciones.
'''    </para>
'''    <para>Las conversiones de valores de tipo Dinero en otros tipos 
'''    de valor son conversiones de restriccion que redondean el valor 
'''    Dinero al valor entero mas proximo a cero. Si el resultado de 
'''    la conversion no puede representarse en el tipo de destino, 
'''    se produce  <see cref="System.OverflowException">OverflowException</see>.
'''    </para>
'''    <para>
'''    Las conversiones entre Single y Double son especiales:
'''    Las conversiones de valores 
'''    Dinero en valores de tipo Single o Double son conversiones de 
'''    restriccion que pueden perder precision. Las conversiones 
'''    de este tipo no producen excepciones.</para>
'''    <para>Las conversiones de Single o Double en Dinero no estan permitidas por 
'''    razones de perdida de informacino por redondeo, producen 
'''    <see cref="System.InvalidCastException">InvalidCastException</see>.</para>
''' 
'''    <para><b>Interfaces implementadas</b>
'''       <list type = "bullet">
'''          <item>
'''            <see cref="System.ICloneable">ICloneable</see>  
'''          </item>
'''          <item>
'''               <see cref="System.IEquatable(of T)">IEquatable(Of Dinero) </see>  
'''          </item>
'''          <item>
'''              <see cref="System.IComparable">IComparable </see>   
'''          </item>
'''          <item>
'''              <see cref="System.IComparable(of T)">IComparable(Of Dinero) </see>  
'''          </item>
'''          <item>
'''             <see cref="System.IFormattable">IFormattable </see>     
'''          </item>
'''          <item>
'''              <see cref="System.IConvertible">IConvertible </see>    
'''          </item>
'''          <item>
'''              <see cref="System.IDisposable">IDisposable </see>    
'''          </item>
'''       </list>
'''    </para>
''' 
'''     <para>
'''       <b> Autor : </b>
'''       <list type = "bullet">
'''          <item>
'''             Autor .....:  <a href= "mailto:mailto:joaquin@medina.name">Joaquin Medina Serrano</a>
'''          </item>
'''          <item>
'''             Correo.....: <a href= "mailto:mailto:joaquin@medina.name">mailto:joaquin@medina.name</a>
'''          </item>
'''          <item>
'''             Web .......: <a href= "http://http://joaquin.medina.name">http://joaquin.medina.name</a>
'''          </item>
'''       </list>
'''    </para>
'''    <para>
'''       <b> Historial de revisiones: </b>
'''       <list type = "bullet">
'''          <item>
'''             22/08/2008 - Creacion 
'''          </item>
'''          <item>
'''             22/08/2008 - Modificacion 
'''          </item>
'''       </list>
'''    </para>
''' 
'''    <para>
'''       <b> Copyright (C) 2008 <a href= "mailto:mailto:joaquin@medina.name">Joaquin Medina Serrano</a> </b>
'''       <list type = "bullet">
'''          <item>
'''        Este código es de mi propiedad sin que exista, ni ahora, ni en el futuro,  
'''        ningún género de dudas. No es un código libre, esta sujeto a las  
'''        siguientes limitaciones: 
'''          </item>
'''          <item>
'''           1)A pesar de las ¿Exhaustivas? , ¿Intesas?, y ¿cuidadosas? (ja, 
'''             ja, ja..) pruebas a que he  sometido este codigo, el principio 
'''             de Murphy siempre esta trabajando y dice que si algo puede ir 
'''             mal seguro que va mal. Por esa razon, este codigo se porporciona 
'''             *Como esta* sin ninguna garantia de ninguna clase 
'''          </item>
'''          <item>
'''           2) La idea de escribirlo es la de ganar dinero con él.  
'''              Si usando este código, ganas dinero, yo quiero mi parte.  
'''              En caso contrario, puedes usarlo como creas conveniente,
'''              pero debes cumplir de forma ineludible las siguientes condiciones:
'''         </item>
'''               <item>
'''                2.a) Indicar claramente el autor (o sea mi nombre)  y el sitio 
'''                     Web de donde se lo has copiado (o sea mi web) 
'''               </item>
'''               <item>
'''                 2.b) Invitar a una cerveza a su creador, (o sea a mi), si no me 
'''                     localizas, me conformo con que me envies un "*emilio*" contandome 
'''                     para que lo has empleado.
'''               </item>
'''               <item>
'''                 2.c) Si haces algo interesante me gustaria verlo 
'''               </item>
'''    </list>
'''    </para>
'''  
'''    <para>
'''       <b> Referencias Bibliográficas MSDN  </b>
'''       <list type = "bullet">
'''          <item>
'''  <a href="http://msdn.microsoft.com/es-es/library/system.globalization.numberformatinfo.aspx">
'''  http://msdn.microsoft.com/es-es/library/system.globalization.numberformatinfo.aspx</a>
'''          </item>
'''          <item>
'''  <a href="http://msdn.microsoft.com/es-es/library/system.globalization.cultureinfo.aspx">
'''  http://msdn.microsoft.com/es-es/library/system.globalization.cultureinfo.aspx</a>
'''          </item>
'''          <item>
'''  <a href="http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo(en-us).aspx">
'''  http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo(en-us).aspx</a>
'''          </item>
'''       </list>
'''    </para>
'''
'''    <para>
'''       <b> Patron Money - Paginas Web consultadas </b>
'''       <list type = "bullet">
'''          <item>
''' <a href="http://martinfowler.com/eaaCatalog/index.html">
''' http://martinfowler.com/eaaCatalog/index.html
''' A short summary of the patterns in 
''' Patterns of Enterprise Application Architecture (P of EAA). </a>
'''          </item>
'''          <item>
''' <a href="http://martinfowler.com/eaaCatalog/money.html">
'''  http://martinfowler.com/eaaCatalog/money.html - Money </a>
'''          </item>
'''          <item>
''' <a href="https://secure.codeproject.com/KB/dotnet/ExtremeFloatingPoint1.aspx">
''' https://secure.codeproject.com/KB/dotnet/ExtremeFloatingPoint1.aspx - 
''' Floating Point in .NET part 1: Concepts and Formats</a>
'''          </item>
'''          <item>
''' <a href="http://www.codeproject.com/KB/vb/moneyDatatype.aspx">
''' http://www.codeproject.com/KB/vb/moneyDatatype.aspx - Money DataType</a>
'''          </item>
'''          <item>
''' <a href="http://www.noticeablydifferent.com/CodeSamples/Money.aspx">
''' http://www.noticeablydifferent.com/CodeSamples/Money.aspx - 
'''  C# 2.0 Money/Currency Class </a>
'''          </item>
'''          <item>
''' <a href="http://www.codeproject.com/KB/recipes/MoneyTypeForCLR.aspx">
''' http://www.codeproject.com/KB/recipes/MoneyTypeForCLR.aspx - 
'''A Money type for the CLR</a>
'''          </item>
'''          <item>
''' <a href="http://www.xe.com/iso4217.php">
''' http://www.xe.com/iso4217.php - ISO 4217 Currency Code List</a>
'''          </item>
'''          <item>
''' <a href="http://es.wikipedia.org/wiki/ISO_4217">
''' http://es.wikipedia.org/wiki/ISO_4217 - ISO 4217 Wikipedia</a>
'''          </item>
'''          <item>
''' <a href="http://research.corsaire.com/whitepapers/080715%20-breaking-the-bank-numeric-processing.pdf">
''' http://research.corsaire.com/whitepapers/080715%20-breaking-the-bank-numeric-processing.pdf
''' Breaking the Bank (Vulnerabilities in Numeric Processing 
''' within Financial Applications) </a>
'''          </item>
'''       </list>
'''    </para>
''' 
'''  </remarks>
''' 
''' -------------------------------------------------
#End Region
<Serializable()> _
Public Structure Dinero
    Implements ICloneable
    Implements IEquatable(Of Dinero)
    Implements IComparable
    Implements IComparable(Of Dinero)
    Implements IFormattable
    Implements IConvertible
    Implements IDisposable


#Region "Campos de la Estructura"

    '-------------------------------------------------------------------
    ' Son las posiciones decimales que se conservan del numero
    ' se emplea en las funcines
    ' [ToDecimal]
    ' [FromDecimal]
    ''' <summary>EL NUMERO 1000000000 (1x10^9)</summary>
    Private Shared _UNO_SEGUIDO_NUEVE_CEROS As Integer = CInt(Math.Pow(10, 9))


    '-------------------------------------------------------------------
    ''' <summary>Parte entera del número</summary>
    Private _ParteEntera As Long
    '-------------------------------------------------------------------
    ''' <summary>Parte decimal del número</summary>
    Private _ParteDecimal As Integer

    '-------------------------------------------------------------------
    ' [CultureInfo]
    ' Proporciona información sobre una referencia cultural concreta 
    ' (lo que se denomina "configuración regional" en desarrollo de código
    ' no administrado). Esta información incluye los nombres de la referencia
    ' cultural, el sistema de escritura, el calendario utilizado y el formato 
    ' de las fechas y de la ordenación de cadenas.
    '------------------------------------------------------------------------------------
    ' Datos Bibliograficos [CultureInfo]
    'http://msdn.microsoft.com/es-es/library/system.globalization.numberformatinfo.aspx
    'http://msdn.microsoft.com/es-es/library/system.globalization.cultureinfo.aspx
    'http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo(en-us).aspx
    '------------------------------------------------------------------------------------
    ''' <summary>
    '''    Referencia cultural concreta de la que se obtiene
    '''    la forma de usar los valores monetarios
    ''' </summary>
    Private _CultureInfo As System.Globalization.CultureInfo


    '---------------------------------------------------------------
    ' En la implementacion de Dispose se emplea la variable
    ' Semáforo que indica si el método Dispose ha sido llamado o no 
    ' Private _DISPOSE_Ejecutado As Boolean
    ' que esta definida en el sitio de la implementacion
    '---------------------------------------------------------------


#End Region

#Region "Propiedades"

    '-------------------------------------------------------------------
    ''' <summary>
    '''     Cantidad de dinero. (Se manejan unicamente los decimales de la moneda) 
    ''' </summary>
    ''' <value>Es la cantidad de dinero que guarda esta estructura</value>
    Public Property Valor() As Decimal
        Get
            'devuelve únicamente los decimales de la moneda
            ' (Por ejemplo el Euro tiene 2 decimales)
            Return Math.Round(ToDecimal(Me.CultureInfo), Me.MonedaNumeroDEDecimales)
        End Get
        Set(ByVal value As Decimal)
            'acepta un numero decimal si ninguna restricción
            Call FromDecimal(value)
        End Set
    End Property


    '-------------------------------------------------------------------
    ''' <summary>
    '''     Cantidad de dinero con todas sus cifras decimales
    ''' </summary>
    ''' <value>Es la cantidad de dinero que guarda esta estructura</value>
    Public Property ValorSinRedondear() As Decimal
        Get
            'devuelve un numero decimal si ninguna restricción
            ' con todos sus decimales
            Return Me.ToDecimal
        End Get
        Set(ByVal value As Decimal)
            'acepta un numero decimal si ninguna restricción
            Me.Valor = value
        End Set
    End Property


    '-------------------------------------------------------------------
    ''' <summary>
    '''    Parte entera del numero
    ''' </summary>
    Public Property ValorParteEntera() As Long
        Get
            Return _ParteEntera
        End Get
        Private Set(ByVal value As Long)
            _ParteEntera = value
        End Set
    End Property

    '            

    '-------------------------------------------------------------------
    ''' <summary>
    '''    Parte decimal del numero (con todos los decimales)(No tiene signo)
    ''' </summary>
    Public Property ValorParteDecimal() As Decimal
        Get
            'la parte decimal no tiene signo, se almacena como un numero porsitivo
            Return CType((Me._ParteDecimal / _UNO_SEGUIDO_NUEVE_CEROS), Decimal)
        End Get
        Private Set(ByVal value As Decimal)
            '  Obtiene la parte decimal de un numero decimal
            ' un número System.Integer que contiene la parte decimal del numero decimal 
            ' multiplicada por la unidad seguida de 9 ceros
            '
            ' Por ejemplo 25,674 devuelve 674 000 000 
            ' Por ejemplo 25,004 devuelve 004 000 000 
            'la parte decimal no tiene signo, se almacena como un numero porsitivo
            If value = 0 Then
                _ParteDecimal = 0
            Else
                _ParteDecimal = ObtenerParteDecimalDelNumero(value)
            End If
        End Set
    End Property


        ''-------------------------------------------------------------------
        '''' <summary>
        ''''    Parte decimal del numero (con todos los decimales)(No tiene signo)
        '''' </summary>
        'Public Property ValorParteDecimal() As Integer
        '    Get
        '        'la parte decimal no tiene signo
        '        Return Math.Abs(_ParteDecimal)
        '    End Get
        '    Private Set(ByVal value As Integer)
        '        _ParteDecimal = Math.Abs(value)
        '        Exit Property
        '    End Set
        'End Property


        '-------------------------------------------------------------------
        ''' <summary>
        '''  Obtiene / Establece el CultureInfo que representa la referencia
        '''  cultural utilizada por el subproceso actual. 
        ''' </summary>
        ''' <value>
        ''' CultureInfo que representa la referencia cultural 
        ''' utilizada por el subproceso actual.
        ''' </value>
    Public Property CultureInfo() As System.Globalization.CultureInfo
        Get
            Return _CultureInfo
        End Get
        ' A esta parte solo se puede acceder desde los constructores
        Private Set(ByVal value As System.Globalization.CultureInfo)
            _CultureInfo = value
        End Set
    End Property


    '-------------------------------------------------------------------
    ''' <summary>
    ''' Obtiene / Establece el nombre de la referencia cultural en el formato 
    ''' [códigoidioma2]-[códigopaís/región2] [es-ES].
    ''' </summary>
    ''' <returns>
    '''    Nombre de la referencia cultural en el formato 
    '''    [códigoidioma2]-[códigopaís/región2] donde 
    '''    [códigoidioma2] es un código de dos letras en minúsculas 
    '''    proveniente de ISO 639-1 y 
    '''    [códigopaís/región2] es un código de dos letras en mayúsculas
    '''    proveniente de ISO 3166. 
    '''    Por Ejemplo para España [es-ES]
    ''' </returns>
    ''' <remarks>Por Ejemplo para España [es-ES]</remarks>
    Public Property CultureName() As String
        Get
            Return CultureInfo.Name
        End Get
        ' A esta parte solo se puede acceder desde los constructores
        Private Set(ByVal value As String)
            value = ControlExistenciaCultura(value)
            Me.CultureInfo = New System.Globalization.CultureInfo(value)
        End Set
    End Property


    '-------------------------------------------------------------------
    ''' <summary>
    '''  La cantidad de dinero, ¿es un número positivo ?
    ''' </summary>
    ''' <returns>
    '''   <para> Un valor logico que indica</para>
    '''   <para> True,  SI, es un numero positivo [o tiene valor cero(0)]</para>
    '''   <para> False, es un numero negativo</para>
    '''   <para> El valor cero se devuelve como positivo</para>
    ''' </returns>
    Public Function IsPositivo() As Boolean
        Dim resultado As Boolean = False
        If Me.ValorParteEntera >= 0 Then resultado = True
        Return resultado
    End Function

    '-------------------------------------------------------------------
    ''' <summary>
    '''  La cantidad de dinero, ¿es un número negativo ?
    ''' </summary>
    ''' <returns>
    '''   <para> Un valor logico que indica</para>
    '''   <para> True,  SI, es un numero negativo </para>
    '''   <para> False, es un numero positivo [o tiene valor cero(0)]</para>
    '''   <para> El valor cero se considera como positivo</para>
    ''' </returns>
    Public Function IsNegativo() As Boolean
        Return Not (Me.IsPositivo)
    End Function


#End Region

#Region "Informacion sobre la cultura y moneda "

#Region "Ejemplo de valores"

    '-----------------------------------------------------
    '  ---------------------- CultureInfo ---------------
    '-----------------------------------------------------
    '  Cultura Definida:              en-US
    '-----------------------------------------------------
    '  Calendar:                       System.Globalization.GregorianCalendar
    '  CompareInfo:                    CompareInfo - 1033
    '  CultureTypes:                   SpecificCultures, InstalledWin32Cultures, FrameworkCultures
    '  CurrentCulture:                 es-ES
    '  CurrentUICulture:               es-ES
    '  DisplayName:                    Inglés (Estados Unidos)
    '  EnglishName:                    English (United States)
    '  IetfLanguageTag:                en-US
    '  InstalledUICulture:             es-ES
    '  IsReadOnly:                     False
    '  KeyboardLayoutId:               1033
    '  LCID:                           1033
    '  Name:                           en-US
    '  NativeName:                     English (United States)
    '  OptionalCalendars:              System.Globalization.GregorianCalendar
    '  Parent:                         en
    '  TextInfo:                       TextInfo - 1033
    '  ThreeLetterISOLanguageName:     eng
    '  ThreeLetterWindowsLanguageName: ENU
    '  TwoLetterISOLanguageName:       en
    '  UseUserOverride:                True


    '-----------------------------------------------------
    '  ---------------------- RegionInfo -----------
    '-----------------------------------------------------
    '  Cultura Definida:          EN-US
    '-----------------------------------------------------
    '  CurrencyEnglishName:          US Dollar
    '  CurrencyNativeName:           US Dollar
    '  CurrencySymbol:               $
    '  CurrentRegion:                ES
    '  DisplayName:                  Estados Unidos
    '  EnglishName:                  United States
    '  GeoId:                        244
    '  IsMetric:                     False
    '  ISOCurrencySymbol:            USD
    '  Name:                         EN-US
    '  NativeName:                   United States
    '  ThreeLetterISORegionName:     USA
    '  ThreeLetterWindowsRegionName: USA
    '  TwoLetterISORegionName:       US


    '-----------------------------------------------------
    '  ---------------------- NumberFormatInfo ------
    '-----------------------------------------------------
    '  Cultura Definida:          en-US
    '-----------------------------------------------------
    '  CurrencyDecimalDigits:     2
    '  CurrencyDecimalSeparator:  .
    '  CurrencyGroupSeparator:    ,
    '  CurrencyGroupSizes:        System.Int32[]
    '  CurrencyNegativePattern:   0
    '  CurrencyPositivePattern:   0
    '  CurrencySymbol:            $
    '  DigitSubstitution:         None
    '  IsReadOnly:                False
    '  NaNSymbol:                 NaN
    '  NativeDigits:              0
    '  NegativeInfinitySymbol:    -Infinity
    '  NegativeSign:              -
    '  NumberDecimalDigits:       2
    '  NumberDecimalSeparator:    .
    '  NumberGroupSeparator:      ,
    '  NumberGroupSizes:          System.Int32[]
    '  NumberNegativePattern:     1
    '  PercentDecimalDigits:      2
    '  PercentDecimalSeparator:   .
    '  PercentGroupSeparator:     ,
    '  PercentGroupSizes:         System.Int32[]
    '  PercentNegativePattern:    0
    '  PercentPositivePattern:    0
    '  PercentSymbol:             %
    '  PerMilleSymbol:            ‰
    '  PositiveInfinitySymbol:    Infinity
    '  PositiveSign:              +

#End Region



    '-------------------------------------------------------------------
    ''' <summary>
    '''    Obtiene el nombre completo del país o región en inglés.
    '''    Por ejemplo, un objeto RegionInfo creado para la referencia cultural 
    '''    Inglés (United States) muestra "United States".
    ''' </summary>
    ''' <remarks>http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo_members.aspx</remarks>
    Public Function PaisEnglishName() As String
        Return New Globalization.RegionInfo(Me.CultureInfo.Name).EnglishName
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    '''    Obtiene el nombre completo del país o región.
    ''' </summary>
    ''' <remarks>
    '''  Se recomienda que las aplicaciones usen el nombre completo de la 
    '''  referencia cultural, por ejemplo, "en-US", al tener acceso a la 
    '''  propiedad NativeName. La cadena utilizada para esta propiedad 
    '''  depende del idioma asociado al país o región. 
    '''  Por ejemplo, las referencias culturales "en-US" y "es-US" de 
    '''  Windows Vista pueden recuperar distintos valores. Por tanto, 
    '''  crear el objeto RegionInfo sólo con un nombre de país o región, 
    '''  "US" en este caso, no es lo bastante concreto como para distinguir 
    '''  la cadena adecuada.
    ''' http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo_members.aspx
    ''' </remarks>
    Public Function PaisNativeName() As String
        Return New Globalization.RegionInfo(Me.CultureInfo.Name).NativeName
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    '''  Obtiene el nombre o el código de país o región de dos letras ISO 3166
    '''  para el objeto RegionInfo actual.
    ''' </summary>
    ''' <remarks>
    ''' http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo_members.aspx
    ''' </remarks>
    Public Function PaisTwoLetterIsoRegionName() As String
        Return New Globalization.RegionInfo(Me.CultureInfo.Name).TwoLetterISORegionName
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    '''     Código de tres letras definido en ISO 3166 para el país o región.
    ''' </summary>
    ''' <returns>
    ''' <para>La propiedad ThreeLetterISORegionName contiene uno de los códigos de 
    '''       tres letras definidos en ISO 3166 para el país o la región. 
    '''       Por ejemplo, el código de tres letras de Estados Unidos es "USA".
    ''' </para>
    ''' <para>La distinción entre mayúsculas y minúsculas no es relevante. 
    '''       Sin embargo, las propiedades Name, TwoLetterISORegionName y 
    '''       ThreeLetterISORegionName devuelven el código correspondiente
    '''       en mayúsculas.
    ''' </para>
    '''</returns>
    ''' <remarks>
    ''' http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo_members.aspx
    ''' </remarks>
    Public Function PaisThreeLetterIsoRegionName() As String
        Return New Globalization.RegionInfo(Me.CultureInfo.Name).ThreeLetterISORegionName
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    '''      Obtiene el símbolo de moneda ISO 4217 de tres caracteres
    '''      asociado al país o región.
    ''' </summary>
    ''' <returns>Por ejemplo para España el valor es [ESP] para Egipto [EGY]</returns>
    ''' <remarks>
    ''' <para><see cref="System.Globalization.RegionInfo">
    ''' http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo(en-us).aspx
    ''' </see></para>
    ''' <para>A diferencia de CultureInfo, RegionInfo no representa las 
    '''       preferencias del usuario y no depende del idioma ni de la 
    '''       referencia cultural del usuario.</para>
    ''' <para>El nombre de RegionInfo es uno de los códigos de dos 
    '''       letras definidos en ISO 3166 para el país o la región. </para>
    ''' <para>La distinción entre mayúsculas y minúsculas no es relevante.</para> 
    ''' <para>Las propiedades Name, TwoLetterISORegionName y ThreeLetterISORegionName
    ''' recuperan los códigos adecuados en mayúsculas.</para>
    ''' <para>A continuación, se incluye un ejemplo de la lista con los nombres de 
    ''' RegionInfo predefinidos permitidos y utilizados por ésta y otras clases en
    ''' el espacio de nombres System.Globalization.
    ''' <see cref="Globalization.RegionInfo.ISOCurrencySymbol">ISOCurrencySymbol</see>
    ''' </para>
    '''<para>  ---- Códigos ISO 3166 ---      País o región </para>
    '''<para>   2 letras       3 letras</para>
    '''<para>      EG             EGY           Egipto </para>
    '''<para>      ES             ESP           España </para>
    '''<para>      FI             FIN           Finlandia </para>
    ''' <para>
    ''' http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo_members.aspx
    ''' </para>
    ''' </remarks>
    Public Function MonedaIsoCurrencySymbol() As System.String
        Return New Globalization.RegionInfo(Me.CultureInfo.Name).ISOCurrencySymbol
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    ''' Obtiene el nombre, en inglés, de la moneda utilizada en el país o región.
    ''' </summary>
    ''' <remarks>
    ''' http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo.currencyenglishname.aspx
    ''' </remarks>
    Public Function MonedaEnglishName() As String
        Return New Globalization.RegionInfo(Me.CultureInfo.Name).CurrencyEnglishName
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    ''' Obtiene el nombre de la moneda utilizada en el país o región, con el 
    ''' formato del idioma nativo del país o región. 
    ''' </summary>
    ''' <remarks>
    ''' http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo.currencynativename.aspx
    ''' </remarks>
    Public Function MonedaNativeName() As String
        Return New Globalization.RegionInfo(Me.CultureInfo.Name).CurrencyNativeName
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    '''  Obtiene el símbolo de moneda asociado al país o región. 
    '''  La cadena que se va a utilizar como símbolo de moneda. 
    '''  Por ejemplo, el símbolo de moneda de Estados Unidos es "$".
    ''' </summary>
    ''' <returns>La cadena que se va a usar como símbolo de moneda.
    '''        El valor predeterminado de InvariantInfo es "$". </returns>
    ''' <remarks>
    ''' http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo_members.aspx
    ''' </remarks>
    Public Function MonedaSimbolo() As String
        Return New Globalization.RegionInfo(Me.CultureInfo.Name).CurrencySymbol
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    '''   Indica el número de posiciones decimales que se van a 
    '''   utilizar en valores de moneda.
    ''' </summary>
    ''' <returns>En número de posiciones decimales que se van a 
    '''          utilizar en valores de moneda.
    '''          El valor predeterminado de InvariantInfo es 2.</returns>
    ''' <remarks>
    ''' http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo_members.aspx
    ''' </remarks>
    Public Function MonedaNumeroDEDecimales() As Integer
        Return Me.CultureInfo.NumberFormat.CurrencyDecimalDigits
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    '''  Obtiene la cadena que se va a utilizar como 
    '''  separador decimal en valores de moneda. 
    ''' </summary>
    ''' <remarks>
    ''' http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo_members.aspx
    ''' </remarks>
    Public Function MonedaSeparadorDecimal() As String
        Return Me.CultureInfo.NumberFormat.CurrencyDecimalSeparator
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    ''' Obtiene la cadena que separa grupos de dígitos a la izquierda 
    ''' de la coma decimal en valores de moneda. 
    ''' </summary>
    ''' <remarks>
    ''' http://msdn.microsoft.com/es-es/library/system.globalization.regioninfo_members.aspx
    ''' </remarks>
    Public Function MonedaSeparadorGrupos() As String
        Return Me.CultureInfo.NumberFormat.CurrencyGroupSeparator
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    ''' Obtiene la cadena que denota que el número asociado es negativo.
    ''' </summary>
    ''' <returns>Cadena que denota que el número asociado es negativo. 
    '''        El valor predeterminado de InvariantInfo es "-". 
    ''' </returns>
    Public Function MonedaNegativeSign() As String
        Return Me.CultureInfo.NumberFormat.NegativeSign
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    '''   Obtiene la cadena que denota que el número asociado es positivo.
    ''' </summary>
    ''' <returns>Cadena que denota que el número asociado es positivo. 
    '''        El valor predeterminado de InvariantInfo es "+". 
    ''' </returns>
    Public Function MonedaPositiveSign() As String
        Return Me.CultureInfo.NumberFormat.PositiveSign
    End Function

#End Region

#Region "Control de errores"

    '--------------------------
    ''' <summary>
    ''' Comprueba que la cantidad de dinero este dentro del rango de valores permitidos
    ''' </summary>
    ''' <param name="value">Un valor de tipo <see cref="System.Decimal">decimal</see> </param>
    ''' <remarks>
    '''     El rango de valores es 
    '''    <see cref="System.Decimal.MaxValue">MaxValue</see> y 
    '''    <see cref="System.Decimal.MinValue">MinValue</see>
    ''' </remarks>
    ''' <exception cref="System.ArgumentOutOfRangeException"> 
    '''     System.ArgumentOutOfRangeException - 
    '''     Excepcion que se dispara cuando la cantidad de dinero 
    '''     supera el rango de valores monetarios permitidos 
    '''     Long.MinValue y Long.MaxValue
    ''' </exception>
    Private Shared Sub ComprobacionValorDecimal(ByVal value As Decimal)
        If (value < Long.MinValue OrElse value > Long.MaxValue) Then
            Using SW As New System.IO.StringWriter( _
                            System.Globalization.CultureInfo.CurrentCulture)
                SW.WriteLine("Estructura Dinero")
                SW.WriteLine("La cantidad de dinero esta fuera del rango permitido")
                SW.WriteLine("[" & value & "] Valor recibido ")
                SW.WriteLine("El valor de la cantidad de dinero debe estar entre " & _
                      Int64.MinValue.ToString(Globalization.CultureInfo.CurrentCulture) & _
                      " y " & _
                      Int64.MaxValue.ToString(Globalization.CultureInfo.CurrentCulture))
                SW.Flush()
                Throw New System.ArgumentOutOfRangeException(SW.ToString)
            End Using
        End If
    End Sub


    '-------------------------------------------------------------------
    ''' <summary>
    '''    Comprueba que las dos cantidades de dinero estén en la misma moneda
    ''' </summary>
    ''' <param name="d1">Una estructura Dinero</param>
    ''' <param name="d2">Una estructura Dinero</param>
    ''' <remarks>Si no es la mimsa moneda se prodcue una excepcion</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    InvalidOperationException - Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    Private Shared Sub ComprobacionMismaMoneda( _
                           ByVal d1 As Dinero, _
                           ByVal d2 As Dinero)
        If d1.MonedaIsoCurrencySymbol <> d2.MonedaIsoCurrencySymbol Then
            Using SW As New System.IO.StringWriter( _
                            System.Globalization.CultureInfo.CurrentCulture)
                SW.WriteLine("Estructura Dinero")
                SW.WriteLine("Las monedas son diferentes, no coinciden)")

                SW.Write("Las monedas son: [")
                SW.Write("{" & d1.MonedaSimbolo & "} " & d1.MonedaNativeName)
                SW.Write("] y [")
                SW.Write("{" & d2.MonedaSimbolo & "} " & d2.MonedaNativeName)
                SW.WriteLine(" ]")

                SW.Write("Hay que convertirlas al mismo tipo de moneda  ")
                SW.WriteLine("antes de poder realizar calculos con ellas")

                SW.WriteLine("Money values are in different currencies.")
                SW.Write("Convert to the same currency ")
                SW.WriteLine("before performing operations on the values.")
                SW.Flush()
                Throw New InvalidOperationException(SW.ToString)
            End Using
        End If
    End Sub


    '-------------------------------------------------------------------
    ''' <summary>
    ''' Comprueba que la cadena recibida en el parametro es un cultura valida
    ''' </summary>
    ''' <param name="value">
    ''' Cadena con la cultura (5 caraceres) [p.e. es-ES, en-GB, en-US]</param>
    ''' <returns>
    ''' La cadena con la cultura corregida 
    ''' (mayusculas y/minusculas en la posición adecuada)
    ''' Si la cadena no es correcta se dispara  DineroCulturaIncorrectaException
    ''' </returns>
    ''' <exception cref="ArgumentNullException">
    '''      Exception que se dispara cuando la cadena del parametro
    '''      <paramref name="value">value</paramref> 
    '''      tiene un valor Null (nothing) o vacio (Empty)
    ''' </exception>
    ''' <exception cref="ArgumentException">
    '''      Exception que se dispara cuando la caadena del parametro
    '''      <paramref name="value">value</paramref> 
    '''      no contiene una refencia cultural correcta
    ''' </exception>
    Private Shared Function ControlExistenciaCultura(ByVal value As String) As String

        If String.IsNullOrEmpty(value) = True Then
            Throw New ArgumentNullException( _
                      ControlExistenciaCulturaMontarMensajeError( _
                        String.Empty, _
                        "La cadena que representa a una cultura no " & _
                        "puede ser una cadena nula o vacia)"))
        End If

        If value.Length <> 5 Then
            Throw New ArgumentException( _
                      ControlExistenciaCulturaMontarMensajeError( _
                       value, _
                       "Debe tener exactamente cinco (5) caracteres"))
        End If

        Dim resultado As String = String.Empty
        Dim encontrada As Boolean = False
        Dim unaCultureInfo As Globalization.CultureInfo
        For Each unaCultureInfo In _
                 Globalization.CultureInfo.GetCultures( _
                               Globalization.CultureTypes.SpecificCultures)

            If unaCultureInfo.Name.ToUpper = value.ToUpper Then
                ' guardo el valor del sistema para evitar errores con las mayusculas y minusculas
                resultado = unaCultureInfo.Name
                ' marcar que la he encontrado
                encontrada = True
                If unaCultureInfo.IsNeutralCulture Then
                    Throw New ArgumentException( _
                              ControlExistenciaCulturaMontarMensajeError( _
                               value, _
                               "No se soportan las culturas neutrales. " & _
                               "Solo se soportan las culturas especificas."))
                End If
                Exit For
            End If
        Next

        If encontrada = False Then
            Throw New ArgumentException( _
                      ControlExistenciaCulturaMontarMensajeError( _
                       value, _
                       "No es una cultura valida."))
        End If

        Return resultado
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    ''' Funcion auxiliar de <see cref="ControlExistenciaCultura">
    ''' ControlExistenciaCultura</see>. 
    ''' SOLO Monta el mensaje del error
    ''' </summary>
    ''' <param name="nombreDeLaCultura">El nombre de la cultura 
    '''   (en formato de 5 carateres) que es erronea </param>
    ''' <param name="cadenatextoDelError">
    ''' El mensaje que hay que montar de error</param>
    ''' <returns>Un mensaje de eeror informando porque la cadena con el  
    '''     <paramref name="nombreDeLaCultura">nombreDeLaCultura
    '''     </paramref> es erronea
    ''' </returns>
    Private Shared Function ControlExistenciaCulturaMontarMensajeError( _
                      ByVal nombreDeLaCultura As String, _
                      ByVal cadenatextoDelError As String) _
                      As String
        Using SW As New System.IO.StringWriter( _
                            System.Globalization.CultureInfo.CurrentCulture)
            SW.WriteLine("")
            SW.WriteLine("Estructura Dinero")
            If nombreDeLaCultura.Length > 0 Then
                SW.WriteLine("La cadena [" & nombreDeLaCultura & "] " & _
                             "que representa a una cultura, " & _
                             "no es una cultura valida.")
            End If
            SW.WriteLine(cadenatextoDelError)
            SW.WriteLine("")
            SW.WriteLine("Información sobre el formato correcto de la cultura")
            SW.WriteLine("")
            SW.WriteLine("El nombre de la referencia cultural debe escribirse " & _
                         "obligatoriamente en el código de cinco (5) caracteres " & _
                         "cuyo formato es el siguiente: " & _
                         "[códigoidioma2]-[códigopaís/región2].")
            SW.WriteLine("Donde:")
            SW.WriteLine("* [códigoidioma2] es un código de dos letras en " & _
                         "minúsculas proveniente de ISO 639-1.")
            SW.WriteLine("* [códigopaís/región2] es un código de dos letras " & _
                         "en mayúsculas proveniente de ISO 3166.")
            SW.WriteLine("Ejemplos: para España [es-ES], pata Italia [it-IT], " & _
                         "para EEUU [en-US], para Francia [fr-FR] y " & _
                         "para Inglaterra [en-GB]")
            SW.WriteLine("Observación: Los corchetes [] no forman parte " & _
                         "del código de cinco letras")
            Return SW.ToString
        End Using
    End Function

#End Region

#Region "Constructores"


    '-------------------------------------------------
    ''' <summary>Inicializa los valores internos de la estructura</summary>
    ''' <remarks>SIEMPRE debe llamarse por el constructor de la estructura</remarks>
    Private Sub InicializacionDeValoresInternos()
        _ParteEntera = 0
        _ParteDecimal = 0
        _CultureInfo = System.Globalization.CultureInfo.CurrentCulture
    End Sub


    '------------------------------------------------------------
    ' Constructor Copia a partir de un objeto del mismo tipo
    '------------------------------------------------------------

    '------------------------------------------------------------
    ''' <summary>Constructor Copia a partir de un objeto del mismo tipo</summary>
    ''' <param name="nuevoDinero">Una estructura de la que se copian los datos</param>
    Public Sub New(ByVal nuevoDinero As Dinero)
        Me._ParteEntera = nuevoDinero._ParteEntera
        Me._ParteDecimal = nuevoDinero._ParteDecimal
        Me._CultureInfo = nuevoDinero._CultureInfo
    End Sub





    '------------------------------------------------------------
    ' Entrada como un numero decimal
    '------------------------------------------------------------

    '------------------------------------------------------------
    ''' <summary>
    '''  Constructor a partir de una cantidad decimal [P.E.: 234,9876]
    ''' </summary>
    ''' <param name="value">
    ''' El número que representa el dinero [P.E.: 234,9876] 
    ''' </param>
    ''' <param name="nombreCultura"> 
    '''     Nombre de la referencia cultural en el formato 
    '''    [códigoidioma2]-[códigopaís/región2] [es-ES] [it-IT].
    ''' </param>
    Public Sub New(ByVal value As Decimal, _
                   ByVal nombreCultura As String)
        '-------------------------
        ' SIEMPRE debe llamarse por el constructor de la estructura
        Call InicializacionDeValoresInternos()
        '-------------------------
        ' lo primero la cultura
        ' Establece  el nombre de la referencia cultural en el formato 
        ' [códigoidioma2]-[códigopaís/región2] [es-ES].
        Me.CultureName = nombreCultura
        ' despues los numeros
        Call Me.FromDecimal(value)
    End Sub


    '------------------------------------------------------------
    ''' <summary>
    '''  Constructor a partir de una cantidad decimal [P.E.: 234,9876]
    ''' </summary>
    ''' <param name="value">
    ''' El número que representa el dinero [P.E.: 234,9876]
    ''' </param>
    ''' <param name="CultureInfo">La cultura donde se trabaja con el dinero 
    ''' (Por ejemplo: [New System.Globalization.CultureInfo("en-US")])
    ''' </param>
    Public Sub New(ByVal value As Decimal, _
                   ByVal cultureInfo As System.Globalization.CultureInfo)
        '-------------------------
        ' SIEMPRE debe llamarse por el constructor de la estructura
        Call InicializacionDeValoresInternos()
        '-------------------------
        ' lo primero la cultura
        Me.CultureInfo = cultureInfo
        ' despues los numeros
        Call Me.FromDecimal(value)
    End Sub


    '------------------------------------------------------------
    ''' <summary>
    '''  Constructor a partir de una cantidad decimal [P.E.: 234,9876]
    ''' </summary>
    ''' <param name="value">
    ''' El número que representa el dinero [P.E.: 234,9876] 
    ''' </param>
    Public Sub New(ByVal value As Decimal)
        '-------------------------
        ' SIEMPRE debe llamarse por el constructor de la estructura
        Call InicializacionDeValoresInternos()
        '-------------------------
        Call Me.FromDecimal(value)
    End Sub



    '------------------------------------------------------------
    ' Entrada como un numero Long
    '------------------------------------------------------------

    '------------------------------------------------------------
    ''' <summary>
    '''  Constructor a partir de una cantidad entera (sin decimales) [P.E.: 2345]
    ''' </summary>
    ''' <param name="value">
    ''' El número que representa el dinero (sin decimales) [P.E.: 2345] 
    ''' </param>
    ''' <param name="nombreCultura"> 
    '''     Nombre de la referencia cultural en el formato 
    '''    [códigoidioma2]-[códigopaís/región2] [es-ES] [it-IT].
    ''' </param>
    Public Sub New(ByVal value As Long, _
                   ByVal nombreCultura As String)
        '-------------------------
        ' SIEMPRE debe llamarse por el constructor de la estructura
        Call InicializacionDeValoresInternos()
        '-------------------------
        ' lo primero la cultura
        Me.CultureName = nombreCultura
        ' despues los numeros
        Call FromLong(value)
    End Sub


    '------------------------------------------------------------
    ''' <summary>
    '''  Constructor a partir de una cantidad entera (sin decimales) [P.E.: 2345]
    ''' </summary>
    ''' <param name="value">
    '''   El número que representa el dinero (sin decimales) [P.E.: 2345]
    '''  </param>
    ''' <param name="CultureInfo">La cultura donde se trabaja con el dinero 
    ''' (Por ejemplo: [New System.Globalization.CultureInfo("en-US")])</param>
    Public Sub New(ByVal value As Long, _
                   ByVal cultureInfo As System.Globalization.CultureInfo)
        '-------------------------
        ' SIEMPRE debe llamarse por el constructor de la estructura
        Call InicializacionDeValoresInternos()
        '-------------------------
        ' lo primero la cultura
        Me.CultureInfo = cultureInfo
        ' despues los numeros
        Call FromLong(value)
    End Sub


    '------------------------------------------------------------
    ''' <summary>
    '''  Constructor a partir de una cantidad entera (sin decimales) [P.E.: 2345]
    ''' </summary>
    ''' <param name="value">
    ''' El número que representa el dinero (sin decimales) [P.E.: 2345]
    ''' </param>
    Public Sub New(ByVal value As Long)
        '-------------------------
        ' SIEMPRE debe llamarse por el constructor de la estructura
        Call InicializacionDeValoresInternos()
        '-------------------------
        Call FromLong(value)
    End Sub


#End Region

#Region "Operadores +, -, * , /"


    '-------------------------------------------------------------------
    ''' <summary>
    '''      Suma dos cantidades de <see cref="Dinero">Dinero</see>
    '''     (Forma alternativa al operador + )
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see></param>
    ''' <param name="d2">Una cantidad de <see cref="Dinero">Dinero</see></param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    ''' Regla FxCop
    ''' Provide a method named 'Add' as a friendly 
    ''' alternate for operator 'Dinero.Operator +(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    Public Overloads Shared Operator +(ByVal d1 As Dinero, _
                                       ByVal d2 As Dinero) _
                                       As Dinero
        Return Add(d1, d2)
    End Operator

    '-------------------------------------------------------------------
    ''' <summary>
    '''      Suma dos cantidades de <see cref="Dinero">Dinero</see>
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see></param>
    ''' <param name="value">
    '''    Un número en formato <see cref="Decimal">Decimal</see> 
    '''    que representa a una cantidad de dinero
    ''' </param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    ''' Regla FxCop
    ''' Provide a method named 'Add' as a friendly 
    ''' alternate for operator 'Dinero.Operator +(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    Public Overloads Shared Operator +(ByVal d1 As Dinero, _
                                       ByVal value As Decimal) _
                                       As Dinero
        Return Add(d1, value)
    End Operator

    '-------------------------------------------------------------------
    ''' <summary>
    '''      Suma dos cantidades de <see cref="Dinero">Dinero</see>
    '''     (Forma alternativa al operador + )
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see></param>
    ''' <param name="d2">Una cantidad de <see cref="Dinero">Dinero</see></param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    ''' Regla FxCop
    ''' Provide a method named 'Add' as a friendly 
    ''' alternate for operator 'Dinero.Operator +(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    Public Overloads Shared Function Add(ByVal d1 As Dinero, _
                                         ByVal d2 As Dinero) _
                                         As Dinero

        ' Primero comprobar misma moneda
        Call ComprobacionMismaMoneda(d1, d2)

        '---------------------------
        'La suma
        'R=(A+X) + (B+Y)
        ' ---------------------------
        'R=(A+B) + (X+Y)
        '---------------------------
        'Ejemplo
        'R= (5,4) + (6,3)  = 11,70
        'Operaciones
        'R= (5+0,4) + (6+0,3) 
        'R=  (5+6)  + (0.4+0,3)
        'R=  (11)   + (0,7)
        'R=  (11 + 0,7) = 11,70
        '---------------------------
        'Ejemplo
        'R= (5,4) + (6,8)  = 12,20
        'Operaciones
        'R= (5+0,4) + (6+0,8) 
        'R=  (5+6)  + (0.4+0,8)
        'R=  (11)   + (1,2)= 12,20
        '---------------------------

        Dim A As Decimal = CType(d1.ValorParteEntera, Decimal)
        Dim X As Decimal = d1.ValorParteDecimal
        ' la parte decimal se guarda sin signo pero para operar tiene que tenerlo
        If d1.IsPositivo = False Then
            X = -X
        End If

        Dim B As Decimal = CType(d2.ValorParteEntera, Decimal)
        Dim Y As Decimal = d2.ValorParteDecimal
        If d2.IsPositivo = False Then
            Y = -Y
        End If


        Dim R As Decimal = Decimal.Zero
        R = (A + B) + (X + Y)

        Dim Resultado As Dinero = R

        Return Resultado
    End Function

    '-------------------------------------------------------------------
    ''' <summary>
    '''      Suma dos cantidades de <see cref="Dinero">Dinero</see>
    '''     (Forma alternativa al operador + )
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see></param>
    ''' <param name="value">
    '''    Un número en formato <see cref="Decimal">Decimal</see> 
    '''    que representa a una cantidad de dinero
    ''' </param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    ''' Regla FxCop
    ''' Provide a method named 'Add' as a friendly 
    ''' alternate for operator 'Dinero.Operator +(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    InvalidOperationException - Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    Public Overloads Shared Function Add(ByVal d1 As Dinero, _
                                         ByVal value As Decimal) _
                                         As Dinero
        ' carga del numero [value] en un objeto dinero
        Dim segundaCantidad As New Dinero(value, d1.CultureInfo)
        Return Add(d1, segundaCantidad)
    End Function


    '-------------------------------------------------------------------
    '-------------------------------------------------------------------
    ''' <summary>
    '''      Resta dos cantidades de <see cref="Dinero">Dinero</see>
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see>El Minuendo</param>
    ''' <param name="d2">Una cantidad de <see cref="Dinero">Dinero</see>El Sustraendo </param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    ''' Regla FxCop
    ''' Provide a method named 'Subtract' as a friendly 
    ''' alternate for operator 'Dinero.Operator -(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    InvalidOperationException - Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    Public Overloads Shared Operator -(ByVal d1 As Dinero, _
                                       ByVal d2 As Dinero) _
                                       As Dinero
        Return Subtract(d1, d2)
    End Operator

    '-------------------------------------------------------------------
    ''' <summary>
    '''      Resta dos cantidades de <see cref="Dinero">Dinero</see>
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see>El Minuendo</param>
    ''' <param name="value">
    '''    Un número en formato <see cref="Decimal">Decimal</see> 
    '''    que representa a una cantidad de dinero. El Sustraendo </param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    ''' Regla FxCop
    ''' Provide a method named 'Subtract' as a friendly 
    ''' alternate for operator 'Dinero.Operator -(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    InvalidOperationException - Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    Public Overloads Shared Operator -(ByVal d1 As Dinero, _
                                       ByVal value As Decimal) _
                                       As Dinero
        Return Subtract(d1, value)
    End Operator

    '-------------------------------------------------------------------
    ''' <summary>
    '''      Resta dos cantidades de <see cref="Dinero">Dinero</see>
    '''     (Forma alternativa al operador - )
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see>El Minuendo</param>
    ''' <param name="d2">Una cantidad de <see cref="Dinero">Dinero</see>El Sustraendo </param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    ''' Regla FxCop
    ''' Provide a method named 'Subtract' as a friendly 
    ''' alternate for operator 'Dinero.Operator -(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    InvalidOperationException - Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    Public Overloads Shared Function Subtract(ByVal d1 As Dinero, _
                                              ByVal d2 As Dinero) _
                                              As Dinero
        Return Add(d1, (-d2))
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    '''      Resta dos cantidades de <see cref="Dinero">Dinero</see>
    '''     (Forma alternativa al operador - )
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see>El Minuendo</param>
    '''     ''' <param name="value">
    '''    Un número en formato <see cref="Decimal">Decimal</see> 
    '''    que representa a una cantidad de dinero. El Sustraendo </param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    ''' Regla FxCop
    ''' Provide a method named 'Subtract' as a friendly 
    ''' alternate for operator 'Dinero.Operator -(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    InvalidOperationException - Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    Public Overloads Shared Function Subtract(ByVal d1 As Dinero, _
                                              ByVal value As Decimal) _
                                              As Dinero
        ' carga del numero [value] en un objeto dinero
        Dim segundaCantidad As New Dinero(value, d1.CultureInfo)
        Return Subtract(d1, segundaCantidad)
    End Function

    '-------------------------------------------------------------------
    ''' <summary>
    '''      Multiplica dos cantidades de <see cref="Dinero">Dinero</see>
    '''     (Forma alternativa al operador * )
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see>El Multiplicando</param>
    ''' <param name="d2">Una cantidad de <see cref="Dinero">Dinero</see>El Multiplicador </param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    '''   Regla FxCop - Provide a method named 'Multiply' as a friendly 
    '''   alternate for operator 'Dinero.Operator *(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    InvalidOperationException - Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    Public Overloads Shared Function Multiply( _
                                       ByVal d1 As Dinero, _
                                       ByVal d2 As Dinero) _
                                       As Dinero
        Call ComprobacionMismaMoneda(d1, d2)


        '---------------------------
        'La Multiplicación
        'R=(A+X) x (B+Y)
        ' ---------------------------
        'R= (AxB) + (AxY) + (XxB) + (XxY)
        'Ejemplo
        'R= (4,2) x (5,6)  = 23,52
        'Operaciones
        'R= (4+0,2) x (5+0,6)
        'R= (4x5) + (4x0,6) + (0,2x5) + (0,2x0,6)
        'R= (20) + (2,4) + (1) + (0.12) = 23,52





        '---------------------------


        Dim A As Decimal = CType(d1.ValorParteEntera, Decimal)
        Dim X As Decimal = d1.ValorParteDecimal
        ' la parte decimal se guarda sin signo pero para operar tiene que tenerlo
        If d1.IsPositivo = False Then
            X = -X
        End If

        Dim B As Decimal = CType(d2.ValorParteEntera, Decimal)
        Dim Y As Decimal = d2.ValorParteDecimal
        If d2.IsPositivo = False Then
            Y = -Y
        End If


        Dim R As Decimal = Decimal.Zero
        R = (A * B) + (A * Y) + (X * B) + (X * Y)

        Dim Resultado As Dinero = R

        Return Resultado

    End Function

    '-------------------------------------------------------------------
    ''' <summary>
    '''      Multiplica dos cantidades de <see cref="Dinero">Dinero</see>
    '''     (Forma alternativa al operador * )
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see>El Multiplicando</param>
    ''' <param name="value">
    '''    Un número en formato <see cref="Decimal">Decimal</see> 
    '''    que representa a una cantidad de dinero. El Multiplicador </param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    '''   Regla FxCop - Provide a method named 'Multiply' as a friendly 
    '''   alternate for operator 'Dinero.Operator *(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    InvalidOperationException - Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    Public Overloads Shared Function Multiply( _
                                        ByVal d1 As Dinero, _
                                        ByVal value As Decimal) _
                                        As Dinero
        ' carga del numero [value] en un objeto dinero
        Dim segundaCantidad As New Dinero(value, d1.CultureInfo)
        Return Multiply(d1, segundaCantidad)
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    '''      Multiplica dos cantidades de <see cref="Dinero">Dinero</see>
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see>El Multiplicando</param>
    ''' <param name="d2">Una cantidad de <see cref="Dinero">Dinero</see>El Multiplicador </param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    '''   Regla FxCop - Provide a method named 'Multiply' as a friendly 
    '''   alternate for operator 'Dinero.Operator *(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    InvalidOperationException - Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    Public Overloads Shared Operator *(ByVal d1 As Dinero, _
                                       ByVal d2 As Dinero) _
                                       As Dinero
        Return Multiply(d1, d2)
    End Operator

    '-------------------------------------------------------------------
    ''' <summary>
    '''      Multiplica dos cantidades de <see cref="Dinero">Dinero</see>
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see>El Multiplicando</param>
    ''' <param name="value">
    '''    Un número en formato <see cref="Decimal">Decimal</see> 
    '''    que representa a una cantidad de dinero. El Multiplicador </param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    '''   Regla FxCop - Provide a method named 'Multiply' as a friendly 
    '''   alternate for operator 'Dinero.Operator *(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    InvalidOperationException - Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    Public Overloads Shared Operator *(ByVal d1 As Dinero, _
                                       ByVal value As Decimal) _
                                       As Dinero
        Return Multiply(d1, value)
    End Operator

    '-------------------------------------------------------------------
    ''' <summary>
    '''      Divide dos cantidades de <see cref="Dinero">Dinero</see>
    '''     (Forma alternativa al operador / )
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see>El Dividendo</param>
    ''' <param name="d2">Una cantidad de <see cref="Dinero">Dinero</see>El Divisor </param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    '''   Regla FxCop - Provide a method named 'Divide' as a friendly 
    ''' alternate for operator 'Dinero.Operator /(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    InvalidOperationException - Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    ''' <exception cref="System.DivideByZeroException">
    '''    DivideByZeroException - El divisor es cero
    ''' </exception>
    Public Overloads Shared Function Divide( _
                                     ByVal d1 As Dinero, _
                                     ByVal d2 As Dinero) _
                                     As Dinero

        Call ComprobacionMismaMoneda(d1, d2)

        ' Error division por cero
        If d2 = 0 Then
            Throw New System.DivideByZeroException("El divisor es cero")
        End If

        ' ---------------------------
        'La división
        ' R=(A+X) / (B+Y)
        ' ---------------------------
        '
        '     (A+X)
        'R = ---------
        '     (B+Y)
        '
        ' Si multiplicamos los dos términos de una fracción por un
        ' mismo número, la fracción no varia
        '     (A+X) x (B+Y)            (AxB) + (AxY) + (XxB) + (XxY)
        '= ------------------------ = ------------------------------------------
        '     (B+Y) x (B+Y)            (BxB) + (BxY) + (YxB) + (YxY)
        '
        '         (AxB) + (AxY) + (XxB) + (XxY)
        'R = ------------------------------------------
        '          (BxB) + (2xBxY) + (YxY)
        '
        'Ejemplo
        'R= (4,2) / (5,6)  = 0,75
        'Operaciones
        'R= (4+0,2) / (5+0,6)
        '
        '     (4+0,2) x (5+0,6)            (4x5) + (4x0,6) + (0,2x5) + (0,2x0,6)
        '= ------------------------  =  ----------------------------------------------
        '     (5+0,6) x (5+0,6)            (5x5) + (5x0,6) + (5x0,6) + (0,6x0,6)
        '
        '     (20) + (2,4) + (1) + (0.12)              23,52
        '= ----------------------------------  =  --------------- = 0,75
        '     (25) + (3) + (3) + (0,36)                31,36
        '
        ' ---------------------------

        Dim A As Decimal = CType(d1.ValorParteEntera, Decimal)
        Dim X As Decimal = d1.ValorParteDecimal
        ' la parte decimal se guarda sin signo pero para operar tiene que tenerlo
        If d1.IsPositivo = False Then
            X = -X
        End If

        Dim B As Decimal = CType(d2.ValorParteEntera, Decimal)
        Dim Y As Decimal = d2.ValorParteDecimal
        If d2.IsPositivo = False Then
            Y = -Y
        End If

        '        (AxB) + (AxY) + (XxB) + (XxY)
        'R = ------------------------------------------
        '          (BxB) + (2xBxY) + (YxY)

        Dim numerador As Decimal = (A * B) + (A * Y) + (X * B) + (X * Y)
        Dim denominador As Decimal = (B * B) + (2 * B * Y) + (Y * Y)
        Dim R As Decimal = Decimal.Zero
        R = numerador / denominador

        ' carga del resultado en un objeto dinero
        Dim Resultado As Dinero = R
        Return Resultado

    End Function

    '-------------------------------------------------------------------
    ''' <summary>
    '''      Divide dos cantidades de <see cref="Dinero">Dinero</see>
    '''     (Forma alternativa al operador / )
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see>El Dividendo</param>
    ''' <param name="value">
    '''    Un número en formato <see cref="Decimal">Decimal</see> 
    '''    que representa a una cantidad de dinero. El Divisor </param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    '''   Regla FxCop - Provide a method named 'Divide' as a friendly 
    ''' alternate for operator 'Dinero.Operator /(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    InvalidOperationException - Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    ''' <exception cref="System.DivideByZeroException">
    '''    DivideByZeroException - El divisor es cero
    ''' </exception>
    Public Overloads Shared Function Divide( _
                                     ByVal d1 As Dinero, _
                                     ByVal value As Decimal) _
                                     As Dinero
        ' carga del numero [value] en un objeto dinero
        Dim segundaCantidad As New Dinero(value, d1.CultureInfo)
        Return Divide(d1, segundaCantidad)
    End Function

    '-------------------------------------------------------------------
    ''' <summary>
    '''      Divide dos cantidades de <see cref="Dinero">Dinero</see>
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see>El Dividendo</param>
    ''' <param name="d2">Una cantidad de <see cref="Dinero">Dinero</see>El Divisor </param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    '''   Regla FxCop - Provide a method named 'Divide' as a friendly 
    ''' alternate for operator 'Dinero.Operator /(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    InvalidOperationException - Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    ''' <exception cref="System.DivideByZeroException">
    '''    DivideByZeroException - El divisor es cero
    ''' </exception>
    Public Overloads Shared Operator /( _
                               ByVal d1 As Dinero, _
                               ByVal d2 As Dinero) _
                               As Dinero
        Return Divide(d1, d2)
    End Operator

    '-------------------------------------------------------------------
    ''' <summary>
    '''      Divide dos cantidades de <see cref="Dinero">Dinero</see>
    ''' </summary>
    ''' <param name="d1">Una cantidad de <see cref="Dinero">Dinero</see>El Dividendo</param>
    ''' <param name="value">
    '''    Un número en formato <see cref="Decimal">Decimal</see> 
    '''    que representa a una cantidad de dinero. El Divisor </param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks> 
    '''   Regla FxCop - Provide a method named 'Divide' as a friendly 
    ''' alternate for operator 'Dinero.Operator /(Dinero, Dinero)
    '''</remarks>
    ''' <exception cref="InvalidOperationException">
    '''    InvalidOperationException - Excepcion que se dispara cuando 
    '''    las monedas de las estructura Dinero no son iguales
    ''' </exception>
    ''' <exception cref="System.DivideByZeroException">
    '''    DivideByZeroException - El divisor es cero
    ''' </exception>
    Public Overloads Shared Operator /( _
                               ByVal d1 As Dinero, _
                               ByVal value As Decimal) _
                               As Dinero
        Return Divide(d1, value)
    End Operator


#End Region


#Region "Operadores \ mod and or, not"


    ''' <summary>
    '''    Representa el mayor valor posible de <see cref="Dinero">Dinero</see>,
    '''    este campo es constante y de solo lectura
    ''' </summary>
    ''' <remarks>El valor de esta constante es más 79.228.162.514.264.337.593.543.950.335.</remarks>
    Public Shared ReadOnly Property MaxValue() As Dinero
        Get
            ' ------------------------------------
            ' regla FxCop
            '   replace this call with a 
            '   call to 'Dinero.New(Decimal, CultureInfo)'. 
            ' ------------------------------------
            ' No la cumplo porque el constructor ya lo hace 
            ' ------------------------------------
            Return New Dinero(Decimal.MaxValue)
        End Get
    End Property

    ''' <summary>
    '''    Representa el menor valor posible de  <see cref="Dinero">Dinero</see>, 
    '''    este campo es constante y de solo lectura
    ''' </summary>
    ''' <remarks>El valor de esta constante es menos 79.228.162.514.264.337.593.543.950.335.</remarks>
    Public Shared ReadOnly Property MinValue() As Dinero
        Get
            ' ------------------------------------
            ' regla FxCop
            '   replace this call with a 
            '   call to 'Dinero.New(Decimal, CultureInfo)'. 
            ' ------------------------------------
            ' No la cumplo porque el constructor ya lo hace 
            ' ------------------------------------
            Return New Dinero(Decimal.MinValue)
        End Get
    End Property

    ''' <summary>
    '''    Representa el valor menos uno (-1,00), este campo es constante y de solo lectura
    ''' </summary>
    Public Shared ReadOnly Property MinusOne() As Dinero
        Get
            ' ------------------------------------
            ' regla FxCop
            '   replace this call with a 
            '   call to 'Dinero.New(Decimal, CultureInfo)'. 
            ' ------------------------------------
            ' No la cumplo porque el constructor ya lo hace 
            ' ------------------------------------
            Return New Dinero(Decimal.MinusOne)
        End Get
    End Property

    ''' <summary>
    '''    Representa el valor cero (0,00), este campo es constante y de solo lectura
    ''' </summary>
    Public Shared ReadOnly Property Zero() As Dinero
        Get
            ' ------------------------------------
            ' regla FxCop
            '   replace this call with a 
            '   call to 'Dinero.New(Decimal, CultureInfo)'. 
            ' ------------------------------------
            ' No la cumplo porque el constructor ya lo hace 
            ' ------------------------------------
            Return New Dinero(Decimal.Zero)
        End Get
    End Property

    ''' <summary>
    '''  El valor del  <see cref="Dinero">Dinero</see> es cero (0) ??
    ''' </summary>
    ''' <returns>
    '''    <para>Un valor  <see cref="System.Boolean">lógico</see> que indica:</para>
    '''    <para>True: el valor de Dinero es la cantidad Cero (0,00)</para>
    '''    <para>False: el cualquier otro valor</para>
    ''' </returns>
    ''' <remarks></remarks>
    Public Function IsZero() As Boolean
        Dim resultado As Boolean = False
        If Me._ParteEntera = 0 AndAlso Me._ParteDecimal = 0 Then
            resultado = True
        End If
        Return resultado
    End Function

    ''' <summary>
    '''    Representa el valor uno (1,00), este campo es constante y de solo lectura
    ''' </summary>
    Public Shared ReadOnly Property One() As Dinero
        Get
            ' ------------------------------------
            ' regla FxCop
            '   replace this call with a 
            '   call to 'Dinero.New(Decimal, CultureInfo)'. 
            ' ------------------------------------
            ' No la cumplo porque el constructor ya lo hace 
            ' ------------------------------------
            Return New Dinero(Decimal.One)
        End Get
    End Property


    ''' <summary>
    '''  El valor del <see cref="Dinero">Dinero</see> es la unidad (1,00) ??
    ''' </summary>
    ''' <returns>
    '''    <para>Un valor  <see cref="System.Boolean">lógico</see> que indica:</para>
    '''    <para>True: el valor de Dinero es la cantidad Uno (1,00)</para>
    '''    <para>False: el cualquier otro valor</para>
    ''' </returns>
    ''' <remarks></remarks>
    Public Function IsOne() As Boolean
        Dim resultado As Boolean = False
        If Me._ParteEntera = 1 AndAlso Me._ParteDecimal = 0 Then
            resultado = True
        End If
        Return resultado
    End Function


    ''' <summary>
    '''  Devuelve el número entero más pequeño mayor o igual que el valor de
    '''  <see cref="Dinero">Dinero</see> especificado.
    ''' </summary>
    ''' <param name="d">Una cantidad de <see cref="Dinero">Dinero</see></param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    '''          Ejemplo Ceiling(-15,15) = -15,00
    '''          Ejemplo Ceiling(+15,15) = +16,00
    ''' </returns>
    ''' <remarks>
    ''' El comportamiento de este método sigue el estándar IEEE 754, sección 4. 
    ''' A este tipo de redondeo a veces se le denomina redondeo hacia el infinito positivo.
    ''' </remarks>
    ''' <example>
    ''' <para>Ceiling(-15,15) = -15,00</para>
    ''' <para>Ceiling(-15,85) = -15,00</para>
    ''' <para>Ceiling(+15,15) = +16,00</para>
    ''' <para>Ceiling(+15,85) = +16,00</para>
    '''</example>
    Public Shared Function Ceiling(ByVal d As Dinero) As Dinero
        Return New Dinero(Math.Ceiling(d.Valor), d.CultureInfo)
    End Function



    ''' <summary>
    '''  Devuelve el número entero más grande menor o igualque el valor de
    '''  <see cref="Dinero">Dinero</see> especificado.
    ''' </summary>
    ''' <param name="d">Una cantidad de <see cref="Dinero">Dinero</see></param>
    ''' <returns>Una cantidad de <see cref="Dinero">Dinero</see> 
    '''          que contiene el resultado de la operación
    '''          Ejemplo Floor(-15,15) = -16,00
    '''          Ejemplo Floor(+15,85) = +15,00
    ''' </returns>
    ''' <remarks>
    ''' El comportamiento de este método sigue el estándar IEEE 754, sección 4. 
    ''' A este tipo de redondeo a veces se le denomina redondeo hacia el infinito negativo
    ''' </remarks>
    ''' <example>
    ''' <para>Floor(-15,15) = -16,00</para>
    ''' <para>Floor(-15,85) = -16,00</para>
    ''' <para>Floor(+15,15) = +15,00</para>
    ''' <para>Floor(+15,85) = +15,00</para>
    '''</example>
    Public Shared Function Floor(ByVal d As Dinero) As Dinero
        Return New Dinero(Math.Floor(d.Valor), d.CultureInfo)
    End Function


    ''' <summary>
    '''  Convierte el valor de una instancia de <see cref="Dinero">Dinero</see> especificada 
    '''  en la representación binaria equivalente.
    ''' </summary>
    ''' <param name="d">Valor de Dinero</param>
    ''' <returns>
    '''     Matriz de enteros de 32 bits con signo con cuatro elementos que  
    '''     contiene la representación binaria de
    '''     <paramref name="d">d</paramref>
    ''' </returns>
    ''' <remarks>
    '''  <para>
    '''       La representación binaria de un número Decimal se compone de un signo de 1 bit, 
    '''       un número entero de 96 bits y un factor de escala que se utiliza para dividir  
    '''       el número entero y especificar la parte del mismo que es una fracción decimal.  
    '''       El factor de escala es implícitamente el número 10, elevado a un exponente  
    '''       comprendido entre el 0 y el 28.
    ''' </para> 
    ''' <para>
    '''       El valor devuelto es una matriz de enteros de 32 bits 
    '''       con signo de cuatro elementos.
    ''' </para> 
    ''' <para> 
    '''       El primer, segundo y tercer elementos de la matriz devuelta contienen los 32  
    '''       bits inferiores, intermedios y superiores del número entero de 96 bits.
    ''' </para> 
    ''' <para>
    '''       El cuarto elemento de la matriz devuelta contiene el factor de escala y  
    '''       el signo. Consta de las siguientes partes: 
    ''' </para> 
    ''' <para>
    '''       Los bits del 0 al 15, la palabra inferior, no se utilizan y deben ser cero.
    ''' </para> 
    ''' <para>
    '''       Los bits del 16 al 23 deben contener un exponente comprendido entre el 
    '''       0 y el 28, que indica la potencia de 10 para con la que se divide 
    '''       el número entero.
    ''' </para> 
    ''' <para>
    '''       Los bits del 24 al 30 no se utilizan y deben ser cero.
    ''' </para> 
    ''' <para>
    '''       El bit 31 contiene el signo; 0 significa positivo y 1 significa negativo.
    ''' </para> 
    ''' <para>
    '''       Tenga en cuenta que en la representación de bits se hace una distinción   
    '''       entre el cero negativo y el positivo. 
    '''       Estos valores se consideran iguales en todas las operaciones.
    ''' </para> 
    ''' <para>
    ''' http://msdn.microsoft.com/es-es/library/system.decimal.getbits(VS.80).aspx
    ''' </para>
    ''' </remarks>
    Public Shared Function GetBits(ByVal d As Dinero) As Integer()
        Dim numDecimal As Decimal = d.ToDecimal
        Return Decimal.GetBits(numDecimal)
    End Function



    ''' <summary>
    '''     Un numero inverso es el resultado de hacer la operacion 1/x
    ''' </summary>
    ''' <param name="d">Una cantidad de <see cref="Dinero">Dinero</see></param>
    ''' <exception cref="System.DivideByZeroException">
    '''    DivideByZeroException - El divisor es cero
    ''' </exception>
    Public Shared Function NumeroInverso(ByVal d As Dinero) As Dinero
        ' un numero inverso es 1/x
        Return Divide(1, d)
    End Function



    ''' <summary>
    '''     Redondea un valor dinero, al número de posiciones decimales definidas en CultureInfo. 
    '''     (P.E. el Euro tiene dos decimales)
    ''' </summary>
    ''' <param name="d">Una estructura<seealso cref="Dinero">Dinero</seealso> 
    '''  que representa a una cantidad de dinero</param>
    ''' <returns>Redondea los decimales, dejando el número de decimales que tiene la moneda,
    '''      y que estan definidas en el campo CultureInfo.
    '''      Por ejemplo, le Euro tiene dos decimales.</returns>
    ''' <remarks>El comportamiento de este método sigue el estándar IEEE 754, sección 4. 
    '''          A este tipo de redondeo a veces se le denomina redondeo al entero más 
    '''          próximo o redondeo bancario.</remarks>
    Public Overloads Shared Function Round(ByVal d As Dinero) As Dinero
        Return New Dinero(Decimal.Round(d.ValorSinRedondear), d.CultureInfo)
    End Function


    ''' <summary>
    '''      Redondea un valor <seealso cref="Dinero">Dinero</seealso> 
    '''      a un número especificado de posiciones decimales
    ''' </summary>
    ''' <param name="d">Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''                 Valor que se va a redondear. </param>
    ''' <param name="decimals">Tipo: <seealso cref="System.Int32"> System.Int32</seealso> 
    '''     Número de posiciones decimales significativas (precisión) 
    '''     del valor devuelto. </param>
    ''' <returns>Tipo:<seealso cref="Dinero">Dinero</seealso> 
    '''        Número Decimal equivalente a <paramref name="d">d</paramref>
    '''        redondeado a un número de posiciones decimales especificado en 
    '''        <paramref name="decimals">decimals</paramref>
    ''' </returns>
    ''' <remarks>
    '''    Cuando <paramref name="d">d</paramref> se encuentra exactamente en medio 
    '''    de dos valores redondeados, el resultado es el valor redondeado que tiene 
    '''    un dígito par en la posición decimal que se encuentra más a la derecha. 
    '''    Por ejemplo, cuando se redondea a dos decimales, el valor 2,345 se 
    '''    convierte en 2,34 y el valor 2,355 pasa a ser 2,36. 
    '''    Este proceso se conoce con el nombre de redondeo al par o redondeo al más próximo.
    '''  </remarks>
    Public Overloads Shared Function Round(ByVal d As Dinero, ByVal decimals As Integer) As Dinero
        Return New Dinero(Decimal.Round(d.ValorSinRedondear, decimals), d.CultureInfo)
    End Function

    ''' <summary>
    '''    Redondea un valor <seealso cref="Dinero">Dinero</seealso> al entero más próximo. 
    '''    Un parámetro especifica el redondeo del valor si está a la mitad del 
    '''    intervalo entre otros dos números.
    ''' </summary>
    ''' <param name="d">Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''                 Valor que se va a redondear. </param>
    ''' <param name="mode">Tipo: <seealso cref="System.MidpointRounding"> 
    '''     System.MidpointRounding</seealso>. Valor que especifica el redondeo de 
    '''    <paramref name="d">d</paramref> si su valor está a la mitad del intervalo 
    '''     entre otros dos números</param>
    ''' <returns>Tipo:<seealso cref="Dinero">Dinero</seealso> Número más cercano 
    '''    al parámetro <paramref name="d">d</paramref>
    '''    Si <paramref name="d">d</paramref> se encuentra a la mitad del intervalo 
    '''    entre dos números, uno par y otro impar, el parámetro 
    '''    <paramref name="mode">mode</paramref> determina el número que se devuelve. 
    ''' </returns>
    ''' <remarks>
    '''    El comportamiento de este método sigue el estándar IEEE 754, sección 4. 
    '''    Este tipo de redondeo se suele denominar redondeo al entero más próximo o 
    '''    redondeo bancario. Si decimals es cero, este redondeo suele denominar 
    '''    redondeo a cero.
    ''' </remarks>
    Public Overloads Shared Function Round(ByVal d As Dinero, ByVal mode As MidpointRounding) As Dinero
        Return New Dinero(Decimal.Round(d.ValorSinRedondear, mode), d.CultureInfo)
    End Function


    ''' <summary>
    '''      Redondea un valor <seealso cref="Dinero">Dinero</seealso> a una 
    '''      precisión especificada. Un parámetro especifica el redondeo del valor 
    '''      si está a la mitad del intervalo entre otros dos números.
    ''' </summary>
    ''' <param name="d">Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''                 Valor que se va a redondear. </param>
    ''' <param name="decimals">Tipo: <seealso cref="System.Int32"> System.Int32</seealso> 
    '''     Número de posiciones decimales significativas (precisión) 
    '''     del valor devuelto. </param>
    ''' <param name="mode">Tipo: <seealso cref="System.MidpointRounding"> 
    '''     System.MidpointRounding</seealso>. Valor que especifica el redondeo de 
    '''    <paramref name="d">d</paramref> si su valor está a la mitad del intervalo 
    '''     entre otros dos números</param>
    ''' <returns>Tipo:<seealso cref="Dinero">Dinero</seealso> Número más cercano 
    '''    al parámetro <paramref name="d">d</paramref> con una precisión igual al 
    '''    parámetro <paramref name="decimals">decimals</paramref>. 
    '''    Si <paramref name="d">d</paramref> se encuentra a la mitad del intervalo 
    '''    entre dos números, uno par y otro impar, el parámetro 
    '''    <paramref name="mode">mode</paramref> determina el número que se devuelve. 
    '''    Si la precisión de <paramref name="d">d</paramref> es menor que 
    '''    <paramref name="decimals">decimals</paramref>, se devuelve 
    '''    <paramref name="d">d</paramref> sin modificar. 
    ''' </returns>
    ''' <remarks>
    ''' <para>
    '''    El parámetro <paramref name="decimals">decimals</paramref> 
    '''    especifica el número de posiciones decimales significativas del valor devuelto,
    '''    que puede oscilar entre 0 y 28. Si decimals es cero, se devuelve un número entero.
    ''' </para>
    '''<para>
    '''    El comportamiento de este método sigue el estándar IEEE 754, sección 4. 
    '''    Este tipo de redondeo se suele denominar redondeo al entero más próximo o 
    '''    redondeo bancario. Si decimals es cero, este redondeo suele denominar 
    '''    redondeo a cero.
    ''' </para> 
    ''' </remarks>
    Public Overloads Shared Function Round( _
                    ByVal d As Dinero, _
                    ByVal decimals As Integer, _
                    ByVal mode As MidpointRounding) As Dinero
        Return New Dinero(Decimal.Round(d.ValorSinRedondear, decimals, mode), d.CultureInfo)
    End Function


    ''' <summary>
    '''     Redondea un valor dinero, al número de posiciones decimales definidas en CultureInfo. 
    '''     (P.E. el Euro tiene dos decimales)
    ''' </summary>
    ''' <param name="d">Una estructura<seealso cref="Dinero">Dinero</seealso> 
    '''  que representa a una cantidad de dinero</param>
    ''' <returns>Redondea los decimales, dejando el número de decimales que tiene la moneda. 
    '''      Por ejemplo, le Euro tiene dos decimales.</returns>
    ''' <remarks>
    '''   <para>El comportamiento de este método sigue el estándar IEEE 754, sección 4. 
    '''          A este tipo de redondeo a veces se le denomina redondeo al entero más 
    '''          próximo o redondeo bancario.</para>
    '''    <para>Cuando <paramref name="d">d</paramref> se encuentra exactamente en medio 
    '''    de dos valores redondeados, el resultado es el valor redondeado que tiene 
    '''    un dígito par en la posición decimal que se encuentra más a la derecha. 
    '''    Por ejemplo, cuando se redondea a dos decimales, el valor 2,345 se 
    '''    convierte en 2,34 y el valor 2,355 pasa a ser 2,36. </para>
    '''    <para>Este proceso se conoce con el nombre de redondeo al 
    '''          par o redondeo al más próximo.</para>
    ''' </remarks>
    Public Overloads Shared Function DigitosDecimalesAjustarAMoneda(ByVal d As Dinero) As Dinero
        Return Dinero.Round(d)
    End Function



    ''' <summary>
    '''     Redondea un valor dinero, al número de posiciones decimales definidas en CultureInfo. 
    '''     (P.E. el Euro tiene dos decimales)
    ''' </summary>
    ''' <returns>Redondea los decimales, dejando el número de decimales que tiene la moneda. 
    '''      Por ejemplo, le Euro tiene dos decimales.</returns>
    ''' <remarks>
    '''   <para>El comportamiento de este método sigue el estándar IEEE 754, sección 4. 
    '''          A este tipo de redondeo a veces se le denomina redondeo al entero más 
    '''          próximo o redondeo bancario.</para>
    '''    <para>Cuando <paramref name="d">d</paramref> se encuentra exactamente en medio 
    '''    de dos valores redondeados, el resultado es el valor redondeado que tiene 
    '''    un dígito par en la posición decimal que se encuentra más a la derecha. 
    '''    Por ejemplo, cuando se redondea a dos decimales, el valor 2,345 se 
    '''    convierte en 2,34 y el valor 2,355 pasa a ser 2,36. </para>
    '''    <para>Este proceso se conoce con el nombre de redondeo al 
    '''          par o redondeo al más próximo.</para>
    ''' </remarks>
    Public Overloads Function DigitosDecimalesAjustarAMoneda() As Dinero
        ' nueva estructura dinero con los valores de esta instancia
        Dim aux As New Dinero(Me)
        'redondear los decimales
        aux = Dinero.Round(aux)
        ' copiar la parte decimal redondeada en esta instancia
        Me._ParteDecimal = aux._ParteDecimal
        ' devolver el valor actual de esta instancia
        Return Me
    End Function



    ''' <summary>
    '''  cuenta el número de cifras decimales (significativas) del dinero. 
    '''   p.e   4,567 devuelve tres(3)
    '''   p.e  12,00  devuelve cero (0)
    ''' </summary>
    ''' <param name="d">una Estructura Dinero que representa a una cantidad de dinero</param>
    ''' <returns>un numero entero que indica las cifras decimales (significativas) del numero.</returns>
    ''' <remarks>
    ''' Ejemplo, 12,12   devuelve 2
    ''' Ejemplo, 12,00   devuelve 0 ' atencion
    ''' Ejemplo, 12,1234 devuelve 4
    ''' </remarks>
    Public Overloads Shared Function DigitosDecimalesCount(ByVal d As Dinero) As Integer

        Try
            Dim numeroDecimales As Integer = 0

            Dim cadena As String = String.Empty
            cadena = d.ToString(d.CultureInfo)

            Dim SeparadorDecimal As String = d.MonedaSeparadorDecimal

            Dim posicionSeparadorDecimal As Integer = 0
            posicionSeparadorDecimal = cadena.IndexOf(SeparadorDecimal)

            If posicionSeparadorDecimal <> -1 Then
                ' hay un separador decimal
                ' 0         1         2
                ' 012345678901234567890 [Posiciones]
                ' 12,1234567890123  
                ' La coma decimal esta en la posicion 2
                ' el tamaño de la cadena es de 15 caracteres
                ' Cuidado count develve 16 caracteres
                ' (16-1)-2 = 13 posiciones decimales

                numeroDecimales = (cadena.Length - 1) - posicionSeparadorDecimal
                If numeroDecimales < 0 Then
                    numeroDecimales = 0
                End If
            End If

            Return numeroDecimales

        Catch ex As Exception
            Throw
        End Try
    End Function


    ''' <summary>
    '''  cuenta el número de cifras decimales (significativas) del dinero. 
    '''   p.e   4,567 devuelve tres(3)
    '''   p.e  12,00  devuelve cero (0)
    ''' </summary>
    ''' <returns>un numero entero que indica las cifras decimales (significativas) del numero.</returns>
    ''' <remarks>
    ''' Ejemplo, 12,12   devuelve 2
    ''' Ejemplo, 12,00   devuelve 0 ' atencion
    ''' Ejemplo, 12,1234 devuelve 4
    ''' </remarks>
    Public Overloads Function DigitosDecimalesCount() As Integer
        Return DigitosDecimalesCount(Me)
    End Function






    ''' <summary>
    ''' Devuelve una cantidad de dinero elevado a la potencia especificada.
    ''' </summary>
    ''' <param name="d1">d1 - Una estructura <see cref="Dinero">Dinero</see> (Base)</param>
    ''' <param name="d2">d2 - Una estructura <see cref="Dinero">Dinero</see> (Exponente)</param>
    ''' <returns>Una estructura <see cref="Dinero">Dinero</see> 
    ''' que contiene el resultado de la operación</returns>
    ''' <remarks></remarks>
    Public Shared Operator ^( _
                          ByVal d1 As Dinero, _
                          ByVal d2 As Dinero) _
                          As Dinero
        Return Dinero.Pow(d1, d2)
    End Operator


    ''' <summary>
    ''' Devuelve una cantidad de dinero elevado a la potencia especificada.
    '''  (Forma alternativa al operador ^ )
    ''' </summary>
    ''' <param name="d1">d1 - Una estructura <see cref="Dinero">Dinero</see> (Base)</param>
    ''' <param name="d2">d2 - Una estructura <see cref="Dinero">Dinero</see> (Exponente)</param>
    ''' <returns>Una estructura <see cref="Dinero">Dinero</see> 
    '''           que contiene el resultado de la operación
    ''' </returns>
    ''' <remarks></remarks>
    Public Shared Function Pow( _
                       ByVal d1 As Dinero, _
                       ByVal d2 As Dinero) _
                       As Dinero


        ' error de infinito - devuelv un valor de dinero de 0
        If d1.IsZero = True Then
            Return Dinero.Zero
        End If

        ' 5 Exp 0 = 1
        If d2.IsZero = True Then
            Return Dinero.One
        End If

        ' control de misma moneda
        Call ComprobacionMismaMoneda(d1, d2)

        ' operaciones de exponenciacion
        Dim decimalPrimero As Decimal = d1.ToDecimal
        Dim decimalSegundo As Decimal = d2.ToDecimal

        Dim decimalResultado As Decimal
        decimalResultado = CType(Math.Pow(decimalPrimero, decimalSegundo), Decimal)

        Return New Dinero(decimalResultado, d1.CultureInfo)

    End Function



    ''' <summary>
    '''    Devuelve el resultado de multiplicar el valor <see cref="Dinero">Dinero</see>
    '''    especificado por menos uno (-1).
    ''' </summary>
    ''' <param name="d"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <returns>el valor de <paramref name="d">d</paramref> pero con signo opuesto. 
    ''' O bien Cero(0), si <paramref name="d">d</paramref> es cero. </returns>
    ''' <remarks></remarks>
    Public Shared Function Negate(ByVal d As Dinero) As Dinero
        If d.Valor = 0 Then
            Return Dinero.Zero
        Else
            Return Multiply(d, (-1D))
        End If
    End Function

    ''' <summary>
    '''    Convierte en negativo el valor del operando 
    '''    <see cref="Dinero">Dinero</see> especificado.
    ''' </summary>
    ''' <param name="d"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <returns>el valor de <paramref name="d">d</paramref> pero con signo opuesto. 
    ''' O bien Cero(0), si <paramref name="d">d</paramref> es cero. </returns>
    ''' <remarks></remarks>
    Public Shared Operator -(ByVal d As Dinero) As Dinero
        ' ------------------------------------
        ' regla FxCop
        '   replace this call with a 
        '   call to 'Dinero.New(d, CultureInfo)'. 
        ' ------------------------------------
        ' No la cumplo porque el constructor ya lo hace 
        ' ------------------------------------
        Return Negate(d)
    End Operator


    ''' <summary>
    '''  Cambia el signo de la cantidad de dinero. 
    '''  Si el signo es negativo pone el signo positivo, 
    '''  si el signo es positivo no cambia nada
    ''' </summary>
    ''' <param name="d"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <returns>El resultado de la operación en una estructura 
    '''  <see cref="Dinero">Dinero</see>.</returns>
    ''' <remarks></remarks>
    Public Shared Operator +(ByVal d As Dinero) As Dinero
        Return Plus(d)
    End Operator


    ''' <summary>
    '''  Cambia el signo de la cantidad de dinero. 
    '''  Si el signo es negativo pone el signo positivo, 
    '''  si el signo es positivo no cambia nada
    ''' </summary>
    ''' <param name="d"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <returns>El resultado de la operación en una estructura 
    '''  <see cref="Dinero">Dinero</see>.</returns>
    ''' <remarks>
    '''   Regla FxCop
    '''   Provide a method named 'Plus' as a friendly alternate 
    '''   for operator 'Dinero.Operator +(Dinero)'.
    '''</remarks>
    Public Shared Function Plus(ByVal d As Dinero) As Dinero
        ' ------------------------------------
        ' regla FxCop
        '   replace this call with a 
        '   call to 'Dinero.New(d, CultureInfo)'. 
        ' ------------------------------------
        ' No la cumplo porque el constructor ya lo hace 
        ' ------------------------------------
        Dim returnValue As New Dinero(d)
        returnValue._ParteEntera = +d._ParteEntera
        Return returnValue
    End Function



    ''' <summary>
    '''   Elimina los dígitos fraccionarios del dinero. 
    '''   Devuelve la parte entera del valor del dinero, los dígitos fraccionarios se descartan.
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a redondear a la parte entera</param>
    ''' <returns>Tipo: <seealso cref="Dinero">Dinero</seealso>
    '''    Resultado de redondear <paramref name="d">d</paramref>
    '''    al número entero más próximo, hacia cero. Es decir una estructura 
    '''    <see cref="Dinero">Dinero</see> sin los decimales, 
    '''    solo con la parte entera del numero
    ''' </returns>
    ''' <remarks>    
    '''    Este método redondea <paramref name="d">d</paramref> al 
    '''    número entero más próximo, que tiende a cero, 
    '''    lo que supone descartar los dígitos situados tras el separador de decimales.
    '''</remarks>
    Public Overloads Shared Function Truncate(ByVal d As Dinero) As Dinero
        Return New Dinero(d.ValorParteEntera, d.CultureInfo)
    End Function


    ''' <summary>
    '''   Elimina los dígitos fraccionarios del dinero. 
    '''   Devuelve la parte entera del valor del dinero, 
    '''   los dígitos fraccionarios se descartan.
    ''' </summary>
    ''' <remarks>
    '''    Este método redondea <paramref name="d">d</paramref>
    '''    al número entero más próximo, que tiende a cero, lo que supone 
    '''    descartar los dígitos situados tras el separador de decimales.
    '''</remarks>
    Public Overloads Sub Truncate()
        ' cargarle la parte decimal
        Me._ParteDecimal = 0
    End Sub





#End Region


#Region "Operadores >, <, = , <>, >=, <="


    '-------------------------------------------------------------------
    ''' <summary>
    ''' Devuelve un valor que indica si un <see cref="Dinero">Dinero</see> 
    ''' especificado es mayor que otro <see cref="Dinero">Dinero</see> especificado. 
    ''' </summary>
    ''' <param name="d1"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <param name="d2"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <returns>
    '''    <para>Un valor  <see cref="System.Boolean">lógico</see> que indica:</para>
    '''    <para>True: Es true si   <paramref name="d1">d1</paramref> 
    '''                es mayor que <paramref name="d2">d2</paramref>
    ''' </para>
    '''    <para>False: en caso contrario</para>
    ''' </returns>
    Public Overloads Shared Operator >( _
                                ByVal d1 As Dinero, _
                                ByVal d2 As Dinero) _
                                As Boolean
        Call ComprobacionMismaMoneda(d1, d2)
        Return d1.Valor > d2.Valor
    End Operator

    '-------------------------------------------------------------------
    ''' <summary>
    ''' Devuelve un valor que indica si un <see cref="Dinero">Dinero</see> 
    ''' especificado es menor que otro <see cref="Dinero">Dinero</see> especificado. 
    ''' </summary>
    ''' <param name="d1"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <param name="d2"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <returns>
    '''    <para>Un valor  <see cref="System.Boolean">lógico</see> que indica:</para>
    '''    <para>True: Es true si   <paramref name="d1">d1</paramref> 
    '''                es menor que <paramref name="d2">d2</paramref>
    ''' </para>
    '''    <para>False: en caso contrario</para>
    ''' </returns>
    Public Overloads Shared Operator <( _
                           ByVal d1 As Dinero, _
                           ByVal d2 As Dinero) _
                           As Boolean
        Call ComprobacionMismaMoneda(d1, d2)
        Return d1.Valor < d2.Valor
    End Operator

    '-------------------------------------------------------------------
    ''' <summary>
    ''' Devuelve un valor que indica si un <see cref="Dinero">Dinero</see> 
    ''' especificado es menor o igual que otro <see cref="Dinero">Dinero</see> especificado. 
    ''' </summary>
    ''' <param name="d1"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <param name="d2"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <returns>
    '''    <para>Un valor  <see cref="System.Boolean">lógico</see> que indica:</para>
    '''    <para>True: Es true si   <paramref name="d1">d1</paramref> 
    '''                es menor o igual que <paramref name="d2">d2</paramref>
    ''' </para>
    '''    <para>False: en caso contrario</para>
    ''' </returns>
    Public Overloads Shared Operator <=( _
                            ByVal d1 As Dinero, _
                            ByVal d2 As Dinero) _
                            As Boolean
        Call ComprobacionMismaMoneda(d1, d2)
        Return d1.Valor <= d2.Valor
    End Operator

    '-------------------------------------------------------------------
    ''' <summary>
    ''' Devuelve un valor que indica si un <see cref="Dinero">Dinero</see> 
    ''' especificado es mayor o igual que otro <see cref="Dinero">Dinero</see> especificado. 
    ''' </summary>
    ''' <param name="d1"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <param name="d2"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <returns>
    '''    <para>Un valor  <see cref="System.Boolean">lógico</see> que indica:</para>
    '''    <para>True: Es true si   <paramref name="d1">d1</paramref> 
    '''                es mayor o igual que <paramref name="d2">d2</paramref>
    ''' </para>
    '''    <para>False: en caso contrario</para>
    ''' </returns>
    Public Overloads Shared Operator >=( _
                            ByVal d1 As Dinero, _
                            ByVal d2 As Dinero) _
                            As Boolean
        Call ComprobacionMismaMoneda(d1, d2)
        Return d1.Valor >= d2.Valor
    End Operator

    '-------------------------------------------------------------------
    ' Regla FxCop
    ' Provide a method named 'Equals' as a friendly 
    ' alternate for operator 'Dinero.Operator =(Dinero, Dinero)
    '------------------------------------------------------------
    '------------------------------------------------------------
    '-------------------------------------------------------------------
    ''' <summary>
    ''' Devuelve un valor que indica si un <see cref="Dinero">Dinero</see> 
    ''' especificado es igual que otro <see cref="Dinero">Dinero</see> especificado. 
    ''' </summary>
    ''' <param name="d1"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <param name="d2"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <returns>
    '''    <para>Un valor  <see cref="System.Boolean">lógico</see> que indica:</para>
    '''    <para>True: Es true si   <paramref name="d1">d1</paramref> 
    '''                es igual que <paramref name="d2">d2</paramref>
    ''' </para>
    '''    <para>False: en caso contrario</para>
    ''' </returns>
    ''' <remarks>
    ''' <para>
    ''' Regla FxCop
    ''' Provide a method named 'Equals' as a friendly 
    ''' alternate for operator 'Dinero.Operator =(Dinero, Dinero)
    '''</para>
    ''' <para> Referencia MSDN
    ''' <a href="http://msdn.microsoft.com/es-es/library/7h9bszxx(VS.71).aspx">
    '''   http://msdn.microsoft.com/es-es/library/7h9bszxx(VS.71).aspx
    ''' </a>
    '''</para>
    ''' </remarks>
    Public Overloads Shared Operator =( _
                                   ByVal d1 As Dinero, _
                                   ByVal d2 As Dinero) _
                                   As Boolean
        Return Equals(d1, d2)
    End Operator



    '-------------------------------------------------------------------
    ''' <summary>
    ''' Devuelve un valor que indica si un <see cref="Dinero">Dinero</see> 
    ''' especificado es distinto que otro <see cref="Dinero">Dinero</see> especificado. 
    ''' </summary>
    ''' <param name="d1"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <param name="d2"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <returns>
    '''    <para>Un valor  <see cref="System.Boolean">lógico</see> que indica:</para>
    '''    <para>True: Es true si   <paramref name="d1">d1</paramref> 
    '''                es distinto que <paramref name="d2">d2</paramref>
    ''' </para>
    '''    <para>False: en caso contrario</para>
    ''' </returns>
    Public Overloads Shared Operator <>( _
                             ByVal d1 As Dinero, _
                             ByVal d2 As Dinero) _
                             As Boolean
        Return Not (Equals(d1, d2))
    End Operator



#End Region


#Region "Interface ICloneable"


    ''' <summary>
    '''  único método de la interfaz ICloneable (Privado)
    ''' </summary>
    ''' <returns> Un objeto con los valores de la isntancia copiados</returns>
    ''' <remarks></remarks>
    Private Function ClonaMe() As Object Implements ICloneable.Clone
        ' Apuntar al método Clone
        Return Me.Clone
    End Function

    ''' <summary>
    '''  Crea una copia Profunda del objeto 
    ''' </summary>
    ''' <returns>
    '''     Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''     Un objeto Dinero con los valores de la isntancia copiados
    ''' </returns>
    Public Function Clone() As Dinero
        ' llamar al metodo generico que realiza la conacion
        Return Clone(Me)
    End Function



    '''---------------------------------------------------------------------------
    ''' <summary>Crea una copia Profunda del objeto </summary>
    ''' <typeparam name="T">El Objeto que se va a clonar</typeparam>
    ''' <returns>Un nuevo objeto con los valores copiados</returns>
    ''' <remarks>Es una función Genérica</remarks>
    Private Shared Function Clone(Of T)(ByVal objAClonar As T) As T
        Dim ms As System.IO.MemoryStream = Nothing
        Dim bf As System.Runtime.Serialization.Formatters.Binary.BinaryFormatter = Nothing

        Try
            ' crear una secuencia en memoria
            ms = New System.IO.MemoryStream(2048)
            bf = New System.Runtime.Serialization.Formatters.Binary.BinaryFormatter( _
                     Nothing, _
                     New System.Runtime.Serialization.StreamingContext( _
                                Runtime.Serialization.StreamingContextStates.Clone))

            ' serializar el objeto en la memoria
            bf.Serialize(ms, objAClonar)
            ' situar el puntero en el primer byte de la secuencia
            ms.Seek(0, System.IO.SeekOrigin.Begin)
            ' eliminar la serializacion 
            Dim ObjNuevo As T
            ObjNuevo = CType(bf.Deserialize(ms), T)
            ' liberar memoria
            ' En finally
            ' devolver el nuevo objeto clonado 
            Return ObjNuevo
        Catch ex As Exception
            Throw
        Finally
            If Not (ms Is Nothing) Then
                ' cerrar la secuencia MemoryStream
                ms.Close()
                ' Comprobar si se puede llamar a la funcion dipose del objeto
                If TypeOf ms Is IDisposable Then
                    ms.Dispose()
                End If
                ' objeto a null
                ms = Nothing
            End If
            If Not (ms Is Nothing) Then
                ' objeto a null
                ms = Nothing
            End If
        End Try
    End Function



    '''' <summary>
    ''''  Crea una copia Profunda del objeto 
    '''' </summary>
    '''' <returns>
    ''''     Tipo: <seealso cref="Dinero">Dinero</seealso> 
    ''''     Un objeto Dinero con los valores de la isntancia copiados
    '''' </returns>
    'Public Function Clone() As Dinero
    '    Dim ms As System.IO.MemoryStream = Nothing
    '    Dim bf As System.Runtime.Serialization.Formatters.Binary.BinaryFormatter = Nothing

    '    Try
    '        ' crear una secuencia en memoria
    '        ms = New System.IO.MemoryStream(1000)
    '        bf = New System.Runtime.Serialization.Formatters.Binary.BinaryFormatter( _
    '                       Nothing, _
    '                       New System.Runtime.Serialization.StreamingContext( _
    '                                  Runtime.Serialization.StreamingContextStates.Clone))

    '        ' serializar el objeto en la memoria
    '        bf.Serialize(ms, Me)
    '        ' situar el puntero en el primer byte de la secuencia
    '        ms.Seek(0, System.IO.SeekOrigin.Begin)
    '        ' eliminar la serializacion 
    '        Dim ObjNuevo As Dinero
    '        ObjNuevo = CType(bf.Deserialize(ms), Dinero)
    '        ' liberar memoria
    '        ' En finally
    '        ' devolver el nuevo objeto clonado toalmente
    '        Return ObjNuevo
    '    Catch ex As Exception
    '        Throw
    '    Finally
    '        If Not (ms Is Nothing) Then
    '            ' cerrar la secuencia MemoryStream
    '            ms.Close()
    '            ' Comprobar si se puede llamar a la funcion dipose del objeto
    '            If TypeOf ms Is IDisposable Then
    '                ms.Dispose()
    '            End If
    '            ' objeto a null
    '            ms = Nothing
    '        End If
    '        If Not (ms Is Nothing) Then
    '            ' objeto a null
    '            ms = Nothing
    '        End If
    '    End Try
    'End Function




    ''' <summary>
    ''' Crea una copia superficial del objeto 
    ''' <seealso cref="system.Object">Object</seealso> actual.
    ''' </summary>
    ''' <returns>Tipo: <seealso cref="System.Object">System.Object</seealso> 
    '''         Copia superficial del objeto 
    '''         <seealso cref="system.Object">Object</seealso> actual.
    ''' </returns>
    ''' <remarks>
    ''' El método MemberwiseClone crea una copia superficial mediante la creación 
    ''' de un nuevo objeto y la copia posterior de los campos no estáticos del 
    ''' objeto actual en el objeto nuevo. Si un campo es un tipo de valor, se 
    ''' realiza una copia bit a bit del campo. Si un campo es un tipo de 
    ''' referencia, se copia la referencia pero no el objeto al que se hace 
    ''' referencia; por consiguiente, el objeto original y su copia hacen 
    ''' referencia al mismo objeto.
    ''' Por ejemplo, considere un objeto denominado X que hace referencia a los 
    ''' objetos A y B. El objeto B, a su vez, hace referencia al objeto C. 
    ''' Una copia superficial de X crea nuevo objeto X2 que también hace referencia 
    ''' a los objetos A y B. Por contraposición, una copia profunda de X crea un 
    ''' nuevo objeto X2 que hace referencia a los nuevos objetos A2 y B2, que son 
    ''' copias de A y B. A su vez, B2 hace referencia al nuevo objeto C2, que es 
    ''' una copia de C. Utilice una clase que implemente la interfaz 
    ''' <seealso cref="System.ICloneable">System.ICloneable</seealso>  
    ''' para realizar una copia profunda o superficial de un objeto. 
    ''' </remarks>
    Public Overloads Function MemberwiseClone() As Object
        Return New Dinero(Me.Valor, Me.CultureInfo)
    End Function

#End Region

#Region " Implements IEquatable(Of Dinero)"


    '-------------------------------------------------------------------
    ' referencia MSDN
    ' http://msdn.microsoft.com/es-es/library/7h9bszxx(VS.71).aspx
    ' Implemente el método GetHashCode siempre que implemente el método Equals. 
    ' De este modo, los métodos Equals y GetHashCode se mantienen sincronizados.
    '
    ' Reemplace el método Equals siempre que implemente el operador (==), y haga 
    ' que los dos realicen las mismas acciones. Esto permite que el código de 
    ' infraestructuras como Hashtable y ArrayList, que utilizan el método Equals, 
    ' se comporten del mismo modo que el código escrito por el usuario con el operador (==).
    '
    ' Reemplace el método Equals siempre que implemente la Interfaz IComparable.
    ' No inicie excepciones desde los métodos Equals o GetHashCode 
    ' ni desde el operador de igualdad (==).
    '-------------------------------------------------------------------
    ' Regla FxCop
    ' Provide a method named 'Equals' as a friendly 
    ' alternate for operator 'Dinero.Operator =(Dinero, Dinero)
    '-------------------------------------------------------------------


    '------------------------------------------------------------
    'Determina si dos objetos son iguales.

    ''' <summary>
    '''     Devuelve un valor que indica si esta instancia de la estructura 
    '''     <see cref="Dinero">Dinero</see>
    '''     y el <see cref="System.Object">Object</see>
    '''     recibido por parámetro tienen el mismo valor
    ''' </summary>
    ''' <param name="d"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <returns>
    '''    <para>Un valor  <see cref="System.Boolean">lógico</see> que indica:</para>
    '''    <para>True:  Si son Iguales</para>
    '''    <para>False: en caso contrario</para>
    ''' </returns>
    ''' <remarks>
    ''' <para>
    ''' Regla FxCop
    ''' Provide a method named 'Equals' as a friendly 
    ''' alternate for operator 'Dinero.Operator =(Dinero, Dinero)
    '''</para>
    ''' <para> Referencia MSDN
    ''' <a href="http://msdn.microsoft.com/es-es/library/7h9bszxx(VS.71).aspx">
    '''   http://msdn.microsoft.com/es-es/library/7h9bszxx(VS.71).aspx
    ''' </a>
    '''</para>
    ''' <para>
    ''' Implemente el método  <see cref="System.Object.GetHashCode">GetHashCode</see>
    '''  siempre que implemente el método <see cref="System.Object.Equals">Equals</see>. 
    '''  De este modo, los métodos Equals y GetHashCode se mantienen sincronizados.
    '''</para>
    ''' <para>
    ''' Reemplace el método Equals siempre que implemente el operador (==), y haga 
    ''' que los dos realicen las mismas acciones. Esto permite que el código de 
    ''' infraestructuras como Hashtable y ArrayList, que utilizan el método Equals, 
    ''' se comporten del mismo modo que el código escrito por el usuario con el operador (==).
    '''</para>
    ''' <para>
    ''' Reemplace el método Equals siempre que implemente la Interfaz IComparable.
    ''' No inicie excepciones desde los métodos Equals o GetHashCode 
    ''' ni desde el operador de igualdad (==).
    '''</para>
    ''' </remarks>
    Public Overloads Function Equals(ByVal d As Dinero) As Boolean _
                 Implements System.IEquatable(Of Dinero).Equals
        Return Equals(Me, d)
    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    '''     Devuelve un valor que indica si esta instancia de la estructura 
    '''     <see cref="Dinero">Dinero</see>
    '''     y el <see cref="System.Object">Object</see>
    '''     recibido por parámetro tienen el mismo valor
    ''' </summary>
    ''' <param name="obj">Tipo <see cref="System.Object">Object</see></param>
    ''' <returns>
    '''    <para>Un valor  <see cref="System.Boolean">lógico</see> que indica:</para>
    '''    <para>True:  Si son Iguales</para>
    '''    <para>False: en caso contrario</para>
    ''' </returns>
    ''' <remarks>
    ''' <para>
    ''' Regla FxCop
    ''' Provide a method named 'Equals' as a friendly 
    ''' alternate for operator 'Dinero.Operator =(Dinero, Dinero)
    '''</para>
    ''' <para> Referencia MSDN
    ''' <a href="http://msdn.microsoft.com/es-es/library/7h9bszxx(VS.71).aspx">
    '''   http://msdn.microsoft.com/es-es/library/7h9bszxx(VS.71).aspx
    ''' </a>
    '''</para>
    ''' <para>
    ''' Implemente el método  <see cref="System.Object.GetHashCode">GetHashCode</see>
    '''  siempre que implemente el método <see cref="System.Object.Equals">Equals</see>. 
    '''  De este modo, los métodos Equals y GetHashCode se mantienen sincronizados.
    '''</para>
    ''' <para>
    ''' Reemplace el método Equals siempre que implemente el operador (==), y haga 
    ''' que los dos realicen las mismas acciones. Esto permite que el código de 
    ''' infraestructuras como Hashtable y ArrayList, que utilizan el método Equals, 
    ''' se comporten del mismo modo que el código escrito por el usuario con el operador (==).
    '''</para>
    ''' <para>
    ''' Reemplace el método Equals siempre que implemente la Interfaz IComparable.
    ''' No inicie excepciones desde los métodos Equals o GetHashCode 
    ''' ni desde el operador de igualdad (==).
    '''</para>
    ''' </remarks>
    Public Overloads Overrides Function Equals(ByVal obj As Object) As Boolean

        ' El parámetro, value, debe ser del mismo tipo que la clase o el 
        ' tipo de valor que implementa esta interfaz; en caso contrario, 
        ' se produce la excepción ArgumentException.
        If Not (obj.GetType Is Me.GetType) Then
            Return False
            ' "Los objetos a comparar no son del mismo tipo"
            '  No inicie excepciones desde los métodos Equals o GetHashCode 
            '  ni desde el operador de igualdad (==).
        End If
        '-----------------------------------------------------------
        'si llega aquí, quiere decir que es una estructura dinero
        'convertirlo a estructura dinero
        Dim unDinero As Dinero
        unDinero = CType(obj, Dinero)

        'realizar la comparacion
        Return Equals(Me, unDinero)

    End Function


    '-------------------------------------------------------------------
    ''' <summary>
    '''     Devuelve un valor que indica si dos instancias de <see cref="Dinero">Dinero</see>
    '''     representan el mismo valor (tienen el mismo valor)
    ''' </summary>
    ''' <param name="d1"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <param name="d2"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <returns>
    '''    <para>Un valor  <see cref="System.Boolean">lógico</see> que indica:</para>
    '''    <para>True: Es true si   <paramref name="d1">d1</paramref> 
    '''                es igual que <paramref name="d2">d2</paramref></para>
    '''    <para>False: en caso contrario</para>
    ''' </returns>
    ''' <remarks>
    ''' <para>
    ''' Regla FxCop
    ''' Provide a method named 'Equals' as a friendly 
    ''' alternate for operator 'Dinero.Operator =(Dinero, Dinero)
    '''</para>
    ''' <para> Referencia MSDN
    ''' <a href="http://msdn.microsoft.com/es-es/library/7h9bszxx(VS.71).aspx">
    '''   http://msdn.microsoft.com/es-es/library/7h9bszxx(VS.71).aspx
    ''' </a>
    '''</para>
    ''' <para>
    ''' Implemente el método  <see cref="System.Object.GetHashCode">GetHashCode</see>
    '''  siempre que implemente el método <see cref="System.Object.Equals">Equals</see>. 
    '''  De este modo, los métodos Equals y GetHashCode se mantienen sincronizados.
    '''</para>
    ''' <para>
    ''' Reemplace el método Equals siempre que implemente el operador (==), y haga 
    ''' que los dos realicen las mismas acciones. Esto permite que el código de 
    ''' infraestructuras como Hashtable y ArrayList, que utilizan el método Equals, 
    ''' se comporten del mismo modo que el código escrito por el usuario con el operador (==).
    '''</para>
    ''' <para>
    ''' Reemplace el método Equals siempre que implemente la Interfaz IComparable.
    ''' No inicie excepciones desde los métodos Equals o GetHashCode 
    ''' ni desde el operador de igualdad (==).
    '''</para>
    ''' </remarks>
    Public Overloads Shared Function Equals(ByVal d1 As Dinero, ByVal d2 As Dinero) As Boolean
        ' si no son la misma moneda, no son iguales
        ' si son la misma moneda mirar las cantidades
        If d1.MonedaIsoCurrencySymbol = d2.MonedaIsoCurrencySymbol Then
            Return (d1.Valor = d2.Valor)
        Else
            Return False
        End If
    End Function


#End Region


#Region "GetHashCode"

    '------------------------------------------------------------
    ' referencia MSDN
    ' http://msdn.microsoft.com/es-es/library/7h9bszxx(VS.71).aspx
    ' Implemente el método GetHashCode siempre que implemente el método Equals. 
    ' De este modo, los métodos Equals y GetHashCode se mantienen sincronizados.

    ''' <summary>
    '''  Devuelve el código hash de esta instancia. 
    ''' </summary>
    ''' <returns>Código hash de un entero de 32 bits con signo.</returns>
    Public Overrides Function GetHashCode() As Integer
        'http://msdn.microsoft.com/es-es/library/7h9bszxx(VS.71).aspx
        'No inicie excepciones desde los métodos Equals o GetHashCode 
        'ni desde el operador de igualdad (==).
        Return CType(Math.Pow(Me.Valor.GetHashCode, Me.CultureInfo.GetHashCode), Integer)
    End Function

#End Region


#Region "Implements System.IComparable.CompareTo"



    ''' <summary>
    '''      Compara dos valores Dinero especificados.
    ''' </summary>
    ''' <param name="d1"><see cref="Dinero">Dinero</see>. - 
    ''' La estructura que contiene una cantidad de Dinero.</param>
    ''' <param name="d2"><see cref="Dinero">Dinero</see>. - 
    '''  La estructura que contiene una cantidad de Dinero.</param>
    ''' <returns>
    '''    <para> Tipo: <seealso cref="System.Int32">Integer</seealso> 
    '''           Número con signo que indica los valores relativos de d1 y d2.</para>
    '''    <para> Menor que cero:  d1 es menor que d2. </para>
    '''    <para> Cero:            d1 y d2 son iguales. </para>
    '''    <para> Mayor que cero:  d1 es mayor que d2. </para>
    ''' </returns>
    ''' <remarks>
    ''' <para>
    ''' Por definición, cualquier objeto es mayor que una referencia de 
    ''' objeto null (Nothing en Visual Basic) y dos referencias nulas son
    ''' iguales entre sí. 
    ''' </para>
    ''' <para>El parámetro, obj, debe ser del mismo tipo que 
    ''' la clase o el tipo de valor que implementa esta interfaz; en caso 
    ''' contrario, se produce la excepción ArgumentException.
    ''' </para>
    ''' <para>Notas para los implementadores Para los objetos A, B y C, lo 
    ''' siguiente debe ser verdadero: A.CompareTo(A) debe devolver cero.
    ''' </para> 
    ''' <para> 
    ''' Si A.CompareTo(B) devuelve cero, entonces B.CompareTo(A) debe 
    ''' devolver cero. Si A.CompareTo(B) devuelve cero y B.CompareTo(C) 
    ''' devuelve cero, entonces A.CompareTo(C) debe devolver cero.
    ''' </para> 
    ''' <para> 
    ''' Si A.CompareTo(B) devuelve un valor distinto de cero, entonces 
    ''' B.CompareTo(A) debe devolver un valor del signo contrario. 
    ''' </para> 
    ''' <para> 
    '''  Si A.CompareTo(B) devuelve un valor x distinto de cero y 
    ''' B.CompareTo(C) devuelve un valor y del mismo signo que x, 
    ''' entonces A.CompareTo(C) debe devolver un valor del mismo 
    ''' signo que x e y.
    ''' </para> 
    ''' <para> 
    '''  Notas para los llamadores Utilice el método 
    ''' CompareTo para determinar el orden de instancias de una clase. 
    '''  </para>
    '''</remarks>
    ''' <exception cref="ArgumentException">
    ''' obj no es del mismo tipo que esta instancia.
    ''' </exception>
    Public Shared Function Compare(ByVal d1 As Dinero, ByVal d2 As Dinero) As Integer


        '---------------------------
        ' poder comparar los dos objetos tienen que ser de la misma moneda
        ' si no error
        Call ComprobacionMismaMoneda(d1, d2)

        '---------------------------
        ' Por definición, cualquier objeto es mayor que una referencia de 
        ' objeto null (Nothing en Visual Basic) y dos referencias nulas son iguales entre sí.
        '---------------------------
        ' Valores Nothing 
        If Object.ReferenceEquals(d1, Nothing) AndAlso _
           Not (Object.ReferenceEquals(d2, Nothing)) Then Return -1

        If Object.ReferenceEquals(d2, Nothing) AndAlso _
           Object.ReferenceEquals(d1, Nothing) Then Return 0

        If Not (Object.ReferenceEquals(d1, Nothing)) AndAlso _
           Object.ReferenceEquals(d2, Nothing) Then Return +1

        ' son la misma referencia (es el mimsmo objeto)
        If Object.ReferenceEquals(d2, d1) = True Then Return 0

        '-------------------------------------------------
        ' resto de valores
        ' son la misma moneda, comparo cantidades
        Dim ValorOther As Decimal = d2.ToDecimal(d2.CultureInfo)
        Dim ValorMe As Decimal = d1.ToDecimal(d1.CultureInfo)

        If ValorMe < ValorOther Then Return -1
        If ValorMe = ValorOther Then Return 0
        If ValorMe > ValorOther Then Return +1

    End Function




    ''' <summary>
    '''     Compara la instancia actual con otro objeto del mismo tipo. 
    ''' </summary>
    ''' <param name="obj">Objeto que se va a comparar con esta instancia. </param>
    ''' <returns>
    '''    <para> Tipo: <seealso cref="System.Int32">Integer</seealso> 
    '''           Número con signo que indica los valores relativos de esta instancia
    '''           y <paramref name="obj">obj</paramref>.</para>
    '''    <para> Menor que cero:  d1 es menor que d2. </para>
    '''    <para> Cero:            d1 y d2 son iguales. </para>
    '''    <para> Mayor que cero:  d1 es mayor que d2. </para>
    ''' </returns>
    Public Overloads Function CompareTo(ByVal obj As Object) As Integer _
           Implements System.IComparable.CompareTo

        ' El parámetro, obj, debe ser del mismo tipo que la clase o el 
        ' tipo de valor que implementa esta interfaz; en caso contrario, 
        ' se produce la excepción ArgumentException.
        If Not (obj.GetType Is Me.GetType) Then
            Throw New ArgumentException( _
            "Los objetos a comparar no son del mismo tipo")
        End If


        '-----------------------------------------------------------
        'si llega aquí, quiere decir que es una estructura dinero
        'si no lo fuera se habría disparado la excepción
        '--
        'convertirlo a estructura dinero

        Dim unaEstrucDinero As Dinero
        unaEstrucDinero = CType(obj, Dinero)

        ' realizar la comparacion
        '    <para> Menor que cero:  d1 es menor que d2. </para>
        '    <para> Cero:            d1 y d2 son iguales. </para>
        '    <para> Mayor que cero:  d1 es mayor que d2. </para>
        '----------------------
        ' Menor que cero  -  ME instancia es menor que value. 
        ' Cero            -  ME instancia es igual a value. 
        ' Mayor que cero  -  ME instancia es mayor que value.

        Return Compare(Me, unaEstrucDinero)

    End Function

#End Region

#Region " Implements IComparable(Of EstructuraDinero)"


    ''' <summary>
    '''     Compara la instancia actual con otro objeto del mismo tipo. 
    ''' </summary>
    ''' <param name="other">Objeto que se va a comparar con esta instancia. </param>
    ''' <returns>
    '''    <para> Tipo: <seealso cref="System.Int32">Integer</seealso> 
    '''           Número con signo que indica los valores relativos de esta instancia
    '''           y <paramref name="obj">obj</paramref>.</para>
    '''    <para> Menor que cero:  d1 es menor que d2. </para>
    '''    <para> Cero:            d1 y d2 son iguales. </para>
    '''    <para> Mayor que cero:  d1 es mayor que d2. </para>
    ''' </returns>
    Public Overloads Function CompareTo(ByVal other As Dinero) As Integer _
           Implements System.IComparable(Of Dinero).CompareTo

        ' realizar la comparacion
        '    <para> Menor que cero:  d1 es menor que d2. </para>
        '    <para> Cero:            d1 y d2 son iguales. </para>
        '    <para> Mayor que cero:  d1 es mayor que d2. </para>
        '----------------------
        ' Menor que cero  -  ME instancia es menor que value. 
        ' Cero            -  ME instancia es igual a value. 
        ' Mayor que cero  -  ME instancia es mayor que value.

        Return Compare(Me, other)

    End Function

#End Region

#Region "Operaciones de conversion de datos y tipos"



    '-------------------------------------------------------------------------------
    'Referencia general de .NET Framework
    'http://msdn.microsoft.com/es-es/library/cc433243(VS.71).aspx
    'Instrucciones de tipos de conversión
    'En las reglas siguientes se describen las instrucciones de uso de conversiones:  
    '
    'No permita conversiones implícitas que den como resultado una pérdida de precisión.  
    'Por ejemplo, no debe haber una conversión implícita de Double a Int32,  
    'pero puede haber una de Int32 a Int64.  
    '
    'No inicie excepciones en conversiones implícitas porque es muy difícil  
    'que el programador comprenda lo que está pasando. 
    'Proporcione conversiones que funcionen en todo el objeto.  
    'El valor que se convierte debe representar todo el objeto,  
    'no un miembro de un objeto. Por ejemplo, no es correcto convertir un Button  
    'en una cadena mediante la devolución de su título.  
    '
    'No genere un valor que sea semánticamente distinto. Por ejemplo, es  
    'conveniente convertir un valor DateTime o TimeSpan a un valor Int32.  
    'El valor Int32 sigue representando el tiempo o la duración.  
    'No tiene sentido, sin embargo convertir una cadena de nombre de  
    'archivo como "c:\mybitmap.gif" a un objeto Bitmap.  
    '
    'No convierta valores de dominios diferentes. Las conversiones funcionan  
    'dentro de un dominio específico de valores. Por ejemplo, los números y  
    'las cadenas pertenecen a distintos dominios. Tiene sentido convertir  
    'un valor Int32 a Double. Sin embargo, no tiene sentido convertir un  
    'valor Int32 a String, ya que estos dos valores se encuentran en  
    'dominios diferentes. 
    '------------------------------------------------------------------------------




#Region "Parse"


    '''----------------------------------------------------------------------------
    ''' <summary>
    '''      Convierte la representación de tipo 
    '''      <seealso cref="System.String">String</seealso> de un número en su 
    '''      <seealso cref="System.Decimal">Decimal</seealso> equivalente    
    '''  </summary>
    '''  <param name="s"> 
    '''      <seealso cref="System.String">String</seealso> 
    '''      Que contiene un número que se va a convertir.
    ''' </param>
    ''' <returns>Una estructura <seealso cref="Dinero">dinero</seealso> 
    '''          equivalente al número contenido en <paramref name="s">s</paramref>.
    ''' </returns>
    ''' <remarks>
    ''' <para>El parámetro <paramref name="s">s</paramref> contiene un número con formato: </para>
    ''' <para>[eb][signo]dígitos[.dígitos fraccionarios][eb] </para>
    ''' <para>Los elementos que aparecen entre corchetes ('[' y ']') 
    '''       son opcionales y el resto de elementos son como se indica a continuación.</para>
    ''' <para>eb - Espacio en blanco opcional. </para>
    ''' <para>signo - Signo opcional. </para>
    ''' <para>dígitos -Secuencia de dígitos comprendidos entre el 0 y el 9. </para>
    ''' <para>'.' - Símbolo separador de decimales específico de la referencia cultural. </para>
    ''' <para>dígitos fraccionarios - Secuencia de dígitos comprendidos entre el 0 y el 9. </para>
    ''' <para>El parámetro <paramref name="s">s</paramref> se interpreta utilizando el estilo 
    '''       <seealso cref="System.Globalization.NumberStyles.Number">NumberStyles.Number</seealso>.
    ''' </para>
    ''' <para>El parámetro <paramref name="s">s</paramref> se analiza utilizando la información 
    '''       de formato de NumberFormatInfo que se inicializa para la referencia cultural 
    '''       del sistema actual. Para obtener más información, vea
    '''       <seealso cref="System.Globalization.CultureInfo">CurrentInfo</seealso> .
    ''' </para>
    ''' <para>Si es necesario, el valor de <paramref name="s">s</paramref> se redondea 
    '''       mediante el redondeo al más próximo.</para>
    ''' <para>Decimal tiene 29 dígitos de precisión. Si <paramref name="s">s</paramref> 
    '''       representa un número de más de 29 dígitos pero tiene una parte fraccionaria y 
    '''       se encuentra comprendido dentro del intervalo de 
    '''      <seealso cref="System.Decimal.MaxValue">MaxValue</seealso>
    '''      y <seealso cref="System.Decimal.MinValue">MinValue</seealso>, el número 
    '''      no se trunca, sino que se redondea a 29 dígitos utilizando el redondeo 
    '''      al más próximo.
    ''' </para>
    ''' <para>Si, durante una operación de análisis, se encuentra un separador en el  
    '''       parámetro <paramref name="s">s</paramref> y los separadores aplicables 
    '''       de moneda, número decimal y grupo son iguales, la operación de análisis 
    '''       supone que se trata de un separador decimal en lugar de un separador de
    '''       grupo. Para obtener más información sobre separadores, vea 
    '''       <seealso cref="System.Globalization.NumberFormatInfo.CurrencyDecimalSeparator">
    '''       CurrencyDecimalSeparator</seealso>, 
    '''       <seealso cref="System.Globalization.NumberFormatInfo.NumberDecimalSeparator">
    '''       NumberDecimalSeparator</seealso>, 
    '''       <seealso cref="System.Globalization.NumberFormatInfo.CurrencyGroupSeparator">
    '''       CurrencyGroupSeparator</seealso> y 
    '''       <seealso cref="System.Globalization.NumberFormatInfo.NumberGroupSeparator">
    '''       NumberGroupSeparator</seealso>.
    ''' </para>
    '''</remarks>
    ''' <exception cref="ArgumentNullException">El valor de <paramref name="s">s</paramref> 
    '''            es null (Nothing en Visual Basic). </exception>
    ''' <exception cref="FormatException"><paramref name="s">s</paramref> 
    '''            no tiene el formato correcto. </exception>
    ''' <exception cref="OverflowException"><paramref name="s">s</paramref> 
    '''            representa un número menor que 
    '''            <seealso cref="System.Decimal.MinValue">MinValue</seealso> o mayor que 
    '''            <seealso cref="System.Decimal.MaxValue">MaxValue</seealso>. 
    ''' </exception>
    Public Overloads Shared Function Parse(ByVal s As String) As Dinero
        Try
            s = CorregirCadenaConNumeroToCulturaEspañola(s)
            s = LimitarNueveDecimales(s)

            Dim numero As Decimal
            numero = Decimal.Parse(s)
            ' ------------------------------------
            ' regla FxCop
            '   replace this call with a 
            '   call to 'Dinero.New(d, CultureInfo)'. 
            ' ------------------------------------
            ' No la cumplo porque el constructor ya lo hace 
            ' ------------------------------------
            Dim unDinero As New Dinero(numero)
            Return unDinero

        Catch ex As Exception
            Throw
        End Try
    End Function




    ''' <summary>
    '''   Convierte la representación de tipo 
    '''   <seealso cref="System.String">String</seealso>  de un número en su 
    '''   <seealso cref="System.Decimal">Decimal</seealso> 
    '''   equivalente, utilizando el estilo de formato especificado.
    ''' </summary>
    ''' <param name="s"><seealso cref="System.String">String</seealso> 
    '''  que contiene un número que se va a convertir. </param>
    ''' <param name="style"><seealso cref="System.Globalization.NumberStyles">
    '''    System.Globalization.NumberStyles</seealso> 
    '''    Una combinación bit a bit de valores NumberStyles que indica el formato 
    '''    permitido de <paramref name="s">s</paramref>. 
    '''    Un valor que se especifica de forma habitual es 
    '''    <seealso cref="System.Globalization.NumberStyles.Number">Number</seealso>  
    ''' </param>
    ''' <returns>Una estructura <seealso cref="Dinero">dinero</seealso> 
    '''          equivalente al número contenido en <paramref name="s">s</paramref>.
    ''' </returns>
    ''' <remarks>
    ''' <para>El parámetro <paramref name="s">s</paramref> contiene un número con formato: </para>
    ''' <para>[eb][signo]dígitos[.dígitos fraccionarios][eb] </para>
    ''' <para>Los elementos que aparecen entre corchetes ('[' y ']')
    '''        son opcionales y el resto de elementos son como se indica a continuación.</para>
    ''' <para>eb - Espacio en blanco opcional. </para>
    ''' <para>signo - Signo opcional. </para>
    ''' <para>dígitos -Secuencia de dígitos comprendidos entre el 0 y el 9. </para>
    ''' <para>'.' - Símbolo separador de decimales específico de la referencia cultural. </para>
    ''' <para>dígitos fraccionarios - Secuencia de dígitos comprendidos entre el 0 y el 9. </para>
    ''' <para>El parámetro <paramref name="s">s</paramref> se interpreta utilizando el 
    '''       estilo <seealso cref="System.Globalization.NumberStyles.Number">
    ''' NumberStyles.Number</seealso>.
    ''' </para>
    ''' <para> El parámetro <paramref name="style">style</paramref> especifica el 
    ''' formato permitido de la cadena numérica. Debe ser una combinación de 
    ''' indicadores de bit de la enumeración <seealso cref="System.Globalization.NumberStyles">
    ''' NumberStyles</seealso>. Si <paramref name="style">style</paramref> es null, 
    ''' <paramref name="s">s</paramref> se interpreta utilizando el estilo 
    ''' <seealso cref="System.Globalization.NumberStyles.Number">NumberStyles.Number</seealso>.
    ''' </para>
    ''' <para>El parámetro <paramref name="s">s</paramref> se analiza utilizando 
    ''' la información de formato de NumberFormatInfo que se inicializa para la 
    ''' referencia cultural del sistema actual. Para obtener más información, vea 
    ''' <seealso cref="System.Globalization.CultureInfo">CurrentInfo</seealso> .
    ''' </para>
    ''' <para>Si es necesario, el valor de <paramref name="s">s</paramref> 
    '''       se redondea mediante el redondeo al más próximo.</para>
    ''' <para>
    '''     Decimal tiene 29 dígitos de precisión. Si s representa un número de más de 
    '''     29 dígitos pero tiene una parte fraccionaria y se encuentra comprendido dentro 
    '''     del intervalo de 
    '''     <seealso cref="System.Decimal.MaxValue">MaxValue</seealso>
    '''     y <seealso cref="System.Decimal.MinValue">MinValue</seealso>, 
    '''    el número no se trunca, sino que se redondea a 29 dígitos utilizando
    '''    el redondeo al más próximo.
    ''' </para>
    ''' <para>
    '''  Si, durante una operación de análisis, se encuentra un separador en el 
    ''' parámetro <paramref name="s">s</paramref>  y los separadores aplicables de
    '''  moneda, número decimal y grupo son iguales, la operación de análisis supone 
    ''' que se trata de un separador decimal en lugar de un separador de grupo. 
    ''' Para obtener más información sobre separadores, vea 
    '''    <seealso cref="System.Globalization.NumberFormatInfo.CurrencyDecimalSeparator">CurrencyDecimalSeparator</seealso>, 
    '''    <seealso cref="System.Globalization.NumberFormatInfo.NumberDecimalSeparator">NumberDecimalSeparator</seealso>, 
    '''    <seealso cref="System.Globalization.NumberFormatInfo.CurrencyGroupSeparator">CurrencyGroupSeparator</seealso> y 
    '''    <seealso cref="System.Globalization.NumberFormatInfo.NumberGroupSeparator">NumberGroupSeparator</seealso> .
    ''' </para>
    '''</remarks>
    ''' 
    ''' <exception cref="ArgumentException">
    ''' <para>El valor <paramref name="style">style</paramref> no es un valor de 
    '''       <seealso cref="System.Globalization.NumberStyles">NumberStyles</seealso>.</para>
    ''' <para>O bien</para>
    ''' <para>El valor de <paramref name="style">style</paramref> es el valor de 
    '''       <seealso cref="System.Globalization.NumberStyles.AllowHexSpecifier">AllowHexSpecifier</seealso>.</para>
    '''</exception>
    ''' 
    ''' <exception cref="ArgumentNullException">El valor de <paramref name="s">s</paramref> es null (Nothing en Visual Basic). </exception>
    ''' <exception cref="FormatException"><paramref name="s">s</paramref> no tiene el formato correcto. </exception>
    ''' <exception cref="OverflowException"><paramref name="s">s</paramref> representa un número menor que <seealso cref="System.Decimal.MinValue">MinValue</seealso> o mayor que <seealso cref="System.Decimal.MaxValue">MaxValue</seealso>. </exception>
    Public Overloads Shared Function Parse(ByVal s As String, _
                                 ByVal style As Globalization.NumberStyles) As Dinero
        Try
            Dim numero As Decimal
            numero = Decimal.Parse(s, style)
            ' ------------------------------------
            ' regla FxCop
            '   replace this call with a 
            '   call to 'Dinero.New(d, CultureInfo)'. 
            ' ------------------------------------
            ' No la cumplo porque el constructor ya lo hace 
            ' ------------------------------------
            Dim unDinero As New Dinero(numero)
            Return unDinero
        Catch ex As Exception
            Throw
        End Try
    End Function





    ''' <summary>
    '''   Convierte la representación de tipo 
    '''   <seealso cref="System.String">String</seealso>  de un número en su 
    '''   <seealso cref="System.Decimal">Decimal</seealso> equivalente utilizando 
    '''   la información de formato específica de la referencia cultural especificada.
    ''' </summary>
    ''' <param name="s"><seealso cref="System.String">String</seealso> que contiene un número que se va a convertir. </param>
    ''' <param name="provider">Objeto 
    ''' <seealso cref="System.IFormatProvider">IFormatProvider</seealso> que proporciona información de análisis específico de la referencia cultural sobre <paramref name="s">s</paramref>. </param>
    ''' <returns>Una estructura <seealso cref="Dinero">dinero</seealso> 
    '''          equivalente al número contenido en <paramref name="s">s</paramref>.
    ''' </returns>
    ''' <remarks>
    ''' <para>El parámetro <paramref name="s">s</paramref> contiene un número con formato: </para>
    ''' <para>[eb][signo]dígitos[.dígitos fraccionarios][eb] </para>
    ''' <para>Los elementos que aparecen entre corchetes ('[' y ']') son opcionales y el resto de elementos son como se indica a continuación.</para>
    ''' <para> eb - Espacio en blanco opcional. </para>
    ''' <para> signo - Signo opcional. </para>
    ''' <para>dígitos -Secuencia de dígitos comprendidos entre el 0 y el 9. </para>
    ''' <para> '.' - Símbolo separador de decimales específico de la referencia cultural. </para>
    ''' <para> dígitos fraccionarios - Secuencia de dígitos comprendidos entre el 0 y el 9. </para>
    ''' <para>El parámetro <paramref name="s">s</paramref> se interpreta utilizando el estilo 
    ''' <seealso cref="System.Globalization.NumberStyles.Number">NumberStyles.Number</seealso>.</para>
    ''' <para>El parámetro <paramref name="s">s</paramref> se analiza utilizando la información de formato de NumberFormatInfo que se inicializa para la referencia cultural del sistema actual. Para obtener más información, vea
    ''' <seealso cref="System.Globalization.CultureInfo">CurrentInfo</seealso> .</para>
    ''' <para>Si es necesario, el valor de <paramref name="s">s</paramref> se redondea mediante el redondeo al más próximo.</para>
    ''' <para>Decimal tiene 29 dígitos de precisión. Si s representa un número de más de 29 dígitos pero tiene una parte fraccionaria y se encuentra comprendido dentro del intervalo de 
    '''  <seealso cref="System.Decimal.MaxValue">MaxValue</seealso>
    '''  y <seealso cref="System.Decimal.MinValue">MinValue</seealso>, el número no se trunca, sino que se redondea a 29 dígitos utilizando el redondeo al más próximo.</para>
    ''' <para>Si, durante una operación de análisis, se encuentra un separador en el parámetro <paramref name="s">s</paramref>  y los separadores aplicables de moneda, número decimal y grupo son iguales, la operación de análisis supone que se trata de un separador decimal en lugar de un separador de grupo. Para obtener más información sobre separadores, vea 
    ''' <seealso cref="System.Globalization.NumberFormatInfo.CurrencyDecimalSeparator">CurrencyDecimalSeparator</seealso>, 
    ''' <seealso cref="System.Globalization.NumberFormatInfo.NumberDecimalSeparator">NumberDecimalSeparator</seealso>, 
    ''' <seealso cref="System.Globalization.NumberFormatInfo.CurrencyGroupSeparator">CurrencyGroupSeparator</seealso> y 
    '''<seealso cref="System.Globalization.NumberFormatInfo.NumberGroupSeparator">NumberGroupSeparator</seealso> .</para>
    '''</remarks>
    ''' <exception cref="ArgumentNullException">El valor de <paramref name="s">s</paramref> es null (Nothing en Visual Basic). </exception>
    ''' <exception cref="FormatException"><paramref name="s">s</paramref> no tiene el formato correcto. </exception>
    ''' <exception cref="OverflowException"><paramref name="s">s</paramref> representa un número menor que <seealso cref="System.Decimal.MinValue">MinValue</seealso> o mayor que <seealso cref="System.Decimal.MaxValue">MaxValue</seealso>. </exception>
    Public Overloads Shared Function Parse(ByVal s As System.String, _
                                 ByVal provider As System.IFormatProvider) As Dinero
        Try
            Dim numero As Decimal
            numero = Decimal.Parse(s, provider)
            ' ------------------------------------
            ' regla FxCop
            '   replace this call with a 
            '   call to 'Dinero.New(d, CultureInfo)'. 
            ' ------------------------------------
            ' No la cumplo porque el constructor ya lo hace 
            ' ------------------------------------
            Dim unDinero As New Dinero(numero)
            Return unDinero
        Catch ex As Exception
            Throw
        End Try
    End Function




    ''' <summary>
    '''   Convierte la representación de tipo 
    '''   <seealso cref="System.String">String</seealso>  de un número en su 
    '''   <seealso cref="System.Decimal">Decimal</seealso> equivalente utilizando 
    '''   la información de formato específica de la referencia cultural especificada.
    ''' </summary>
    ''' <param name="s"><seealso cref="System.String">String</seealso> que contiene un número que se va a convertir. </param>
    ''' <param name="style"><seealso cref="System.Globalization.NumberStyles">System.Globalization.NumberStyles</seealso> Una combinación bit a bit de valores NumberStyles que indica el formato permitido de <paramref name="s">s</paramref>. Un valor que se especifica de forma habitual es <seealso cref="System.Globalization.NumberStyles.Number">Number</seealso>  </param>
    ''' <param name="provider">Objeto 
    ''' <seealso cref="System.IFormatProvider">IFormatProvider</seealso> que proporciona información de análisis específico de la referencia cultural sobre <paramref name="s">s</paramref>. </param>
    ''' <returns>Una estructura <seealso cref="Dinero">dinero</seealso> 
    '''          equivalente al número contenido en <paramref name="s">s</paramref>.
    ''' </returns>
    ''' <remarks>
    ''' <para>El parámetro <paramref name="s">s</paramref> contiene un número con formato: </para>
    ''' <para>[eb][signo]dígitos[.dígitos fraccionarios][eb] </para>
    ''' <para>Los elementos que aparecen entre corchetes ('[' y ']') son opcionales y el resto de elementos son como se indica a continuación.</para>
    ''' <para>eb - Espacio en blanco opcional. </para>
    ''' <para>signo - Signo opcional. </para>
    ''' <para>dígitos -Secuencia de dígitos comprendidos entre el 0 y el 9. </para>
    ''' <para>'.' - Símbolo separador de decimales específico de la referencia cultural. </para>
    ''' <para>dígitos fraccionarios - Secuencia de dígitos comprendidos entre el 0 y el 9. </para>
    ''' <para>El parámetro <paramref name="s">s</paramref> se interpreta utilizando el estilo <seealso cref="System.Globalization.NumberStyles.Number">NumberStyles.Number</seealso>.</para>
    ''' <para>El parámetro <paramref name="s">s</paramref> se analiza utilizando la información de formato de NumberFormatInfo que se inicializa para la referencia cultural del sistema actual. Para obtener más información, vea <seealso cref="System.Globalization.CultureInfo">CurrentInfo</seealso> .</para>
    ''' <para>El parámetro style puede ser una o varias constantes enumeradas NumberStyles combinadas mediante una operación OR bit a bit.</para>
    ''' <para> El parámetro <paramref name="style">style</paramref> especifica el formato permitido de la cadena numérica. Debe ser una combinación de indicadores de bit de la enumeración <seealso cref="System.Globalization.NumberStyles">NumberStyles</seealso>. Si <paramref name="style">style</paramref> es null, <paramref name="s">s</paramref> se interpreta utilizando el estilo <seealso cref="System.Globalization.NumberStyles.Number">NumberStyles.Number</seealso>.</para>
    ''' <para>El parámetro <paramref name="style">provider</paramref>  es un 
    '''     <seealso cref="System.IFormatProvider">IFormatProvider</seealso>, como un objeto 
    '''     <seealso cref="System.Globalization.NumberFormatInfo">NumberFormatInfo</seealso> o 
    '''     <seealso cref="System.Globalization.CultureInfo">CurrentInfo</seealso>. 
    ''' </para>
    '''<para>El parámetro <paramref name="style">provider</paramref> proporciona información 
    '''      específica de la referencia cultural utilizada en el análisis. 
    '''       Si provider es null (Nothing en Visual Basic), se utiliza la referencia cultural 
    '''      actual del subproceso( vease <seealso cref="System.Globalization.CultureInfo">CurrentInfo</seealso>).
    ''' </para>
    ''' <para>Si es necesario, el valor de <paramref name="s">s</paramref> 
    '''      se redondea mediante el redondeo al más próximo.</para>
    ''' <para>
    '''     Decimal tiene 29 dígitos de precisión. Si s representa un número de más de 29 dígitos 
    '''     pero tiene una parte fraccionaria y se encuentra comprendido dentro del intervalo de 
    '''     <seealso cref="System.Decimal.MaxValue">MaxValue</seealso>
    '''     y <seealso cref="System.Decimal.MinValue">MinValue</seealso>, el número no se trunca, 
    '''     sino que se redondea a 29 dígitos utilizando el redondeo al más próximo.
    ''' </para>
    ''' <para>
    '''    Si, durante una operación de análisis, se encuentra un separador en el parámetro 
    '''    <paramref name="s">s</paramref>  y los separadores aplicables de moneda, número decimal 
    '''    y grupo son iguales, la operación de análisis supone que se trata de un separador decimal 
    '''    en lugar de un separador de grupo. Para obtener más información sobre separadores, vea 
    '''    <seealso cref="System.Globalization.NumberFormatInfo.CurrencyDecimalSeparator">CurrencyDecimalSeparator</seealso>, 
    '''    <seealso cref="System.Globalization.NumberFormatInfo.NumberDecimalSeparator">NumberDecimalSeparator</seealso>, 
    '''    <seealso cref="System.Globalization.NumberFormatInfo.CurrencyGroupSeparator">CurrencyGroupSeparator</seealso> y 
    '''    <seealso cref="System.Globalization.NumberFormatInfo.NumberGroupSeparator">NumberGroupSeparator</seealso> .
    ''' </para>
    '''</remarks>
    ''' <exception cref="ArgumentException">
    ''' <para>El valor <paramref name="style">style</paramref> no es un valor de 
    '''       <seealso cref="System.Globalization.NumberStyles">NumberStyles</seealso>.</para>
    ''' <para>O bien</para>
    ''' <para>El valor de <paramref name="style">style</paramref> es el valor de 
    '''       <seealso cref="System.Globalization.NumberStyles.AllowHexSpecifier">AllowHexSpecifier</seealso>.</para>
    '''</exception>
    ''' <exception cref="ArgumentNullException">
    '''  El valor de <paramref name="s">s</paramref> es null (Nothing en Visual Basic). </exception>
    ''' <exception cref="FormatException"><paramref name="s">s</paramref> no tiene el formato correcto. </exception>
    ''' <exception cref="OverflowException"><paramref name="s">s</paramref> 
    '''    representa un número menor que <seealso cref="System.Decimal.MinValue">MinValue</seealso> 
    '''    o mayor que <seealso cref="System.Decimal.MaxValue">MaxValue</seealso>. 
    ''' </exception>
    Public Overloads Shared Function Parse( _
                        ByVal s As String, _
                        ByVal style As Globalization.NumberStyles, _
                        ByVal provider As System.IFormatProvider) As Dinero
        Try
            Dim numero As Decimal
            numero = Decimal.Parse(s, style, provider)
            ' ------------------------------------
            ' regla FxCop
            '   replace this call with a 
            '   call to 'Dinero.New(d, CultureInfo)'. 
            ' ------------------------------------
            ' No la cumplo porque el constructor ya lo hace 
            ' ------------------------------------
            Dim unDinero As New Dinero(numero)
            Return unDinero
        Catch ex As Exception
            Throw
        End Try

    End Function


#Region "Controles de formato de la cadena de texto"

    '''------------------------------------------------------------------------
    ''' <summary>
    '''    La  <seealso cref="System.String">cadena</seealso>  
    '''    de texto que contiene un número se turnca a nueve decimales
    ''' </summary>
    ''' <param name="s">
    '''    <seealso cref="System.String">String</seealso> 
    '''    Que contiene el número al que se van a dejar solo nueve decimales. 
    ''' </param>
    ''' <returns>La cadena truncada a nueve decimales</returns>
    ''' <remarks>
    ''' <para>  Ejemplo: 12,1234567890123  Resultado: 12,123456789</para>
    ''' <para>
    '''    La función comprueba que existe un separador decimal y cuenta los 
    '''    caracteres a la derecha del separador. Si hay más de nueve la trunca.
    ''' </para>
    ''' <para>
    '''    La función no comprueba que la cadena recibida sea un número correcto, 
    '''    es decir, si se recibe una la cadena 
    '''    [cadena con, una coma en medio de la frase], la función devuelve 
    '''    la cadena [cadena con, una coma]
    ''' </para>
    '''</remarks>
    Private Shared Function LimitarNueveDecimales(ByVal s As String) As String

        '-----------------------------------------------------------
        ' localizar si la cadena tiene mas de 9 decimales
        ' 12,1234567890123  -->  truncarlos a 9 -> 12,123456789
        Dim SeparadorDecimal As Char = _
           CType(System.Globalization.NumberFormatInfo.CurrentInfo.CurrencyDecimalSeparator, Char)

        Dim posicionSeparadorDecimal As Integer = 0
        posicionSeparadorDecimal = s.IndexOf(SeparadorDecimal)
        If posicionSeparadorDecimal > 0 Then
            ' hay un separador decimal
            ' si hay mas de nueve caracteres detras del separador, extarer solo nueve
            ' 0         1         2
            ' 012345678901234567890 [Posiciones]
            ' 12,1234567890123  
            ' La coma decimal esta en la posicion 2
            ' el tamaño de la cadena es de 15 caracteres
            ' 15-2 = 13 posiciones decimales
            ' extraer desde la posicion (coma+1) [3],  9 caracteres
            ' corregir para esxtraer el numero desde el principio desde posicion 0 hasta (9+(coma+1))
            If s.Length - posicionSeparadorDecimal > 9 Then
                s = s.Substring(0, (posicionSeparadorDecimal + 10))
            End If
        End If

        '---------------------------------------------------------------
        ' devolver la cadena con el numero corregido
        Return s

    End Function



    '''------------------------------------------------------------------------
    ''' <summary>
    '''   La <seealso cref="System.String">cadena</seealso>  
    '''   de texto que contiene un número se modifica para que tenga formato español [es-ES]. 
    ''' </summary>
    ''' <param name="s">
    '''    <seealso cref="System.String">String</seealso> 
    '''     Que contiene el número que se va a transformar a formato numerico español. 
    ''' </param>
    ''' <returns>La cadena que contiene un numero pero con formato numérico [es-ES] </returns>
    ''' <remarks>
    ''' <para>El parámetro <paramref name="s">s</paramref> 
    '''       debe contiener un número con el siguiente formato: </para>
    ''' <para>[eb][signo]dígitos[.dígitos fraccionarios][eb]</para>
    ''' <para>
    '''        Los elementos que aparecen entre corchetes ('[' y ']') son opcionales y el 
    '''        resto de elementos son como se indica a continuación.
    ''' </para>
    ''' <para>[eb] - Espacio en blanco opcional. </para>
    ''' <para>[signo] - Signo opcional. </para>
    ''' <para>[dígitos] -Secuencia de dígitos comprendidos entre el 0 y el 9. </para>
    ''' <para>['.'] - Símbolo separador de decimales específico de la referencia cultural. </para>
    ''' <para>[dígitos fraccionarios] - Secuencia de dígitos comprendidos entre el 0 y el 9. </para>
    ''' <para>
    ''' El proceso de corrección de los números se rige por el siguiente protocolo:
    ''' </para>
    ''' <para>
    ''' El proceso de corrección solo funciona si la cultura local tiene el mismo 
    ''' formato numérico que la cultura española (es-ES) es decir, 
    ''' una coma decimal (,) y el punto (.) de separación los millares.
    ''' </para>
    ''' <para>
    ''' El numero [1,234,567.25] y el número [1,234,567] están seguro en ingles, y 
    ''' además sin ninguna duda es decir, si un numero tiene dos o más comas y 
    ''' cero (0) o un (1) punto, está en ingles seguro y sin ninguna duda, 
    ''' y cambio su formato
    ''' </para>
    ''' <para>
    ''' Hay números en los que es más difícil saber en qué formato están y son 
    ''' los que tienen una coma y un punto por ejemplo [12.346,67] y [12,345.78], 
    ''' para averiguarlo tengo que mirar las posiciones , si la coma es la que 
    ''' está a la derecha es un numero español. Si el punto es el que está a la 
    ''' derecha es un número inglés, y cambio su formato
    ''' </para>
    ''' <para>
    ''' Cuando el número solo tiene una coma o un punto entonces también tengo la 
    ''' duda porque puede ser el número [1.200] el mil doscientos número en español, 
    ''' o el una unidad con veinte decimal en inglés. Pero, pensando en que cuando 
    ''' yo introduzco los números (por vagancia) no pulso los separadores de millar, 
    ''' y además, este código funciona siempre que el formato numérico sea el español, 
    ''' supongo que el número tiene que ser un número decimal español que esta 
    ''' introducido con el teclado numérico usando el punto del teclado en lugar de la 
    ''' coma decimal, por lo que cambio su formato
    ''' </para>
    ''' <para>
    '''    Si, durante una operación de análisis, se encuentra un caracter de moneda 
    '''    (por ejemplo, $, €) se produce un error del tipo 
    '''   <seealso cref="System.FormatException">FormatException</seealso>
    ''' </para>
    '''</remarks>
    ''' <exception cref="FormatException">
    '''            <paramref name="s">s</paramref> 
    '''            no tiene el formato correcto. 
    ''' </exception>
    Private Shared Function CorregirCadenaConNumeroToCulturaEspañola(ByVal s As String) As String

        Dim signoMas As Char = _
        CType(System.Globalization.NumberFormatInfo.CurrentInfo.PositiveSign, Char)

        Dim signoMenos As Char = _
        CType(System.Globalization.NumberFormatInfo.CurrentInfo.NegativeSign, Char)

        Dim SeparadorDecimal As Char = _
        CType(System.Globalization.NumberFormatInfo.CurrentInfo.CurrencyDecimalSeparator, Char)

        Dim SeparadorMillares As Char = _
        CType(System.Globalization.NumberFormatInfo.CurrentInfo.CurrencyGroupSeparator, Char)

        '--------------------
        'Caracter que leo en la cadena cuando hago exploraciones
        Dim unCaracter As Char


        '-----------------------------------------------------------
        '--------- Condicion ------------
        '* Tiene que tener algún carácter. Si vacía o null poner un cero [0])
        If String.IsNullOrEmpty(s) = True Then
            s = "0"
        End If


        '-----------------------------------------------------------
        '--------- Condicion ------------
        '* Si aparece una cadena del tipo [,234] convertirla en [0,234]
        If s.IndexOf(SeparadorDecimal) = 0 Then
            s = "0" & s
        End If


        '-----------------------------------------------------------
        '--------- Condicion ------------
        '* Si aparece una cadena del tipo [+,234] convertirla en [+0,234]
        If s.IndexOf(signoMas & SeparadorDecimal, _
                     System.StringComparison.OrdinalIgnoreCase) = 0 Then
            s = signoMas & "0" & _
                 s.Substring(2, (s.Length - 2))
        End If

        '-----------------------------------------------------------
        '--------- Condicion ------------
        '* Si aparece una cadena del tipo [-,234] convertirla en [-0,234]
        If s.IndexOf(signoMenos & SeparadorDecimal, _
                     System.StringComparison.OrdinalIgnoreCase) = 0 Then
            s = signoMenos & "0" & _
                 s.Substring(2, (s.Length - 2))
        End If



        '-----------------------------------------------------------
        '--------- Condicion ------------
        '* no puede haber dos puntos (..) seguidos, es un error
        If s.IndexOf( _
                     SeparadorMillares & SeparadorMillares, _
                     System.StringComparison.OrdinalIgnoreCase) <> -1 Then
            Throw New FormatException( _
            "La cadena con el número[" & s & _
            "] contiene un carácter prohibido [" & _
            SeparadorMillares & SeparadorMillares & "]")
        End If


        '-----------------------------------------------------------
        '--------- Condicion ------------
        '* no puede haber dos comas  (,,) seguidas, es un error
        If s.IndexOf( _
                                SeparadorDecimal & SeparadorDecimal, _
                                System.StringComparison.OrdinalIgnoreCase) <> -1 Then
            Throw New FormatException( _
            "La cadena con el número[" & s & _
            "] contiene un carácter prohibido [" & _
            SeparadorDecimal & SeparadorDecimal & "]")
        End If

        '-----------------------------------------------------------
        '--------- Condicion ------------
        ' * espacios SI delante y/o detras, pero NO en medio del numero
        ' primero quitar los espacios delante y detras
        s = s.Trim
        ' despues busar espacios en medio de la cadena
        If s.IndexOf(" ", 0, System.StringComparison.OrdinalIgnoreCase) <> -1 Then
            Throw New FormatException( _
            "La cadena con el número[" & s & _
            "] contiene un carácter prohibido [Un Espacio]")
        End If

        '-----------------------------------------------------------
        '--------- Condicion ------------
        ' control de signos + y -
        '* signo [+] y [-] ninguno, o solo uno de ellos
        '* Al principio de la cadena
        Dim contador As Integer = 0
        Dim caracter As Char

        For Each caracter In s
            If caracter = signoMas Then contador += 1
            If caracter = signoMenos Then contador += 1
        Next
        If contador > 1 Then
            Throw New ArgumentException( _
            "La cadena con el número[" & s & _
            "] contiene un carácter prohibido [" & _
            signoMas & "/" & signoMenos & _
            "] no esta permitido mas de un signo")
        End If

        If contador = 1 Then
            If s.IndexOf(signoMas, 0) <> -1 AndAlso _
               s.IndexOf(signoMenos, 0) <> -1 Then
                Throw New FormatException( _
                "Los signos + y - tienen que estar al primnicipo de la cadena")
            End If
        End If


        '-----------------------------------------------------------
        '--------- Condicion ------------
        ' Los caracteres afabeticos no estan permitiods
        For Each caracter In s
            If caracter = signoMas Then contador += 1
            If caracter = signoMenos Then contador += 1

            ' Busqueda de letras en la cadena 
            If Char.IsLetter(caracter) = True Then
                Throw New ArgumentException( _
                "La cadena con el número[" & s & _
                "] contiene un carácter prohibido [" & caracter & _
                "] no estan permitidos los caracteres alfabéticos (solo digitos)")
            End If
        Next



        '--------------------------------------------------------
        '--------------------------------------------------------------------
        ' ESTA PARTE DE CODIGO SOLO TIENE QUE FUNCIONAR SI LA 
        ' CURRENT CULTURE UTILIZA EL FOMATO NUMERICO ESPAÑOL
        ' POR ESO SE UTILIZAN DE FORMA EXPLICTA EL CARACTER (,) Y EL CARACTER (.)
        '--------------------------------------------------------------------
        ' intentar corregir el que exista un punto decimal
        ' este trozo de codigo solo funciona con la cultura española
        ' currentculture="es-ES"
        ' la idea es corregir numeros decimales que vengan en formato anglosajon
        ' utilizando como separador decimal el punto anglosajon
        '-----------------------------------
        ' 
        'Problema: ¿como averiguo el formato del numero?
        ' + El proceso de corrección de los números se rige por el siguiente protocolo:
        ' + El numero [1,234,567.25] y el número [1,234,567] están seguro en ingles,  
        '   y además sin ninguna duda es decir, si un numero tiene dos o más comas y  
        '   cero (0) o un (1) punto, está en ingles seguro y sin ninguna duda,  
        '   y cambio su formato
        ' + Hay números en los que es más difícil saber en qué formato están y  
        '   son los que tienen una coma y un punto por ejemplo [12.346,67] y [12,345.78],  
        '   para averiguarlo tengo que mirar las posiciones , si la coma es la que  
        '   está a la derecha es un numero español. Si el punto es el que está a la  
        '   derecha es un número inglés, y cambio su formato
        ' + Cuando el número solo tiene una coma o un punto entonces también tengo  
        '   la duda porque puede ser el número [1.200] el mil doscientos número en  
        '   español, o el una unidad con veinte decimal en inglés.  
        '   Pero, pensando en que cuando yo introduzco los números (por vagancia)  
        '   no pulso los separadores de millar, y además, este código funciona  
        '   siempre que el formato numérico sea el español, supongo que el número  
        '   tiene que ser un número decimal español que esta introducido con el  
        '   teclado numérico usando el punto del teclado en lugar de la coma  
        '   decimal, por lo que cambio su formato
        '--------------------------------------------------------------------
        ' ESTA PARTE DE CODIGO SOLO TIENE QUE FUNCIONAR SI LA 
        ' CURRENT CULTURE UTILIZA EL FOMATO NUMERICO ESPAÑOL
        ' POR ESO SE UTILIZAN DE FORMA EXPLICTA EL CARACTER (,) Y EL CARACTER (.)
        '--------------------------------------------------------------------
        '
        If System.Globalization.CultureInfo.CurrentCulture.NumberFormat.CurrencyDecimalSeparator = ","c AndAlso _
           System.Globalization.CultureInfo.CurrentCulture.NumberFormat.CurrencyGroupSeparator = "."c Then

            'vale, aquí la cultura current culture utiliza la coma decimal y el punto de los millares

            '----------------------------------------------------------------
            ' Lo primero es contar el numero de comas y de puntos que tiene el numero
            Dim contadorComas As Integer = 0
            Dim contadorPuntos As Integer = 0

            For Each unCaracter In s
                If unCaracter = ","c Then contadorComas += 1
                If unCaracter = "."c Then contadorPuntos += 1
            Next

            '----------------------------------------------------------------
            ' + El numero [1,234,567.25] y el número [1,234,567] están seguro en ingles,  
            '   y además sin ninguna duda es decir, si un numero tiene dos o más comas y  
            '   cero (0) o un (1) punto, está en ingles seguro y sin ninguna duda,  
            '   y cambio su formato
            If contadorPuntos <= 1 AndAlso contadorComas > 1 Then
                'reemplazo la coma  por el carácter A
                ' sé que no hay ninguna letra porque se ha comprobado en 
                ' esta misma función un poco antes
                s = s.Replace(","c, "A"c)
                'reemplazo los puntos por las comas
                s = s.Replace("."c, ","c)
                s = s.Replace("A"c, "."c)
            End If


            '----------------------------------------------------------------
            ' + Hay números en los que es más difícil saber en qué formato están y  
            '   son los que tienen una coma y un punto por ejemplo [12.346,67] y [12,345.78],  
            '   para averiguarlo tengo que mirar las posiciones , si la coma es la que  
            '   está a la derecha es un numero español. Si el punto es el que está a la  
            '   derecha es un número inglés, y cambio su formato
            If contadorPuntos = 1 AndAlso contadorComas = 1 Then
                Dim posicionPunto As Integer = s.IndexOf("."c, 0)
                Dim posicionComa As Integer = s.IndexOf(","c, 0)

                If posicionPunto > posicionComa Then
                    'Si el punto es el que está a la derecha es un número ingles
                    'reemplazo la coma  por el carácter A
                    ' sé que no hay ninguna letra porque se ha comprobado en 
                    ' esta misma función un poco antes
                    s = s.Replace(","c, "A"c)
                    'reemplazo los puntos por las comas
                    s = s.Replace("."c, ","c)
                    s = s.Replace("A"c, "."c)
                End If
            End If


            '----------------------------------------------------------------
            ' + Cuando el número solo tiene una coma o un punto entonces también tengo  
            '   la duda porque puede ser el número [1.200] el mil doscientos número en  
            '   español, o el una unidad con veinte decimal en inglés.  
            '   Pero, pensando en que cuando yo introduzco los números (por vagancia)  
            '   no pulso los separadores de millar, y además, este código funciona  
            '   siempre que el formato numérico sea el español, supongo que el número  
            '   tiene que ser un número decimal español que esta introducido con el  
            '   teclado numérico usando el punto del teclado en lugar de la coma  
            '   decimal, por lo que cambio su formato
            If contadorPuntos = 1 AndAlso contadorComas = 0 Then
                'reemplazo la coma  por el carácter A
                ' sé que no hay ninguna letra porque se ha comprobado en 
                ' esta misma función un poco antes
                s = s.Replace(","c, "A"c)
                'reemplazo los puntos por las comas
                s = s.Replace("."c, ","c)
                s = s.Replace("A"c, "."c)
            End If
        End If

        '---------------------------------------------------------------
        ' devolver la cadena con el numero corregida al formato español 
        ' casi con toda seguridad
        Return s

    End Function


#End Region





    ''' <summary>
    ''' Convierte la representación de 
    ''' <seealso cref="System.String">cadena</seealso> de un número en su 
    ''' <seealso cref="Dinero">Dinero</seealso> equivalente. 
    ''' Un valor devuelto indica si la conversión se realizó correctamente o si se produjeron errores.
    ''' </summary>
    ''' <param name="s">Representación de <seealso cref="System.String">cadena</seealso> del número que va a convertirse.</param>
    ''' <param name="result">El resultado de este método contiene el valor <seealso cref="Dinero">Dinero</seealso> equivalente al valor numérico que <paramref name="s">s</paramref> contiene si la conversión se realizó correctamente, o bien, es cero si no se realizó correctamente. La conversión no finaliza correctamente si el parámetro <paramref name="s">s</paramref> es null (Nothing en Visual Basic), no es un número con formato válido o representa un número menor que 
    ''' <seealso cref="Dinero.MinValue">MinValue</seealso>. o mayor que 
    ''' <seealso cref="Dinero.MaxValue">MaxValue</seealso>. 
    ''' Este parámetro se pasa sin inicializar. </param>
    ''' <returns>true si <paramref name="s">s</paramref> se convirtió correctamente; en caso contrario, false</returns>
    ''' <remarks>
    ''' <para>
    ''' Esta sobrecarga se diferencia del método <c>Dinero.Parse(String)</c> en que devuelve un valor 
    ''' <seealso cref="System.Boolean">booleano</seealso>  que indica si la operación de análisis 
    ''' se realizó correctamente en lugar de devolver el valor numérico analizado. 
    ''' Elimina la necesidad de usar el control de excepciones para probar una excepcion del tipo
    ''' <seealso cref="System.FormatException">FormatException</seealso> en caso de que el 
    '''  parámetro <paramref name="s">s</paramref> no sea válido y no pueda analizarse correctamente.
    ''' </para><para>
    '''El parámetro <paramref name="s">s</paramref> contiene un número con formato: 
    ''' </para><para>
    '''[eb][signo][dígitos,]dígitos[.dígitos fraccionarios][eb] 
    ''' </para><para>
    '''Los elementos entre corchetes ([ y ]) son opcionales. En la tabla siguiente se describe cada elemento.
    ''' </para><para>
    ''' eb - Espacio en blanco opcional. 
    ''' </para><para>
    ''' signo - Signo opcional. 
    ''' </para><para>
    ''' dígitos - Secuencia de dígitos comprendidos entre el 0 y el 9. 
    ''' </para><para>
    ''' , - Símbolo separador de miles específico de la referencia cultural.
    ''' </para><para>
    ''' . - Símbolo separador de decimales específico de la referencia cultural. 
    ''' </para><para>
    ''' dígitos fraccionarios - Secuencia de dígitos comprendidos entre el 0 y el 9. 
    ''' </para><para>
    '''El parámetro <paramref name="s">s</paramref> se interpreta utilizando el estilo 
    ''' <seealso cref="System.Globalization.NumberStyles.Number">NumberStyles.Number</seealso>. 
    ''' Esto significa que se permiten espacios en blanco y separadores de miles, pero no símbolos de moneda. Para definir explícitamente los elementos (como símbolos de moneda, separadores de miles y espacios en blanco) que pueden estar presentes en <paramref name="s">s</paramref>, use la sobrecarga del método 
    ''' <c>Dinero.TryParse(String, NumberStyles, IFormatProvider, Dinero)</c>.
    ''' </para><para>
    '''El parámetro <paramref name="s">s</paramref> 
    ''' se analiza utilizando la información de formato de un objeto 
    ''' <seealso cref="System.Globalization.NumberFormatInfo">NumberFormatInfo</seealso> 
    ''' inicializado para la referencia cultural actual del sistema. Para obtener más información, vea
    ''' <seealso cref="System.Globalization.NumberFormatInfo.CurrentInfo">CurrentInfo</seealso>  . 
    ''' Para analizar una cadena con la información de formato de alguna otra referencia cultural especificada, use la sobrecarga del método 
    ''' <c>Dinero.TryParse(String, NumberStyles, IFormatProvider, Dinero)</c>.
    ''' </para><para>
    '''Si es necesario, el valor de <paramref name="s">s</paramref> se redondea mediante el redondeo al más próximo.
    ''' </para><para>
    '''Si, durante una operación de análisis, se encuentra un separador en el parámetro <paramref name="s">s</paramref> y los separadores aplicables de moneda, número decimal y grupo son iguales, la operación de análisis supone que se trata de un separador decimal en lugar de un separador de grupo. Para obtener más información sobre separadores, vea 
    '''<seealso cref="System.Globalization.NumberFormatInfo.CurrencyDecimalSeparator">CurrencyDecimalSeparator</seealso> , 
    '''<seealso cref="System.Globalization.NumberFormatInfo.NumberDecimalSeparator">NumberDecimalSeparator</seealso> , 
    ''' <seealso cref="System.Globalization.NumberFormatInfo.CurrencyGroupSeparator">CurrencyGroupSeparator</seealso> y 
    '''<seealso cref="System.Globalization.NumberFormatInfo.NumberGroupSeparator">NumberGroupSeparator</seealso> .
    ''' </para>
    ''' <example>
    '''  Dim s As String
    '''  Dim result As Decimal
    '''  Dim returnValue As Boolean
    '''  returnValue = Decimal.TryParse(s, result)
    ''' </example>
    ''' </remarks>
    Public Overloads Shared Function TryParse( _
           ByVal s As System.String, _
           ByRef result As Dinero _
           ) As System.Boolean

        Try
            s = Dinero.CorregirCadenaConNumeroToCulturaEspañola(s)
            s = Dinero.LimitarNueveDecimales(s)

            Dim numero As Decimal
            numero = Decimal.Parse(s)
            ' ------------------------------------
            ' regla FxCop
            '   replace this call with a 
            '   call to 'Dinero.New(d, CultureInfo)'. 
            ' ------------------------------------
            ' No la cumplo porque el constructor ya lo hace 
            ' ------------------------------------
            result = New Dinero(numero)
            Return True

        Catch ex As Exception
            Return False
        End Try

    End Function



    ''' <summary>
    ''' Convierte la representación de 
    ''' <seealso cref="System.String">cadena</seealso> de un número en su 
    ''' <seealso cref="Dinero">Dinero</seealso> equivalente.  utilizando el estilo y el formato específico de la referencia cultural que se hayan especificado. Un valor devuelto indica si la conversión se realizó correctamente o si se produjeron errores.
    ''' </summary>
    ''' <param name="s">Representación de <seealso cref="System.String">cadena</seealso> del número que va a convertirse.</param>
    ''' <param name="style">Una combinación bit a bit de valores NumberStyles que indica el formato permitido de  <paramref name="s">s</paramref>. Un valor que se especifica de forma habitual es Number.</param>
    ''' <param name="provider">Objeto IFormatProvider que proporciona información de análisis específico de la referencia cultural sobre  <paramref name="s">s</paramref>. </param>
    ''' <param name="result">
    ''' El resultado de este método contiene el Dinero equivalente al valor numérico que 
    ''' <paramref name="s">s</paramref> contiene si la conversión se realizó correctamente,
    '''  o bien, es cero si no se realizó correctamente. La conversión no se realiza 
    ''' correctamente si el parámetro <paramref name="s">s</paramref> es null 
    ''' (Nothing en Visual Basic), no tiene un formato compatible con style 
    ''' o representa un número menor que    
    ''' <seealso cref="Dinero.MinValue">MinValue</seealso>. o mayor que 
    ''' <seealso cref="Dinero.MaxValue">MaxValue</seealso>. . 
    ''' Este parámetro se pasa sin inicializar. </param>
    ''' <returns>ES true si <paramref name="s">s</paramref> se convirtió correctamente; en caso contrario, es false.</returns>
    ''' <remarks>
    ''' <para>Esta sobrecarga se diferencia del método 
    '''  <c>Dinero.Parse(String, NumberStyles, IFormatProvider)</c> en que devuelve un valor    
    '''  <seealso cref="System.Boolean">booleano</seealso>  que indica si la operación de análisis 
    '''  se realizó correctamente en lugar de devolver el valor numérico analizado. 
    '''  Elimina la necesidad de usar el control de excepciones para probar una excepcion del tipo
    ''' <seealso cref="System.FormatException">FormatException</seealso> en caso de que el 
    '''  parámetro <paramref name="s">s</paramref> no sea válido y no pueda analizarse correctamente.
    ''' </para><para>
    ''' El parámetro<paramref name="style">style</paramref> define el formato permitido del parámetro <paramref name="s">s</paramref> para que la operación de análisis se realice correctamente. Debe ser una combinación de marcadores de bit de la enumeración 
    ''' <seealso cref="System.Globalization.NumberStyles">NumberStyles</seealso>.
    '''  No se admiten los siguientes miembros de 
    ''' <seealso cref="System.Globalization.NumberStyles">NumberStyles</seealso>:
    ''' </para><para>
    '''<seealso cref="System.Globalization.NumberStyles.AllowHexSpecifier">NumberStyles.AllowHexSpecifier</seealso> 
    ''' </para><para>
    '''<seealso cref="System.Globalization.NumberStyles.HexNumber">NumberStyles.HexNumber</seealso>. 
    ''' </para><para>
    '''  En función del valor de estilo, <paramref name="s">s</paramref> puede incluir los elementos siguientes: 
    ''' </para><para>
    ''' [eb][$][signo][dígitos,]dígitos[.dígitos fraccionarios][e[signo]dígitos][eb] 
    ''' </para><para>
    ''' Los elementos entre corchetes ([ y ]) son opcionales.
    ''' </para><para>
    '''  En la tabla siguiente se describe cada elemento. 
    ''' </para><para>
    ''' eb - Espacio en blanco opcional. El espacio en blanco puede aparecer al principio de 
    ''' <paramref name="s">s</paramref> si <paramref name="style">style</paramref> incluye el marcador
    ''' <seealso cref="System.Globalization.NumberStyles.AllowLeadingWhite">NumberStyles.AllowLeadingWhite</seealso>. 
    ''' Puede aparecer al final de <paramref name="s">s</paramref> si 
    ''' <paramref name="style">style</paramref> incluye el marcador 
    ''' <seealso cref="System.Globalization.NumberStyles.AllowLeadingWhite">NumberStyles.AllowLeadingWhite</seealso>. 
    ''' </para><para>
    '''  $ - Símbolo de moneda específico de la referencia cultural. Su posición en la cadena viene definida por las propiedades 
    ''' <seealso cref="System.Globalization.NumberFormatInfo.CurrencyNegativePattern">NumberFormatInfo.CurrencyNegativePattern</seealso> o 
    ''' <seealso cref="System.Globalization.NumberFormatInfo.CurrencyPositivePattern">NumberFormatInfo.CurrencyPositivePattern</seealso> 
    ''' del objeto 
    '''  <seealso cref="System.Globalization.NumberFormatInfo">NumberFormatInfo</seealso> devuelto por el método 
    ''' <seealso cref="System.IFormatProvider.GetFormat">IFormatProvider.GetFormat</seealso>  
    ''' del parámetro <paramref name="style">provider</paramref> . 
    ''' El símbolo de moneda puede aparecer en <paramref name="s">s</paramref> si 
    ''' <paramref name="style">style</paramref> incluye el marcador 
    ''' <seealso cref="System.Globalization.NumberStyles.AllowCurrencySymbol">NumberStyles.AllowCurrencySymbol</seealso>.  
    ''' </para><para>
    ''' signo - Signo opcional.  
    ''' </para><para>
    ''' dígitos - Secuencia de dígitos comprendidos entre el 0 y el 9.  
    ''' </para><para>
    ''' . - Símbolo separador de decimales específico de la referencia cultural.  
    ''' </para><para>
    ''' dígitos fraccionarios - Secuencia de dígitos comprendidos entre el 0 y el 9. 
    ''' </para><para>
    ''' El parámetro <paramref name="style">style</paramref> especifica el formato permitido del parámetro  <paramref name="s">s</paramref>, y puede consistir en una o más constantes enumeradas de 
    ''' <seealso cref="System.Globalization.NumberStyles">NumberStyles</seealso> 
    ''' combinadas mediante una operación OR bit a bit. 
    ''' Si <paramref name="style">style</paramref> es null, 
    ''' <paramref name="s">s</paramref> se interpreta utilizando el estilo 
    '''  <seealso cref="System.Globalization.NumberStyles.Number">NumberStyles.Number</seealso>.
    ''' </para><para>
    ''' El parámetro  <paramref name="provider">provider</paramref> es una implementación de 
    '''  <seealso cref="System.IFormatProvider">IFormatProvider</seealso>., 
    ''' como un objeto 
    '''  <seealso cref="System.Globalization.NumberFormatInfo">NumberFormatInfo</seealso> o
    '''  <seealso cref="System.Globalization.CultureInfo">CultureInfo</seealso> . 
    ''' El parámetro <paramref name="provider">provider</paramref> proporciona información específica de la referencia cultural utilizada en el análisis. Si <paramref name="provider">provider</paramref> es null (Nothing en Visual Basic), se utiliza la referencia cultural actual del subproceso.
    ''' </para><para>
    '''Si, durante una operación de análisis, se encuentra un separador en el parámetro 
    ''' <paramref name="s">s</paramref> y los separadores aplicables de moneda, número decimal y grupo son iguales, la operación de análisis supone que se trata de un separador decimal en lugar de un separador de grupo. Para obtener más información sobre separadores, vea 
    '''<seealso cref="System.Globalization.NumberFormatInfo.CurrencyDecimalSeparator">CurrencyDecimalSeparator</seealso> , 
    '''<seealso cref="System.Globalization.NumberFormatInfo.NumberDecimalSeparator">NumberDecimalSeparator</seealso> , 
    ''' <seealso cref="System.Globalization.NumberFormatInfo.CurrencyGroupSeparator">CurrencyGroupSeparator</seealso> y 
    '''<seealso cref="System.Globalization.NumberFormatInfo.NumberGroupSeparator">NumberGroupSeparator</seealso> .
    ''' </para>
    ''' <example>
    ''' Dim s As String
    ''' Dim style As NumberStyles
    ''' Dim provider As IFormatProvider
    ''' Dim result As Decimal
    ''' Dim returnValue As Boolean
    ''' returnValue = Decimal.TryParse(s, style, provider, result)
    '''</example>
    '''</remarks>
    ''' <exception cref="System.ArgumentException"> 
    ''' <para><paramref name="style">style</paramref> no es un valor de 
    ''' <seealso cref="System.Globalization.NumberStyles">NumberStyles</seealso>. 
    ''' </para>
    ''' <para>O bien</para>
    ''' <para>El valor de <paramref name="style">style</paramref> es el valor de 
    ''' <seealso cref="System.Globalization.NumberStyles.AllowHexSpecifier">AllowHexSpecifier</seealso>.
    ''' </para>
    '''</exception>
    Public Overloads Shared Function TryParse( _
               ByVal s As System.String, _
               ByVal style As System.Globalization.NumberStyles, _
               ByVal provider As System.IFormatProvider, _
               ByRef result As Dinero _
               ) As System.Boolean

        Try
            Dim numero As Decimal
            numero = Decimal.Parse(s, style, provider)
            ' ------------------------------------
            ' regla FxCop
            '   replace this call with a 
            '   call to 'Dinero.New(d, CultureInfo)'. 
            ' ------------------------------------
            ' No la cumplo porque el constructor ya lo hace 
            ' ------------------------------------
            result = New Dinero(numero)
            Return True
        Catch ex As Exception
            Return False
        End Try

    End Function



#End Region


#Region "  From / TO"

    '-----------------------------------------------------------------------
    ' Recuerda ...
    ' Estas dos funciones estan en la interfaz IConvertible
    '     Public Overloads Function ToDecimal() As Decimal
    '     Public Overloads Function ToDecimal(ByVal A As Dinero) As Decimal
    '-----------------------------------------------------------------------


    ''' <summary>
    '''  Recibe un numero Decimal y carga en la estructura Dinero
    ''' </summary>
    ''' <param name="value">En numero</param>
    Private Sub FromDecimal(ByVal value As Decimal)
        ' separar el numero
        Me._ParteEntera = ObtenerParteEnteraDelNumero(value)
        Me._ParteDecimal = ObtenerParteDecimalDelNumero(value)
    End Sub


    ''' <summary>
    '''  Obtiene la parte entera de un numero decimal
    ''' </summary>
    ''' <param name="value">El número System.Decimal que se va a analizar</param>
    ''' <returns>un número System.Long que contiene la parte entera del numero decimal</returns>
    ''' <remarks>Por ejemplo 25,67 devuelve 25 </remarks>
    Private Shared Function ObtenerParteEnteraDelNumero(ByVal value As Decimal) As System.Int64

        Try
            ' comprobar que la cantidad es mas o menos correcta
            Call ComprobacionValorDecimal(value)

            Dim parteEntera As Long = 0L
            '--------------------------------------------
            ' recuperar la parte entera del numero
            If value < 0 Then
                ' es un numero negativo
                ' Por ejemplo -15.15
                ' --> Math.Ceiling = devuelve 15
                ' --> Math.Floor   = devuelve 16
                parteEntera = CType(Math.Ceiling(value), Long)
            End If

            If value = 0 Then
                parteEntera = 0L
            End If

            If value > 0 Then
                'es un numero positivo
                parteEntera = CType(Math.Floor(value), Long)
            End If

            Return parteEntera
        Catch ex As Exception
            Throw
        End Try
    End Function


    ''' <summary>
    '''  Obtiene la parte decimal de un numero decimal
    ''' </summary>
    ''' <param name="value">El número System.Decimal que se va a analizar</param>
    ''' <returns>Un número System.Integer que contiene la parte decimal del numero decimal 
    '''          multiplicada por la unidad seguida de 9 ceros</returns>
    ''' <remarks>
    ''' Por ejemplo 25,674 devuelve 674 000 000 
    ''' Por ejemplo 25,004 devuelve 004 000 000 
    ''' </remarks>
    Private Shared Function ObtenerParteDecimalDelNumero(ByVal value As Decimal) As Integer

        Try
            ' comprobar que la cantidad es mas o menos correcta
            Call ComprobacionValorDecimal(value)

            Dim parteEntera As Long = 0L
            Dim parteDecimal As Decimal = Decimal.Zero
            Dim resultado As Integer = 0
            If value <> 0 Then
                '--------------------------------------------
                ' recuperar la parte entera del numero
                parteEntera = ObtenerParteEnteraDelNumero(value)

                '--------------------------------------------
                'Recuperar la parte decimal del numero
                ' Variable para calculos intermedios
                ' calculos
                parteDecimal = value - parteEntera
                '(siempre será un numero positivo)
                parteDecimal = Math.Abs(parteDecimal)
                ' multiplicarlo por el numerio de decimales que voy a almacenar
                ' por ejemplo, con tres decimales 0,02345 X (10E3) 1000 = 23,45
                ' ahora extraigo solo la parte entera = 23
                If parteDecimal > 0 Then
                    parteDecimal = parteDecimal * _UNO_SEGUIDO_NUEVE_CEROS
                    ' recuperar la parte entera del numero ( siempre es un numero positivo)
                End If
                resultado = CType(Math.Floor(parteDecimal), Integer)
            End If

            Return resultado
        Catch ex As Exception
            Throw
        End Try

    End Function



    ''' <summary>
    '''  Recibe un numero Long y carga en la estructura Dinero
    ''' </summary>
    ''' <param name="value">En numero</param>
    Private Sub FromLong(ByVal value As Long)
        Me.ValorParteEntera = value
        Me._ParteDecimal = 0
    End Sub

#End Region


#Region "Operadores de conversion CTYPE Explicit / Implicit"

    ' Explicit operator
    'http://msdn.microsoft.com/es-es/library/system.decimal.op_explicit(VS.80).aspx

    ' ms-help://MS.VSCC.v90/MS.msdnexpress.v90.es/dv_vbalr/html/54203dfa-c24b-463f-9942-d5153e89e762.htm

    ' Widening - Indica que un operador de conversión (CType) convierte una clase o 
    ' una estructura en un tipo que puede incluir todos los valores posibles  
    ' de la clase o la estructura original.
    '
    ' Narrowing - Indica que un operador de conversión (CType) convierte una clase o  
    ' una estructura en un tipo que quizá no pueda incluir algunos de los  
    ' valores posibles de la clase o la estructura original.

#Region "Explicit"

    '-------------------------------------------------------------------------------
    ' Ver un ejemplo en
    ' http://msdn.microsoft.com/es-es/library/system.decimal.op_explicit(VS.80).aspx
    '-------------------------------------------------------------------------------

    '-------------------------------------------------------------
    ''' <summary>
    '''   Convierte <see cref="Dinero">Dinero</see> en un entero de 8 bits sin signo. 
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> que se va a convertir. </param>
    ''' <returns>Entero de 8 bits sin signo que representa el Dinero convertido</returns>
    ''' <exception cref="System.OverflowException">OverflowException-  value es menor que 
    ''' <seealso cref="Byte.MinValue">Byte.MinValue</seealso> o mayor que 
    ''' <seealso cref="Byte.Maxvalue">Byte.Maxvalue</seealso>. 
    ''' </exception>
    Public Shared Narrowing Operator CType(ByVal value As Dinero) As Byte
        Return CType(value.Valor, Byte)
    End Operator


    ''' <summary>
    '''   [NO SOPORTADO] Convierte <see cref="Dinero">Dinero</see> en un  Carácter Unicode - 
    '''   Esta conversion no es posible
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> 
    '''         que se va a convertir. </param>
    ''' <returns>Una Exception <seealso cref="System.OverflowException">
    '''          OverflowException</seealso> Esta conversion no es posible. </returns>
    ''' <exception cref="System.OverflowException">OverflowException-  
    '''       Esta conversion no es posible. 
    ''' </exception>
    Public Shared Narrowing Operator CType(ByVal value As Dinero) As Char
        Throw New System.InvalidCastException("Esta conversion no es posible")
    End Operator




    ''' <summary>
    '''   [NO SOPORTADO] Convierte <see cref="Dinero">Dinero</see> en una fecha - 
    '''   Esta conversion no es posible
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> 
    '''         que se va a convertir. </param>
    ''' <returns>Una Exception <seealso cref="System.OverflowException">
    '''          OverflowException</seealso> Esta conversion no es posible. </returns>
    ''' <exception cref="System.OverflowException">OverflowException-  
    '''       Esta conversion no es posible. 
    ''' </exception>
    Public Shared Narrowing Operator CType(ByVal value As Dinero) As Date
        Throw New System.InvalidCastException("Esta conversion no es posible")
    End Operator




    ''' <summary>
    '''   Convierte <see cref="Dinero">Dinero</see> 
    '''   en un número de punto flotante de precisión doble en un Double. 
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> 
    '''         que se va a convertir. </param>
    ''' <returns> Número de punto flotante de precisión doble.    
    '''          <seealso cref="System.Decimal">Double</seealso>, 
    '''          que representa el Dinero convertido. </returns>
    ''' <remarks>
    '''  Esta operación puede producir errores de redondeo porque un número
    '''  de punto flotante de precisión simple tiene menos dígitos significativos 
    '''  que el valor Decimal subyacente en la estructura Dinero.
    ''' </remarks>
    ''' <exception cref="System.OverflowException">OverflowException-  value es menor que 
    ''' <seealso cref="Double.MinValue">doble.MinValue</seealso> o mayor que 
    ''' <seealso cref="Double.Maxvalue">doble.Maxvalue</seealso>.  O bien, value es 
    ''' <seealso cref="Double.NaN">Double.NaN</seealso>, 
    ''' <seealso cref="Double.PositiveInfinity">Double.PositiveInfinity</seealso> o 
    ''' <seealso cref="Double.NegativeInfinity">Double.NegativeInfinity</seealso>. 
    ''' </exception>
    Public Shared Narrowing Operator CType(ByVal value As Dinero) As Double
        Return CType(value.Valor, Double)
    End Operator


    ''' <summary>
    '''   Convierte <see cref="Dinero">Dinero</see> 
    '''   en un entero de 16 bits (Short) con signo. 
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> 
    '''         que se va a convertir. </param>
    ''' <returns> Entero de 16 bits con signo,  
    '''          <seealso cref="System.Short">Short</seealso>,  
    '''          que representa el Dinero convertido. </returns>
    ''' <remarks></remarks>
    ''' <exception cref="System.OverflowException">OverflowException-  value es menor que 
    ''' <seealso cref="int16.MinValue">int16.MinValue</seealso> o mayor que 
    ''' <seealso cref="int16.Maxvalue">int16.Maxvalue</seealso>. 
    ''' </exception>
    Public Shared Narrowing Operator CType(ByVal value As Dinero) As Short
        Return CType(value.Valor, Short)
    End Operator


    ''' <summary>
    '''   Convierte <see cref="Dinero">Dinero</see> 
    '''   en un entero de 32 bits (Integer) con signo. 
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> 
    '''         que se va a convertir. </param>
    ''' <returns> Entero de 32 bits con signo,  
    '''          <seealso cref="System.Int32">Int32</seealso>,  Integer
    '''          que representa el Dinero convertido. </returns>
    ''' <remarks></remarks>
    ''' <exception cref="System.OverflowException">OverflowException-  value es menor que 
    ''' <seealso cref="Int32.MinValue">Int32.MinValue</seealso> o mayor que 
    ''' <seealso cref="Int32.Maxvalue">Int32.Maxvalue</seealso>. 
    ''' </exception>
    Public Shared Narrowing Operator CType(ByVal value As Dinero) As Integer
        Return CType(value.Valor, Integer)
    End Operator


    ''' <summary>
    '''   Convierte <see cref="Dinero">Dinero</see> 
    '''   en un entero de 64 bits (Long) con signo. 
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> 
    '''         que se va a convertir. </param>
    ''' <returns> Entero de 64 bits con signo,  
    '''          <seealso cref="System.Int32">long</seealso>, 
    '''          que representa el Dinero convertido. </returns>
    ''' <remarks></remarks>
    ''' <exception cref="System.OverflowException">OverflowException-  value es menor que 
    ''' <seealso cref="Int64.MinValue">Int64.MinValue</seealso> o mayor que 
    ''' <seealso cref="Int64.Maxvalue">Int64.Maxvalue</seealso>. 
    ''' </exception>
    Public Shared Narrowing Operator CType(ByVal value As Dinero) As Long
        Return CType(value.Valor, Long)
    End Operator


    ''' <summary>
    '''   Convierte <see cref="Dinero">Dinero</see> 
    '''   en un entero de 8 bits (SByte) con signo. 
    '''   Este método no es compatible con CLS.
    '''   La alternativa compatible con CLS es ToInt16
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> 
    '''         que se va a convertir. </param>
    ''' <returns> Entero de 8 bits con signo,  
    '''          <seealso cref="System.SByte">SByte</seealso>, 
    '''          que representa el Dinero convertido. </returns>
    ''' <remarks></remarks>
    ''' <exception cref="System.OverflowException">OverflowException-  value es menor que 
    ''' <seealso cref="SByte.MinValue">SByte.MinValue</seealso> o mayor que 
    ''' <seealso cref="SByte.Maxvalue">SByte.Maxvalue</seealso>. 
    ''' </exception>
    <CLSCompliantAttribute(False)> _
    Public Shared Narrowing Operator CType(ByVal value As Dinero) As SByte
        Return CType(value.Valor, SByte)
    End Operator


    ''' <summary>
    '''   Convierte <see cref="Dinero">Dinero</see> 
    '''   en un número de punto flotante de precisión simple. 
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> 
    '''         que se va a convertir. </param>
    ''' <returns> Número de punto flotante de precisión simple    
    '''          <seealso cref="System.Single">Single</seealso>, 
    '''          que representa el Dinero convertido. </returns>
    ''' <remarks>
    '''  Esta operación puede producir errores de redondeo porque un número
    '''  de punto flotante de precisión simple tiene menos dígitos significativos 
    '''  que el valor Decimal subyacente en la estructura Dinero.
    ''' </remarks>
    ''' <exception cref="System.OverflowException">OverflowException-  value es menor que 
    ''' <seealso cref="Single.MinValue">Single.MinValue</seealso> o mayor que 
    ''' <seealso cref="Single.Maxvalue">Single.Maxvalue</seealso>. 
    ''' </exception>
    Public Shared Narrowing Operator CType(ByVal value As Dinero) As Single
        Return CType(value.Valor, Single)
    End Operator


    ''' <summary>
    '''   Convierte <see cref="Dinero">Dinero</see> 
    '''   en un entero de 16 bits [UShort] sin signo.
    '''   Este método no es compatible con CLS.  La alternativa compatible con CLS es ToInt32.
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> 
    '''         que se va a convertir. </param>
    ''' <returns> Entero 16 bits sin signo,  
    '''          <seealso cref="System.UShort">UShort</seealso>, 
    '''          que representa el Dinero convertido. </returns>
    ''' <remarks></remarks>
    ''' <exception cref="System.OverflowException">OverflowException-  value es menor que 
    ''' <seealso cref="UShort.MinValue">UShort.MinValue</seealso> o mayor que 
    ''' <seealso cref="UShort.Maxvalue">UShort.Maxvalue</seealso>. 
    ''' </exception>
    <CLSCompliantAttribute(False)> _
    Public Shared Narrowing Operator CType(ByVal value As Dinero) As UShort
        Return CType(value.Valor, UShort)
    End Operator


    ''' <summary>
    '''   Convierte <see cref="Dinero">Dinero</see> 
    '''   en un entero de 32 bits [UInteger] sin signo.
    '''   Este método no es compatible con CLS.  La alternativa compatible con CLS es ToInt64.
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> 
    '''         que se va a convertir. </param>
    ''' <returns> Entero 32 bits sin signo,  
    '''          <seealso cref="System.UInteger">UInteger</seealso>, 
    '''          que representa el Dinero convertido. </returns>
    ''' <remarks></remarks>
    ''' <exception cref="System.OverflowException">OverflowException-  value es menor que 
    ''' <seealso cref="UInteger.MinValue">UInteger.MinValue</seealso> o mayor que 
    ''' <seealso cref="UInteger.Maxvalue">UInteger.Maxvalue</seealso>. 
    ''' </exception>
    <CLSCompliantAttribute(False)> _
    Public Shared Narrowing Operator CType(ByVal value As Dinero) As UInteger
        Return CType(value.Valor, UInteger)
    End Operator

    ''' <summary>
    '''   Convierte <see cref="Dinero">Dinero</see> 
    '''   en un entero de 64 bits [ULong] sin signo.
    '''   Este método no es compatible con CLS.
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> 
    '''         que se va a convertir. </param>
    ''' <returns> Entero 64 bits sin signo,  
    '''          <seealso cref="System.ULong">ULong</seealso>, 
    '''          que representa el Dinero convertido. </returns>
    ''' <remarks></remarks>
    ''' <exception cref="System.OverflowException">OverflowException-  value es menor que 
    ''' <seealso cref="ULong.MinValue">ULong.MinValue</seealso> o mayor que 
    ''' <seealso cref="ULong.Maxvalue">ULong.Maxvalue</seealso>. 
    ''' </exception>
    <CLSCompliantAttribute(False)> _
    Public Shared Narrowing Operator CType(ByVal value As Dinero) As ULong
        Return CType(value.Valor, ULong)
    End Operator


    ''' <summary>
    '''   Convierte <see cref="Dinero">Dinero</see> 
    '''   en un número de punto flotante de precisión doble en un Decimal. 
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> 
    '''         que se va a convertir. </param>
    ''' <returns> Número de punto flotante de precisión simple    
    '''          <seealso cref="System.Decimal">Decimal</seealso>, 
    '''          que representa el Dinero convertido. </returns>
    ''' <remarks>
    '''  Esta operación puede producir errores de redondeo porque un número
    '''  de punto flotante de precisión simple tiene menos dígitos significativos 
    '''  que el valor Decimal subyacente en la estructura Dinero.
    ''' </remarks>
    ''' <exception cref="System.OverflowException">OverflowException-  value es menor que 
    ''' <seealso cref="Decimal.MinValue">Decimal.MinValue</seealso> o mayor que 
    ''' <seealso cref="Decimal.Maxvalue">Decimal.Maxvalue</seealso>. 
    ''' </exception>
    Public Shared Narrowing Operator CType(ByVal value As Dinero) As Decimal
        Return CType(value.Valor, Decimal)
    End Operator

    '----------------------------------------------
    ' Cuidado con estas dos, rompen la secuencia
    '----------------------------------------------
    ''' <summary>
    '''   [NO SOPORTADO] Convierte un número de punto flotante de precisión doble [Double]
    '''   en un valor <see cref="Dinero">Dinero</see>. Esta conversion no es posible
    '''   por problemas de redondeo de cantidades, utiliza en su lugar un valor [Decimal] 
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> 
    '''         que se va a convertir. </param>
    ''' <returns>Una Exception <seealso cref="System.InvalidCastException">
    '''          InvalidCastException</seealso> Esta conversion no es posible. </returns>
    ''' <exception cref="System.InvalidCastException">InvalidCastException-  
    '''       Esta conversion no es posible. 
    ''' </exception>
    Public Shared Narrowing Operator CType(ByVal value As Double) As Dinero
        Using SW As New System.IO.StringWriter( _
                        System.Globalization.CultureInfo.CurrentCulture)
            SW.WriteLine("Estructura Dinero")
            SW.WriteLine("El valor de punto flotante [System.Double][" & value & "] no puede ")
            SW.WriteLine("ser una cantidad de dinero por problemas de redondeo de cantidades ")
            SW.WriteLine("Utiliza en su lugar un valor numerico de tipo [System.Decimal] ")
            SW.Flush()
            Throw New System.InvalidCastException(SW.ToString)
        End Using
    End Operator

    ''' <summary>
    '''   [NO SOPORTADO] Convierte un número de punto flotante de precisión simple [single] 
    '''   en un valor <see cref="Dinero">Dinero</see>. Esta conversion no es posible
    '''   por problemas de redondeo de cantidades, utiliza en su lugar un valor [Decimal] 
    ''' </summary>
    ''' <param name="value">Estructura <see cref="Dinero">Dinero</see> 
    '''         que se va a convertir. </param>
    ''' <returns>Una Exception <seealso cref="System.InvalidCastException">
    '''          InvalidCastException</seealso> Esta conversion no es posible. </returns>
    ''' <exception cref="System.InvalidCastException">InvalidCastException-  
    '''       Esta conversion no es posible. 
    ''' </exception>
    Public Shared Narrowing Operator CType(ByVal value As Single) As Dinero
        Using SW As New System.IO.StringWriter( _
                        System.Globalization.CultureInfo.CurrentCulture)
            SW.WriteLine("Estructura Dinero")
            SW.WriteLine("El valor de punto flotante [System.Single][" & value & "] no puede ")
            SW.WriteLine("ser una cantidad de dinero por problemas de redondeo de cantidades ")
            SW.WriteLine("Utiliza en su lugar un valor numerico de tipo [System.Decimal] ")
            SW.Flush()
            Throw New System.InvalidCastException(SW.ToString)
        End Using
    End Operator

#End Region



#Region "Implicit"


    ''' <summary>
    '''   Convierte un valor logico en una estructura Dinero. 
    ''' </summary>
    ''' <param name="value">un valor logico [Boolean]</param>
    ''' <returns> Una estructura Dinero. 
    ''' Si el valor es TRUE  la cantidad de dinero es 1,00
    ''' Si el valor es FALSE La cantidad de dinero es 0,00
    ''' </returns>
    Public Shared Narrowing Operator CType(ByVal value As Boolean) As Dinero
        Dim unDinero As Dinero = Dinero.Zero
        If value = True Then
            unDinero.ValorParteEntera = 1
        End If
        Return unDinero
    End Operator

    ''' <summary>Convierte un valor [Byte] en una estructura Dinero.</summary>
    ''' <param name="value">un valor [Byte]</param>
    ''' <returns> Una estructura Dinero.</returns>
    Public Shared Narrowing Operator CType(ByVal value As Byte) As Dinero
        Dim unDinero As Dinero = Dinero.Zero
        unDinero.ValorParteEntera = CType(value, Int64)
        Return unDinero
    End Operator

    ''' <summary>
    '''   [NO SOPORTADO]  Esta conversion no es posible
    ''' </summary>
    Public Shared Narrowing Operator CType(ByVal value As Char) As Dinero
        Using SW As New System.IO.StringWriter( _
                        System.Globalization.CultureInfo.CurrentCulture)
            SW.WriteLine("Estructura Dinero")
            SW.WriteLine("Un valor[Char][" & value & "] no puede ")
            SW.WriteLine("ser transformado en una cantidad de dinero")
            SW.WriteLine("Utiliza en su lugar un valor numerico de tipo [System.Decimal] ")
            SW.Flush()
            Throw New System.InvalidCastException(SW.ToString)
        End Using
    End Operator

    ''' <summary>
    '''   [NO SOPORTADO]  Esta conversion no es posible
    ''' </summary>
    Public Shared Narrowing Operator CType(ByVal value As Date) As Dinero
        Using SW As New System.IO.StringWriter( _
                        System.Globalization.CultureInfo.CurrentCulture)
            SW.WriteLine("Estructura Dinero")
            SW.WriteLine("Un valor[Date], [" & value & "] no puede ")
            SW.WriteLine("ser transformado en una cantidad de dinero")
            SW.WriteLine("Utiliza en su lugar un valor numerico de tipo [System.Decimal] ")
            SW.Flush()
            Throw New System.InvalidCastException(SW.ToString)
        End Using
    End Operator


    ''' <summary>
    ''' Convierte un entero de 16 bits con signo en una estructura Dinero. 
    ''' </summary>
    ''' <param name="value">Entero de 16 bits con signo</param>
    ''' <returns> una estructura Dinero</returns>
    ''' <remarks></remarks>
    Public Shared Widening Operator CType(ByVal value As Short) As Dinero
        Return New Dinero(CType(value, Long), Globalization.CultureInfo.CurrentCulture)
    End Operator

    ''' <summary>
    ''' Convierte un entero de 32 bits con signo en una estructura Dinero. 
    ''' </summary>
    ''' <param name="value">Entero de 32 bits con signo</param>
    ''' <returns>una estructura Dinero</returns>
    ''' <remarks></remarks>
    Public Shared Widening Operator CType(ByVal value As Integer) As Dinero
        Return New Dinero(CType(value, Long), Globalization.CultureInfo.CurrentCulture)
    End Operator


    ''' <summary>
    ''' Convierte un entero de 64 bits con signo en una estructura Dinero. 
    ''' </summary>
    ''' <param name="value">Entero de 64 bits con signo</param>
    ''' <returns>una estructura Dinero</returns>
    Public Shared Widening Operator CType(ByVal value As Long) As Dinero
        Return New Dinero(value, Globalization.CultureInfo.CurrentCulture)
    End Operator



    ''' <summary>
    ''' Convierte numero decimal con signo en una estructura Dinero. 
    ''' </summary>
    ''' <param name="value">numero decimal con signo</param>
    ''' <returns>una estructura Dinero</returns>
    Public Shared Widening Operator CType(ByVal value As Decimal) As Dinero
        Return New Dinero(value, Globalization.CultureInfo.CurrentCulture)
    End Operator


    ''' <summary>
    ''' Convierte un entero de 8 bits con signo en una estructura Dinero. 
    ''' Este método no es compatible con CLS.
    ''' </summary>
    ''' <param name="value">Entero de 8 bits con signo. </param>
    ''' <returns>una estructura Dinero</returns>
    ''' <remarks></remarks>
    <CLSCompliantAttribute(False)> _
    Public Shared Widening Operator CType(ByVal value As SByte) As Dinero
        Return New Dinero(CType(value, Long), Globalization.CultureInfo.CurrentCulture)
    End Operator


    ''' <summary>
    ''' Convierte un entero de 16 bits sin signo en una estructura Dinero. 
    ''' Este método no es compatible con CLS.
    ''' </summary>
    ''' <param name="value">Entero de 16 bits sin signo. </param>
    ''' <returns>una estructura Dinero</returns>
    ''' <remarks></remarks>
    <CLSCompliantAttribute(False)> _
    Public Shared Widening Operator CType(ByVal value As UShort) As Dinero
        Return New Dinero(CType(value, Long), Globalization.CultureInfo.CurrentCulture)
    End Operator


    ''' <summary>
    ''' Convierte un entero de 32 bits sin signo en una estructura Dinero. 
    ''' Este método no es compatible con CLS.
    ''' </summary>
    ''' <param name="value">Entero de 32 bits sin signo. </param>
    ''' <returns>una estructura Dinero</returns>
    ''' <remarks></remarks>
    <CLSCompliantAttribute(False)> _
    Public Shared Widening Operator CType(ByVal value As UInteger) As Dinero
        Return New Dinero(CType(value, Long), Globalization.CultureInfo.CurrentCulture)
    End Operator


    ''' <summary>
    ''' Convierte un entero de ULong  bits sin signo en una estructura Dinero. 
    ''' Este método no es compatible con CLS.
    ''' </summary>
    ''' <param name="value">Entero de ULong  bits sin signo. </param>
    ''' <returns>una estructura Dinero</returns>
    ''' <remarks></remarks>
    <CLSCompliantAttribute(False)> _
    Public Shared Widening Operator CType(ByVal value As ULong) As Dinero
        Return New Dinero(CType(value, Long), Globalization.CultureInfo.CurrentCulture)
    End Operator

#End Region

#End Region

#Region "    Implements IFormattable"

    '-------------------------------------------------------------------------
    ' http://msdn.microsoft.com/es-es/library/system.iconvertible(VS.80).aspx
    '-------------------------------------------------------------------------


    ''' <summary>
    '''   Convierte el valor numérico de esta instancia en la representación de cadena equivalente.
    ''' </summary>
    ''' <returns>Tipo: <seealso cref="System.String">System.String</seealso>. 
    '''         Cadena que representa el valor de esta instancia.</returns>
    ''' <remarks>
    '''<para>
    '''    Al valor devuelto se le aplica formato con el especificador de formato 
    '''    numérico de monedas ("C") y el objeto 
    '''    <seealso cref="System.Globalization.NumberFormatInfo">NumberFormatInfo</seealso> 
    '''    para la referencia cultural actual.
    ''' </para>
    ''' <para>Para definir el formato de la representación de cadena del valor decimal, 
    '''   llame al método Dinero.ToString(String). Para definir la referencia cultural 
    '''   cuyo formato se utiliza en la representación de cadena del valor decimal, 
    '''   llame al método Dinero.ToString(IFormatProvider). Para definir los 
    '''   especificadores de formato y la referencia cultural que se utilizan 
    '''   para crear la representación de cadena de un valor decimal, llame al 
    '''   método Dinero.ToString(String, IFormatProvider).
    ''' </para> 
    '''<para>.NET Framework proporciona una amplia compatibilidad de formato, que
    '''  se describe con más detalle en los siguientes temas referentes al formato:</para>
    '''<para>Para obtener más información sobre los especificadores de formato numérico, vea 
    '''     <a href="http://msdn.microsoft.com/es-es/library/dwhawy9k.aspx"> 
    '''      Cadenas con formato numérico estándar</a> y  
    '''     <a href="http://msdn.microsoft.com/es-es/library/0c899ak8.aspx">
    '''     Cadenas con formato numérico personalizado.</a> 
    ''' </para>
    '''<para>Para obtener más información sobre la forma de aplicar formato, vea 
    '''  <a href="http://msdn.microsoft.com/es-es/library/fbxft59x.aspx">
    '''           Aplicar formato a tipos</a> y 
    '''  <a href="http://msdn.microsoft.com/es-es/library/26etazsy.aspx">
    '''          Información general sobre formatos</a>. 
    ''' </para>
    '''</remarks>
    Public Overloads Overrides Function ToString() As String
        If Me.CultureInfo Is Nothing Then
            ' esto es redundante, tal y como esta escrita la 
            ' estructura ¿nunca? puede darse este caso
            Return String.Format( _
                      System.Globalization.CultureInfo.CurrentCulture, _
                      "{0:C}", _
                      Me.ValorSinRedondear)
            ' Return Me.ToString("C")
        Else
            'http://msdn.microsoft.com/es-es/library/b1csw23d.aspx
            Return String.Format(Me.CultureInfo, "{0:C}", Me.ValorSinRedondear)
            ' Return Me.ValorSinRedondear.ToString("C", Me.CultureInfo)
        End If
    End Function


    ''-----------------------------------------------------------
    '' Esta esta en la Implements IConvertible
    ''-----------------------------------------------------------
    'Public Overloads Function ToString(ByVal provider As System.IFormatProvider) As String _
    '     Implements System.IConvertible.ToString
    '    Return CType(Me.ValorSinRedondear, IConvertible).ToString(provider)
    '    ' Return String.Format(provider, "{0}", Me.Valor)
    'End Function



    ''' <summary>
    '''    Convierte el valor numérico de esta instancia en la representación de 
    '''    cadena equivalente utilizando el formato especificado.
    ''' </summary>
    ''' <param name="format">Tipo:  <seealso cref="System.String">String</seealso> 
    '''      <a href="http://msdn.microsoft.com/es-es/library/427bttx3.aspx"> 
    '''         Cadena de formato numérico</a>.
    ''' </param>
    ''' <returns>Tipo: <seealso cref="System.String">String</seealso> 
    ''' Representación de cadena del valor de esta instancia tal y como especifica 
    ''' <paramref name="format">format</paramref>.
    ''' </returns>
    ''' <remarks>
    ''' <para>
    '''    El parámetro <paramref name="format">format</paramref>
    '''    puede ser cualquier especificador de formato 
    '''    numérico estándar válido salvo D, R y X, así como cualquier 
    '''    combinación de especificadores del formato numérico personalizados. 
    '''    Si format es null (Nothing en Visual Basic) o una cadena vacía, 
    '''    al valor devuelto de esta instancia se le aplica formato con el 
    '''    especificador de formato numérico general (G).
    '''</para>
    '''<para>.NET Framework proporciona una amplia compatibilidad de formato, que
    '''  se describe con más detalle en los siguientes temas referentes al formato:</para>
    '''<para>Para obtener más información sobre los especificadores de formato numérico, vea 
    '''     <a href="http://msdn.microsoft.com/es-es/library/dwhawy9k.aspx"> 
    '''      Cadenas con formato numérico estándar</a> y  
    '''     <a href="http://msdn.microsoft.com/es-es/library/0c899ak8.aspx">
    '''     Cadenas con formato numérico personalizado.</a> 
    ''' </para>
    '''<para>Para obtener más información sobre la forma de aplicar formato, vea 
    '''  <a href="http://msdn.microsoft.com/es-es/library/fbxft59x.aspx">
    '''           Aplicar formato a tipos</a> y 
    '''  <a href="http://msdn.microsoft.com/es-es/library/26etazsy.aspx">
    '''          Información general sobre formatos</a>. 
    ''' </para>
    ''' <para>
    ''' Al valor devuelto se le aplica formato con 
    ''' <seealso cref="System.Globalization.NumberFormatInfo">NumberFormatInfo</seealso>  
    ''' para la referencia cultural actual.
    ''' </para>
    ''' </remarks>
    Public Overloads Function ToString(ByVal format As String) As String
        'Return Me.ValorSinRedondear.ToString(format)
        If String.IsNullOrEmpty(format) = True Then
            'http://msdn.microsoft.com/es-es/library/b1csw23d.aspx
            format = "{0:C}"
        End If
        Return String.Format(format, Me.ValorSinRedondear)
    End Function


    ''' <summary>
    ''' Convierte el valor numérico de esta instancia en su representación de 
    ''' cadena equivalente mediante el formato y la información de formato 
    ''' específica de la referencia cultural que se hayan especificado.    
    ''' </summary>
    ''' <param name="format">Tipo:  <seealso cref="System.String">String</seealso> 
    '''      <a href="http://msdn.microsoft.com/es-es/library/427bttx3.aspx"> 
    '''         Cadena de formato numérico</a>.
    '''     String que especifica el formato que se va a utilizar. O bien, 
    '''     referencia de objeto null (Nothing en Visual Basic) para utilizar el 
    '''     formato predeterminado que se define para el tipo de la 
    '''     implementación de IFormattable. 
    ''' </param>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    '''    IFormatProvider que se va a utilizar para dar formato al valor.  O bien, 
    '''    referencia de objeto null (Nothing en Visual Basic) para obtener la 
    '''    información de formato para valores numéricos de la configuración 
    '''    regional actual del sistema operativo. 
    ''' </param>
    ''' <returns>
    ''' Representación de cadena del valor de esta instancia tal y como especifica 
    ''' <paramref name="format">format</paramref> y 
    ''' <paramref name="provider">provider</paramref>.
    ''' </returns>
    ''' <remarks>
    ''' <para>
    '''    El parámetro <paramref name="format">format</paramref>
    '''    puede ser cualquier especificador de formato 
    '''    numérico estándar válido salvo D, R y X, así como cualquier 
    '''    combinación de especificadores del formato numérico personalizados. 
    '''    Si format es null (Nothing en Visual Basic) o una cadena vacía, 
    '''    al valor devuelto de esta instancia se le aplica formato con el 
    '''    especificador de formato numérico general (G).
    '''</para>
    '''<para>.NET Framework proporciona una amplia compatibilidad de formato, que
    '''  se describe con más detalle en los siguientes temas referentes al formato:</para>
    '''<para>Para obtener más información sobre los especificadores de formato numérico, vea 
    '''     <a href="http://msdn.microsoft.com/es-es/library/dwhawy9k.aspx"> 
    '''      Cadenas con formato numérico estándar</a> y  
    '''     <a href="http://msdn.microsoft.com/es-es/library/0c899ak8.aspx">
    '''     Cadenas con formato numérico personalizado.</a> 
    ''' </para>
    '''<para>Para obtener más información sobre la forma de aplicar formato, vea 
    '''  <a href="http://msdn.microsoft.com/es-es/library/fbxft59x.aspx">
    '''           Aplicar formato a tipos</a> y 
    '''  <a href="http://msdn.microsoft.com/es-es/library/26etazsy.aspx">
    '''          Información general sobre formatos</a>. 
    ''' </para>
    '''<para>
    '''     El parámetro <paramref name="provider">provider</paramref> 
    '''     es una implementación de 
    '''     <seealso cref="System.IFormatProvider">IFormatProvider</seealso>  cuyo método 
    '''     <seealso cref="System.IFormatProvider.GetFormat ">IFormatProvider.GetFormat </seealso>
    '''     devuelve un objeto 
    '''     <seealso cref="System.Globalization.NumberFormatInfo">NumberFormatInfo</seealso>
    '''     Normalmente, <paramref name="provider">provider</paramref> 
    '''     es un objeto  
    '''     <seealso cref="System.Globalization.NumberFormatInfo">NumberFormatInfo</seealso> o 
    '''     <seealso cref="System.Globalization.CultureInfo">CultureInfo</seealso>. 
    '''     El parámetro <paramref name="provider">provider</paramref> 
    '''     proporciona información específica de la referencia cultural utilizada en 
    '''     la aplicación de formato. 
    '''     Si <paramref name="provider">provider</paramref>  
    '''     es null(Nothing en Visual Basic), se utiliza la referencia cultural 
    '''     actual del subproceso.
    ''' </para>
    '''</remarks>
    Public Overloads Function ToString( _
                    ByVal format As String, _
                    ByVal provider As System.IFormatProvider) As String _
                    Implements System.IFormattable.ToString
        ' Return Me.ValorSinRedondear.ToString(format, provider)
        If String.IsNullOrEmpty(format) = True Then
            'http://msdn.microsoft.com/es-es/library/b1csw23d.aspx
            format = "{0:C}"
        End If
        Return String.Format(provider, format, Me.ValorSinRedondear)
    End Function



#End Region


#Region " Implements IConvertible"

    ''' ---------------------------------------------------------
    ''' <summary>
    '''   Devuelve el objeto TypeCode para el tipo de valor Dinero.
    ''' </summary>
    ''' <returns>Tipo: 
    '''       <seealso cref="System.TypeCode">System.TypeCode</seealso> 
    '''       Constante enumerada <seealso cref="System.TypeCode.Object">
    '''       System.TypeCode.Object</seealso>
    ''' </returns>
    ''' <remarks></remarks>
    Public Function GetTypeCode() As System.TypeCode _
        Implements System.IConvertible.GetTypeCode
        Return TypeCode.Object
    End Function


    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.Boolean">Boolean</see>].
    ''' </summary>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns> 
    '''   <para>Un valor lógico que indica lo siguiente:</para>
    '''   <para>Si existe alguna cantidad de dinero devuelve el valor TRUE, </para>
    '''   <para>En caso contrario, si la cantidad es cero (0) devuelve el valor FALSE</para>
    ''' </returns>
    Public Overloads Function ToBoolean(ByVal provider As System.IFormatProvider) As Boolean _
           Implements System.IConvertible.ToBoolean
        Dim salida As Boolean = False
        If Me.Valor <> 0 Then
            salida = True
        End If
        Return salida
    End Function


    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> recibido por parametro, convertido un dato de tipo 
    '''   [<see cref="System.Boolean">Boolean</see>].
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns> 
    '''   <para>Un valor lógico que indica lo siguiente:</para>
    '''   <para>Si existe alguna cantidad de dinero devuelve el valor TRUE, </para>
    '''   <para>En caso contrario, si la cantidad es cero (0) devuelve el valor FALSE</para>
    ''' </returns>
    Public Overloads Shared Function ToBoolean(ByVal d As Dinero) As Boolean
        Dim salida As Boolean = False
        If d.Valor <> 0 Then
            salida = True
        End If
        Return salida
    End Function


    ''' <summary>
    '''   [NO SOPORTADO] el valor 
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.Byte">Byte</see>].
    '''   Esta conversion no es posible
    ''' </summary>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns>Un error de tipo <see cref="System.InvalidCastException">InvalidCastException</see>
    ''' </returns>
    Public Overloads Function ToByte(ByVal provider As System.IFormatProvider) As Byte _
       Implements System.IConvertible.ToByte
        Throw New InvalidCastException("Conversión no permitida")
    End Function


    ''' <summary>
    '''   [NO SOPORTADO] el valor 
    '''   El valor <see cref="Dinero">Dinero</see> recibido por parametro, convertido un dato de tipo 
    '''   [<see cref="System.Byte">Byte</see>].
    '''   Esta conversion no es posible
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns>Un error de tipo <see cref="System.InvalidCastException">InvalidCastException</see>
    ''' </returns>
    Public Overloads Shared Function ToByte(ByVal d As Dinero) As Byte
        Throw New InvalidCastException("Conversión no permitida")
    End Function


    ''' <summary>
    '''   [NO SOPORTADO] el valor 
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.Char">Char</see>].
    '''   Esta conversion no es posible
    ''' </summary>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns>Un error de tipo <see cref="System.InvalidCastException">InvalidCastException</see>
    ''' </returns>
    Public Overloads Function ToChar(ByVal provider As System.IFormatProvider) As Char _
        Implements System.IConvertible.ToChar
        Throw New InvalidCastException("Conversión no permitida")
    End Function


    ''' <summary>
    '''   [NO SOPORTADO] el valor 
    '''   El valor <see cref="Dinero">Dinero</see> recibido por parametro, convertido un dato de tipo 
    '''   [<see cref="System.Char">Char</see>].
    '''   Esta conversion no es posible
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns>Un error de tipo <see cref="System.InvalidCastException">InvalidCastException</see>
    ''' </returns>
    Public Overloads Shared Function ToChar(ByVal d As Dinero) As Char
        Throw New InvalidCastException("Conversión no permitida")
    End Function

    ''' <summary>
    '''   [NO SOPORTADO] el valor 
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.Date">Date</see>].
    '''   Esta conversion no es posible
    ''' </summary>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns>Un error de tipo <see cref="System.InvalidCastException">InvalidCastException</see>
    ''' </returns>
    Public Overloads Function ToDateTime(ByVal provider As System.IFormatProvider) As Date _
        Implements System.IConvertible.ToDateTime
        Throw New InvalidCastException("Conversión no permitida")
    End Function


    ''' <summary>
    '''   [NO SOPORTADO] el valor 
    '''   El valor <see cref="Dinero">Dinero</see> recibido por parametro, convertido un dato de tipo 
    '''   [<see cref="System.Date">Date</see>].
    '''   Esta conversion no es posible
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns>Un error de tipo <see cref="System.InvalidCastException">InvalidCastException</see>
    ''' </returns>
    Public Overloads Shared Function ToDateTime(ByVal d As Dinero) As Date
        Throw New InvalidCastException("Conversión no permitida")
    End Function


    '--------------------------------------------------------
    ' ------ Cuidado CUIDADO Cuidado -------
    '--------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.Decimal">Decimal</see>], 
    '''   con todos sus decimales (Normalmente 9 posiciones decimales)
    ''' </summary>
    ''' <returns>Un Valor de tipo <see cref="System.Decimal">Decimal</see> 
    '''  normalmente con nueve (9) posiciones decimales
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.Decimal.MinValue"> Decimal.MinValue</see> o mayor que 
    '''    <see cref="System.Decimal.MaxValue"> Decimal.MaxValue</see>.
    ''' </exception>
    Public Overloads Function ToDecimal() As Decimal

        '----------------------------------------------------
        'Bug - 16/08/2008
        'El código Inicial
        'Return CType((Me.ValorParteEntera + _
        '             (Me.ValorParteDecimal / _UNO_SEGUIDO_NUEVE_CEROS)), _
        '             Decimal)
        'tiene el problema de que pierde información con números grandes
        'ejemplo 1234567890123,123456789 -> devuelve -> 1234567890123,12

        'por esa razón esta sustituido por este otro que devuelve 
        'todos los números sin que se pierda ni redondee ninguno
        '----------------------------------------------------

        Dim numeroDecimal As Decimal = Decimal.Zero
        Dim parteEntera As Decimal = Decimal.Zero
        Dim parteDecimal As Decimal = Decimal.Zero

        parteEntera = CType(Me.ValorParteEntera, Decimal)
        parteDecimal = Me.ValorParteDecimal

        If parteEntera < 0 Then
            parteDecimal = -parteDecimal
        End If

        numeroDecimal = parteEntera + parteDecimal
        Return numeroDecimal
    End Function

    '--------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> recibido por parametro, convertido un dato de tipo 
    '''   [<see cref="System.Decimal">Decimal</see>], 
    '''   con todos sus decimales (Normalmente 9 posiciones decimales)
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.Decimal">Decimal</see> 
    '''  normalmente con nueve (9) posiciones decimales
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.Decimal.MinValue"> Decimal.MinValue</see> o mayor que 
    '''    <see cref="System.Decimal.MaxValue"> Decimal.MaxValue</see>.
    ''' </exception>
    Public Overloads Shared Function ToDecimal(ByVal d As Dinero) As Decimal
        Return d.ToDecimal()
    End Function

    '--------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.Decimal">Decimal</see>], 
    '''   con todos sus decimales (Normalmente 9 posiciones decimales)
    ''' </summary>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.Decimal">Decimal</see> 
    '''  normalmente con nueve (9) posiciones decimales
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.Decimal.MinValue"> Decimal.MinValue</see> o mayor que 
    '''    <see cref="System.Decimal.MaxValue"> Decimal.MaxValue</see>.
    ''' </exception>
    Public Overloads Function ToDecimal(ByVal provider As System.IFormatProvider) As Decimal _
                  Implements System.IConvertible.ToDecimal
        Return CType(Me.ValorSinRedondear, IConvertible).ToDecimal(provider)
    End Function


    '--------------------------------------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.Double">Double</see>], 
    ''' </summary>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.Double">Double</see> 
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.Double.MinValue"> Double.MinValue</see> o mayor que 
    '''    <see cref="System.Double.MaxValue"> Double.MaxValue</see>.
    ''' </exception>
    Public Overloads Function ToDouble(ByVal provider As System.IFormatProvider) As Double _
               Implements System.IConvertible.ToDouble
        Return CType(Me.ValorSinRedondear, IConvertible).ToDouble(provider)
    End Function


    '--------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> recibido por parametro, convertido un dato de tipo 
    '''   [<see cref="System.Double">Double</see>], 
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.Double">Double</see> </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.Double.MinValue"> Double.MinValue</see> o mayor que 
    '''    <see cref="System.Double.MaxValue"> Double.MaxValue</see>.
    ''' </exception>
    Public Overloads Shared Function ToDouble(ByVal d As Dinero) As Double
        Return Decimal.ToDouble(d.ToDecimal)
    End Function


    '--------------------------------------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.Short">Short</see>], 
    ''' </summary>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.Short">Short</see> 
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.Short.MinValue"> Short.MinValue</see> o mayor que 
    '''    <see cref="System.Short.MaxValue"> Short.MaxValue</see>.
    ''' </exception>
    Public Overloads Function ToInt16(ByVal provider As System.IFormatProvider) As Short _
                 Implements System.IConvertible.ToInt16
        Return CType(Me.ValorSinRedondear, IConvertible).ToInt16(provider)
    End Function


    '--------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> recibido por parametro, convertido un dato de tipo 
    '''   [<see cref="System.Short">Short</see>], 
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.Short">Short</see> </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.Short.MinValue"> Short.MinValue</see> o mayor que 
    '''    <see cref="System.Short.MaxValue"> Short.MaxValue</see>.
    ''' </exception>
    Public Overloads Shared Function ToInt16(ByVal d As Dinero) As Short
        Return Decimal.ToInt16(d.ToDecimal)
    End Function


    '--------------------------------------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.Integer">Integer</see>], 
    ''' </summary>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.Short">Integer</see> 
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.Integer.MinValue"> Integer.MinValue</see> o mayor que 
    '''    <see cref="System.Integer.MaxValue"> Integer.MaxValue</see>.
    ''' </exception>
    Public Overloads Function ToInt32(ByVal provider As System.IFormatProvider) As Integer _
          Implements System.IConvertible.ToInt32
        Return CType(Me.ValorSinRedondear, IConvertible).ToInt32(provider)
    End Function


    '--------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> recibido por parametro, convertido un dato de tipo 
    '''   [<see cref="System.Integer">Integer</see>], 
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.Integer">Integer</see> </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.Integer.MinValue"> Integer.MinValue</see> o mayor que 
    '''    <see cref="System.Integer.MaxValue"> Integer.MaxValue</see>.
    ''' </exception>
    Public Overloads Shared Function ToInt32(ByVal d As Dinero) As Integer
        Return Decimal.ToInt32(d.ToDecimal)
    End Function


    '--------------------------------------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.Long">Long</see>], 
    ''' </summary>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.Long">Long</see> 
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.Long.MinValue"> Long.MinValue</see> o mayor que 
    '''    <see cref="System.Long.MaxValue"> Long.MaxValue</see>.
    ''' </exception>
    Public Overloads Function ToInt64(ByVal provider As System.IFormatProvider) As Long _
            Implements System.IConvertible.ToInt64
        Return CType(Me.ValorSinRedondear, IConvertible).ToInt64(provider)
    End Function


    '--------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> recibido por parametro, convertido un dato de tipo 
    '''   [<see cref="System.Long">Long</see>], 
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.Long">Long</see> </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.Long.MinValue"> Long.MinValue</see> o mayor que 
    '''    <see cref="System.Long.MaxValue"> Long.MaxValue</see>.
    ''' </exception>
    Public Overloads Shared Function ToInt64(ByVal d As Dinero) As Long
        Return Decimal.ToInt64(d.ToDecimal)
    End Function


    '--------------------------
    ''' <summary>
    '''  Convierte el valor <see cref="Dinero">Dinero</see> recibido por parametro,  
    '''  en el valor de moneda de automatización OLE equivalente, que está contenido
    '''  en un entero de 64 bits con signo [<see cref="System.Long">Long</see>].
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns>Entero de <see cref="System.Long">64 bits con signo [Long]</see> 
    '''  que contiene el equivalente de automatización OLE de value. 
    ''' </returns>
    ''' <remarks><seealso cref="System.Decimal.ToOACurrency"></seealso></remarks>
    Public Shared Function ToOACurrency(ByVal d As Dinero) As Long
        Return Decimal.ToOACurrency(d.ToDecimal)
    End Function


    ' El valor de esta funcion no es confome a CLS
    '--------------------------------------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.SByte">SByte</see>]. El valor de esta función no es confome a CLS
    ''' </summary>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.SByte">SByte</see> 
    '''  El valor de esta función no es confome a CLS
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.SByte.MinValue"> SByte.MinValue</see> o mayor que 
    '''    <see cref="System.SByte.MaxValue"> SByte.MaxValue</see>.
    ''' </exception>
    <CLSCompliantAttribute(False)> _
      Public Overloads Function ToSByte(ByVal provider As System.IFormatProvider) As SByte _
         Implements System.IConvertible.ToSByte
        Return CType(Me.ValorSinRedondear, IConvertible).ToSByte(provider)
    End Function


    ' El valor de esta funcion no es confome a CLS
    '--------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> recibido por parametro, convertido un dato de tipo 
    '''   [<see cref="System.SByte">SByte</see>].  El valor de esta función no es confome a CLS
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.SByte">SByte</see> 
    '''  El valor de esta función no es confome a CLS
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.SByte.MinValue"> SByte.MinValue</see> o mayor que 
    '''    <see cref="System.SByte.MaxValue"> SByte.MaxValue</see>.
    ''' </exception>
    <CLSCompliantAttribute(False)> _
     Public Overloads Shared Function ToSByte(ByVal d As Dinero) As SByte
        Return Decimal.ToSByte(d.ToDecimal)
    End Function


    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.Single">Single</see>], 
    ''' </summary>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.Single">Single</see> 
    ''' </returns>
    Public Overloads Function ToSingle(ByVal provider As System.IFormatProvider) As Single _
       Implements System.IConvertible.ToSingle
        Return CType(Me.ValorSinRedondear, IConvertible).ToSingle(provider)
    End Function


    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> recibido por parametro, convertido un dato de tipo 
    '''   [<see cref="System.Single">Single</see>], 
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.Single">Single</see> </returns>
    Public Overloads Shared Function ToSingle(ByVal d As Dinero) As Single
        Return Decimal.ToSingle(d.ToDecimal)
    End Function


    '--------------------------------------------------------
    ' ------ Cuidado CUIDADO Cuidado -------
    '--------------------------------------------------------
    '
    '--------------------------------------------------
    ' ToString () 
    ' No tiene sentido una Function ToString(dinero as string)
    '--------------------------------------------------
    ' Estan en      Implements IFormattable
    '               Implements IConvertible
    '--------------------------------------------------
    'Public Overloads Overrides Function ToString() As String
    'Public Overloads Function ToString(provider) As String (Implements IConvertible)
    'Public Overloads Function ToString(String) As String
    'Public Overloads Function ToString(String, provider) As String (Implements IFormattable)
    '--------------------------------------------------


    ''' <summary>
    ''' Convierte el valor numérico de esta instancia en la representación de 
    ''' cadena equivalente utilizando la información de formato específica de 
    ''' la referencia cultural especificada.
    ''' </summary>
    ''' <param name="provider">
    ''' Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    ''' que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns>
    ''' Representación de cadena del valor de esta instancia tal y como especifica 
    ''' <paramref name="provider">provider</paramref>.
    ''' </returns>
    ''' <remarks>
    ''' <para>
    ''' Al valor devuelto se le aplica formato con el especificador de 
    ''' formato numérico general ("G").
    ''' </para>
    '''<para>.NET Framework proporciona una amplia compatibilidad de formato, que
    '''  se describe con más detalle en los siguientes temas referentes al formato:</para>
    '''<para>Para obtener más información sobre los especificadores de formato numérico, vea 
    '''     <a href="http://msdn.microsoft.com/es-es/library/dwhawy9k.aspx"> 
    '''      Cadenas con formato numérico estándar</a> y  
    '''     <a href="http://msdn.microsoft.com/es-es/library/0c899ak8.aspx">
    '''     Cadenas con formato numérico personalizado.</a> 
    ''' </para>
    '''<para>Para obtener más información sobre la forma de aplicar formato, vea 
    '''  <a href="http://msdn.microsoft.com/es-es/library/fbxft59x.aspx">
    '''           Aplicar formato a tipos</a> y 
    '''  <a href="http://msdn.microsoft.com/es-es/library/26etazsy.aspx">
    '''          Información general sobre formatos</a>. 
    ''' </para>
    '''<para>
    '''     El parámetro <paramref name="provider">provider</paramref> 
    '''     es una implementación de 
    '''     <seealso cref="System.IFormatProvider">IFormatProvider</seealso>  cuyo método 
    '''     <seealso cref="System.IFormatProvider.GetFormat ">IFormatProvider.GetFormat </seealso>
    '''     devuelve un objeto 
    '''     <seealso cref="System.Globalization.NumberFormatInfo">NumberFormatInfo</seealso>
    '''     Normalmente, <paramref name="provider">provider</paramref> 
    '''     es un objeto  
    '''     <seealso cref="System.Globalization.NumberFormatInfo">NumberFormatInfo</seealso> o 
    '''     <seealso cref="System.Globalization.CultureInfo">CultureInfo</seealso>. 
    '''     El parámetro <paramref name="provider">provider</paramref> 
    '''     proporciona información específica de la referencia cultural utilizada en 
    '''     la aplicación de formato. 
    '''     Si <paramref name="provider">provider</paramref>  
    '''     es null(Nothing en Visual Basic), se utiliza la referencia cultural 
    '''     actual del subproceso.
    ''' </para>
    ''' 
    '''<para>
    '''     Para convertir un valor <seealso cref="Dinero">Dinero</seealso>  
    '''     en su representación de <seealso cref="System.String">cadena</seealso>
    '''     utilizando una referencia cultural especificada y una cadena de 
    '''     formato concreta, llame al método <c>Dinero.ToString(String,IFormatProvide)</c>. 
    ''' </para>
    '''</remarks>
    Public Overloads Function ToString(ByVal provider As System.IFormatProvider) As String _
        Implements System.IConvertible.ToString
        Return CType(Me.ValorSinRedondear, IConvertible).ToString(provider)
        ' Return String.Format(provider, "{0}", Me.Valor)
    End Function


    ''' <summary>
    '''  Esta API es compatible con la infraestructura de .NET Framework y no 
    '''  está diseñada para utilizarse directamente desde el código.
    ''' </summary>
    ''' <param name="conversionType">Tipo <seealso cref="System.Type">System.Type</seealso> 
    '''   Tipo al que se va a convertir el valor de esta instancia de Dinero</param>
    ''' <param name="provider">Tipo: 
    '''   <seealso cref="System.IFormatProvider">System.IFormatProvider</seealso>
    '''   Implementación de IFormatProvider que proporciona información específica 
    '''   de la referencia cultural sobre el formato del valor devuelto.</param>
    ''' <returns>Tipo: <seealso cref="System.Object">System.Object</seealso>
    '''         Valor de la instancia actual, convertido en conversionType.</returns>
    ''' <remarks>
    '''    Este miembro es una implementación explícita de un miembro de interfaz. 
    '''    Sólo se puede usar cuando la instancia de 
    '''    <seealso cref="Dinero">Dinero</seealso> se convierte en una interfaz
    '''    <seealso cref="System.IConvertible">System.IConvertible</seealso>. 
    '''    Como alternativa, se recomienda llamar al método 
    '''    static (Shared en Visual Basic) 
    '''    <seealso cref="System.Convert.ChangeType">System.Convert.ChangeType</seealso>
    ''' </remarks>
    Private Function ToType( _
                        ByVal conversionType As System.Type, _
                        ByVal provider As System.IFormatProvider) _
                        As Object _
                        Implements System.IConvertible.ToType
        ' Forma Uno
        'Return Convert.ChangeType(Me.ValorSinRedondear(provider), conversionType, provider)
        ' Forma Dos
        Return CType(Me.ValorSinRedondear, IConvertible).ToType(conversionType, provider)
    End Function


    ' El valor de esta funcion no es confome a CLS
    '--------------------------------------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.UShort">UShort</see>]. El valor de esta función no es confome a CLS
    ''' </summary>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.UShort">UShort</see> 
    '''  El valor de esta función no es confome a CLS
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.UShort.MinValue"> UShort.MinValue</see> o mayor que 
    '''    <see cref="System.UShort.MaxValue"> UShort.MaxValue</see>.
    ''' </exception>
    <CLSCompliantAttribute(False)> _
       Public Function ToUInt16(ByVal provider As System.IFormatProvider) As UShort _
       Implements System.IConvertible.ToUInt16
        Return CType(Me.ValorSinRedondear, IConvertible).ToUInt16(provider)
    End Function


    ' El valor de esta funcion no es confome a CLS
    '--------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> recibido por parametro, convertido un dato de tipo 
    '''   [<see cref="System.UShort">UShort</see>].  El valor de esta función no es confome a CLS
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.UShort">UShort</see> 
    '''  El valor de esta función no es confome a CLS
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.UShort.MinValue"> UShort.MinValue</see> o mayor que 
    '''    <see cref="System.UShort.MaxValue"> UShort.MaxValue</see>.
    ''' </exception>
    <CLSCompliantAttribute(False)> _
           Public Shared Function ToUInt16(ByVal d As Dinero) As UShort
        Return Decimal.ToUInt16(d.ToDecimal)
    End Function


    ' El valor de esta funcion no es confome a CLS
    '--------------------------------------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.UInteger">UInteger</see>]. El valor de esta función no es confome a CLS
    ''' </summary>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.UInteger">UInteger</see> 
    '''  El valor de esta función no es confome a CLS
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.UInteger.MinValue"> UInteger.MinValue</see> o mayor que 
    '''    <see cref="System.UInteger.MaxValue"> UInteger.MaxValue</see>.
    ''' </exception>
    <CLSCompliantAttribute(False)> _
      Public Function ToUInt32(ByVal provider As System.IFormatProvider) As UInteger _
      Implements System.IConvertible.ToUInt32
        Return CType(Me.ValorSinRedondear, IConvertible).ToUInt32(provider)
    End Function


    ' El valor de esta funcion no es confome a CLS
    '--------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> recibido por parametro, convertido un dato de tipo 
    '''   [<see cref="System.UInteger">UInteger</see>].  El valor de esta función no es confome a CLS
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.UInteger">UInteger</see> 
    '''  El valor de esta función no es confome a CLS
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.UInteger.MinValue"> UInteger.MinValue</see> o mayor que 
    '''    <see cref="System.UInteger.MaxValue"> UInteger.MaxValue</see>.
    ''' </exception>
    <CLSCompliantAttribute(False)> _
    Public Shared Function ToUInt32(ByVal d As Dinero) As UInteger
        Return Decimal.ToUInt32(d.ToDecimal)
    End Function


    ' El valor de esta funcion no es confome a CLS
    '--------------------------------------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> de esta instancia convertido un dato de tipo 
    '''   [<see cref="System.ULong">ULong</see>]. El valor de esta función no es confome a CLS
    ''' </summary>
    ''' <param name="provider">
    '''    Tipo: <seealso cref="System.IFormatProvider">IFormatProvider</seealso> 
    '''    que aporta información de formato específica de la referencia cultural. 
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.ULong">ULong</see> 
    '''  El valor de esta función no es confome a CLS
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.ULong.MinValue"> ULong.MinValue</see> o mayor que 
    '''    <see cref="System.ULong.MaxValue"> ULong.MaxValue</see>.
    ''' </exception>
    <CLSCompliantAttribute(False)> _
        Public Function ToUInt64(ByVal provider As System.IFormatProvider) As ULong _
        Implements System.IConvertible.ToUInt64
        Return CType(Me.ValorSinRedondear, IConvertible).ToUInt64(provider)
    End Function


    ' El valor de esta funcion no es confome a CLS
    '--------------------------
    ''' <summary>
    '''   El valor <see cref="Dinero">Dinero</see> recibido por parametro, convertido un dato de tipo 
    '''   [<see cref="System.ULong">ULong</see>].  El valor de esta función no es confome a CLS
    ''' </summary>
    ''' <param name="d"> Tipo: <seealso cref="Dinero">Dinero</seealso> 
    '''   La cantidad de dinero que se va a convertir
    ''' </param>
    ''' <returns>Un Valor de tipo <see cref="System.ULong">ULong</see> 
    '''  El valor de esta función no es confome a CLS
    ''' </returns>
    ''' <exception cref="OverflowException">OverflowException - 
    '''    El valor de Dinero es menor que 
    '''    <see cref="System.ULong.MinValue"> ULong.MinValue</see> o mayor que 
    '''    <see cref="System.ULong.MaxValue"> ULong.MaxValue</see>.
    ''' </exception>
    <CLSCompliantAttribute(False)> _
         Public Shared Function ToUInt64(ByVal d As Dinero) As ULong
        Return Decimal.ToUInt64(d.ToDecimal)
    End Function


#End Region


#End Region


#Region "Dispose"

    '--------------------------------------------------------------------------
    ' Atencion
    ' Hay veces que el IDE mete directamente una definicion vacia del moetodo 
    ' dispose al final del codigo, por esata razon hay veces que aparece un error 
    ' en la definion de nuenstra funion dispose.
    ' la forma de resolverlo es borrar la funcion Dispose que ha escrito el IDE 
    ' normalmente al final del codigo de la clase, justo antes del End Class
    '--------------------------------------------------------------------------


    '---------------------------------------------------------------
    ' Referencia bibliográfica
    '---------------------------------------------------------------
    ' MSDN
    ' Biblioteca de clases de .NET Framework   
    ' GC.SuppressFinalize (Método)  [Visual Basic]
    ' IDisposable.Dispose (Método)  [Visual Basic] 
    '---------------------------------------------------------------


    '---------------------------------------------------------------
    ''' <summary>
    ''' Semáforo que indica se el método Dispose ha sido llamado o no 
    ''' </summary>
    Private _DISPOSE_Ejecutado As Boolean
    ' Toma el valor por defecto, no hace falta inicializar = False


    ''' -------------------------------------------------
    ''' <summary>
    '''     El metodo Dispose
    ''' </summary>
    ''' <remarks>
    '''     <para>Referencia bibliografica</para>
    '''     <para>Biblioteca MSDN</para>
    '''     <para>Biblioteca de clases de .NET Framework   </para>
    '''     <para>GC.SuppressFinalize (Metodo)  [Visual Basic]</para>
    '''     <para>IDisposable.Dispose (Metodo)  [Visual Basic] </para>
    '''  <seealso cref = "GC.SuppressFinalize"> GC.SuppressFinalize </seealso>
    '''  <seealso cref = "IDisposable.Dispose"> IDisposable.Dispose </seealso>
    ''' </remarks>
    ''' -------------------------------------------------
    Public Overloads Sub Dispose() Implements System.IDisposable.Dispose

        Dispose(True)
        ' This object will be cleaned up by the Dispose method.
        ' Therefore, you should call GC.SupressFinalize to
        ' take this object off the finalization queue 
        ' and prevent finalization code for this object
        ' from executing a second time.
        GC.SuppressFinalize(Me)

    End Sub


    '---------------------------------------------------------------
    ' El metodo Dispose QUE HACE EL TRABAJO
    '---------------------------------------------------------------
    ' Dispose(quienLLama) SE EJECUTA EN DOS ESCENARIOS
    ' SI quienLLama equals true, el metodo se ha llamado diretamente
    ' o indirectamente por el usuario del codigo.
    ' los recursos administrados y no administrados pueden ser liberados(disposed.)
    ' 
    ' Si quienLLama equals false, el metodo ha sido llamado por el runtime
    ' mediante el metodo [Finalize] y no tiene que hacer referenia a ningun objeto
    ' Solo los recursos NO administrados pueden ser liberados (disposed.)
    '---------------------------------------------------------------
    ''' -------------------------------------------------
    ''' <summary>
    '''     El metodo Dispose
    ''' </summary>
    ''' -------------------------------------------------
    Private Overloads Sub Dispose(ByVal quienLLama As Boolean)

        ' --------------------------------------------------------------------
        ' Check para ver si [Dispose] ya ha sido llamado.
        If _DISPOSE_Ejecutado = False Then
            Try
                ' Si quienLLama equals true, Liberar todos los recursos
                ' administrados y no administrados
                If quienLLama = True Then
                    '----------------------------------------------------------
                    ' Dispose de los recursos administrados
                    ' normalmente componentes como PageSetupDialog 
                    ' componente.Dispose
                    '----------------------------------------------------------
                    ' Liberar los recursos propios 
                    '----------------------------------------------------------
                    '  ---1--- Cerrar archivos abiertos  
                    '          (No hay ninguno)
                    '  -------------------------------------
                    '  ---2--- Objetos Contenios a nothing
                    '          [Objeto]=nothing
                    If Not (Me._CultureInfo Is Nothing) Then
                        ' objeto a null
                        _CultureInfo = Nothing
                    End If
                    '  -------------------------------------
                    '  ---3--- Variables de la clase a nothing
                    '          [Variable]=nothing
                    '  -
                    Me._ParteEntera = Nothing
                    Me._ParteDecimal = Nothing

                End If
                '----------------------------------------------------------
                ' Llamar a los mentodos apropiados para liberar la menoria 
                ' ocupada por los recursos no administrados
                ' Por ejemplo un puntero a un recurso externo
                ' Son bastante raros
                ' Si quienLLama is false, 
                ' Solo el codigo siguiente se ejecutara
                '----------------------------------------------------------
                '---   Aqui hay que escribir el código limpieza ---
                '---   de los recursos no administrados         ---
                '---         Son bastante raros                 ---
                '----------------------------------------------------------
                '

            Finally
                ' OJO -------->>> Solo en clases derivadas <<< ---------
                ' llamar al metodo Dispose de la clase base
                'MyBase.Dispose()
                ' o bien si es una clase mia
                'MyBase.Dispose(true)
            End Try

        End If
        ' cerrar la puerta
        _DISPOSE_Ejecutado = True
    End Sub


    '---------------------------------------------------------------
    '---------------------------------------------------------------
    '---------------------------------------------------------------
    ' FINALIZE
    '---------------------------------------------------------------
    '---------------------------------------------------------------
    ' Este metodo debe trabajar SOLO si el metodo Dispose
    ' no ha sido llamado
    ' Da a la clase base la oportunidad de finalizar
    '
    ' ----------------->>>> ¡¡¡ ATENCION !!! <<<<-----------------
    'No Implementar este metodo en clases derivadas de esta clase
    '---------------------------------------------------------------

    ''' -------------------------------------------------
    ''' <summary>
    '''     Permite que un objeto Object intente liberar recursos y realizar 
    '''     otras operaciones de limpieza antes de que el objeto Object sea reclamado 
    '''     por el recolector de elementos no utilizados.
    ''' </summary>
    ''' <remarks>
    ''' http://msdn.microsoft.com/es-es/library/system.object.finalize.aspx
    ''' </remarks>
    ''' -------------------------------------------------
    Protected Overrides Sub Finalize()
        ' NO VOLVER A INCLUIR el codigo de limpieza del metodo [Dispose]
        ' LLamar a [Dispose(false) es optimo en terminos eficacia y mantenimiento 
        Dispose(False)
    End Sub

#End Region


End Structure

